
#include <mpi.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifndef _EXTERN_C_
#ifdef __cplusplus
#define _EXTERN_C_ extern "C"
#else /* __cplusplus */
#define _EXTERN_C_
#endif /* __cplusplus */
#endif /* _EXTERN_C_ */




#if (defined(PIC) || defined(__PIC__))
/* For shared libraries, declare these weak and figure out which one was linked
   based on which init wrapper was called.  See mpi_init wrappers.  */
#pragma weak pmpi_init
#pragma weak PMPI_INIT
#pragma weak pmpi_init_
#pragma weak pmpi_init__
#pragma weak pmpi_init_thread
#pragma weak PMPI_INIT_THREAD
#pragma weak pmpi_init_thread_
#pragma weak pmpi_init_thread__
#endif /* PIC */

_EXTERN_C_ void pmpi_init(MPI_Fint *ierr);
_EXTERN_C_ void PMPI_INIT(MPI_Fint *ierr);
_EXTERN_C_ void pmpi_init_(MPI_Fint *ierr);
_EXTERN_C_ void pmpi_init__(MPI_Fint *ierr);
_EXTERN_C_ void pmpi_init_thread(MPI_Fint *required, MPI_Fint *provided, MPI_Fint *ierr);
_EXTERN_C_ void PMPI_INIT_THREAD(MPI_Fint *required, MPI_Fint *provided, MPI_Fint *ierr);
_EXTERN_C_ void pmpi_init_thread_(MPI_Fint *required, MPI_Fint *provided, MPI_Fint *ierr);
_EXTERN_C_ void pmpi_init_thread__(MPI_Fint *required, MPI_Fint *provided, MPI_Fint *ierr);


/* Macros to enable and disable compiler warnings for deprecated functions.
 * These macros will be used to silent warnings about deprecated MPI functions,
 * as these should be wrapped, even if they are deprecated.
 *
 * Note: The macros support GCC and clang compilers only. For other compilers
 *       just add similar macros for your compiler.
 */
#ifndef WRAP_MPI_CALL_PREFIX
#if defined(__clang__)
#define WRAP_MPI_CALL_PREFIX        \
  _Pragma("clang diagnostic push"); \
  _Pragma("clang diagnostic ignored \"-Wdeprecated-declarations\"");
#define WRAP_MPI_CALL_POSTFIX _Pragma("clang diagnostic pop");
#elif defined(__INTEL_COMPILER)
#define WRAP_MPI_CALL_PREFIX \
  _Pragma("warning push");   \
  _Pragma("warning(disable:1786)");
#define WRAP_MPI_CALL_POSTFIX _Pragma("warning pop")
#elif (defined(__GNUC__) && \
  ((__GNUC__ == 4 && __GNUC_MINOR__ >= 6) || __GNUC__ > 4))
#define  WRAP_MPI_CALL_PREFIX     \
  _Pragma("GCC diagnostic push"); \
  _Pragma("GCC diagnostic ignored \"-Wdeprecated-declarations\"");
#define WRAP_MPI_CALL_POSTFIX _Pragma("GCC diagnostic pop");
#else
#define WRAP_MPI_CALL_PREFIX
#define WRAP_MPI_CALL_POSTFIX
#endif
#endif

/* This file is part of P^nMPI.
 *
 * Copyright (c)
 *  2008-2019 Lawrence Livermore National Laboratories, United States of America
 *  2011-2016 ZIH, Technische Universitaet Dresden, Federal Republic of Germany
 *  2013-2019 RWTH Aachen University, Federal Republic of Germany
 *
 *
 * P^nMPI is free software; you can redistribute it and/or modify it under the
 * terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation version 2.1 dated February 1999.
 *
 * P^nMPI is distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR
 * A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with P^nMPI; if not, write to the
 *
 *   Free Software Foundation, Inc.
 *   51 Franklin St, Fifth Floor
 *   Boston, MA 02110, USA
 *
 *
 * Written by Martin Schulz, schulzm@llnl.gov.
 *
 * LLNL-CODE-402774
 */

typedef int (*pnmpi_int_MPI_Pcontrol_t)(int level, ... );

void mpi_init_(int *ierr);
double mpi_wtick_(void);
double mpi_wtime_(void);


int NQJ_Abort(MPI_Comm comm, int errorcode);

int NQJ_Accumulate(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win);

int NQJ_Add_error_class(int *errorclass);

int NQJ_Add_error_code(int errorclass, int *errorcode);

int NQJ_Add_error_string(int errorcode, const char *string);

int NQJ_Address(void *location, MPI_Aint *address);

int NQJ_Allgather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm);

int NQJ_Allgatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm);

int NQJ_Alloc_mem(MPI_Aint size, MPI_Info info, void *baseptr);

int NQJ_Allreduce(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm);

int NQJ_Alltoall(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm);

int NQJ_Alltoallv(const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm);

int NQJ_Alltoallw(const void *sendbuf, const int sendcounts[], const int sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const int rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm);

int NQJ_Attr_delete(MPI_Comm comm, int keyval);

int NQJ_Attr_get(MPI_Comm comm, int keyval, void *attribute_val, int *flag);

int NQJ_Attr_put(MPI_Comm comm, int keyval, void *attribute_val);

int NQJ_Barrier(MPI_Comm comm);

int NQJ_Bcast(void *buffer, int count, MPI_Datatype datatype, int root, MPI_Comm comm);

int NQJ_Bsend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm);

int NQJ_Bsend_init(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);

int NQJ_Buffer_attach(void *buffer, int size);

int NQJ_Buffer_detach(void *buffer, int *size);

int NQJ_Cancel(MPI_Request *request);

int NQJ_Cart_coords(MPI_Comm comm, int rank, int maxdims, int coords[]);

int NQJ_Cart_create(MPI_Comm old_comm, int ndims, const int dims[], const int periods[], int reorder, MPI_Comm *comm_cart);

int NQJ_Cart_get(MPI_Comm comm, int maxdims, int dims[], int periods[], int coords[]);

int NQJ_Cart_map(MPI_Comm comm, int ndims, const int dims[], const int periods[], int *newrank);

int NQJ_Cart_rank(MPI_Comm comm, const int coords[], int *rank);

int NQJ_Cart_shift(MPI_Comm comm, int direction, int disp, int *rank_source, int *rank_dest);

int NQJ_Cart_sub(MPI_Comm comm, const int remain_dims[], MPI_Comm *new_comm);

int NQJ_Cartdim_get(MPI_Comm comm, int *ndims);

int NQJ_Close_port(const char *port_name);

int NQJ_Comm_accept(const char *port_name, MPI_Info info, int root, MPI_Comm comm, MPI_Comm *newcomm);

int NQJ_Comm_call_errhandler(MPI_Comm comm, int errorcode);

int NQJ_Comm_compare(MPI_Comm comm1, MPI_Comm comm2, int *result);

int NQJ_Comm_connect(const char *port_name, MPI_Info info, int root, MPI_Comm comm, MPI_Comm *newcomm);

int NQJ_Comm_create(MPI_Comm comm, MPI_Group group, MPI_Comm *newcomm);

int NQJ_Comm_create_errhandler(MPI_Comm_errhandler_function *function, MPI_Errhandler *errhandler);

int NQJ_Comm_create_group(MPI_Comm comm, MPI_Group group, int tag, MPI_Comm *newcomm);

int NQJ_Comm_create_keyval(MPI_Comm_copy_attr_function *comm_copy_attr_fn, MPI_Comm_delete_attr_function *comm_delete_attr_fn, int *comm_keyval, void *extra_state);

int NQJ_Comm_delete_attr(MPI_Comm comm, int comm_keyval);

int NQJ_Comm_disconnect(MPI_Comm *comm);

int NQJ_Comm_dup(MPI_Comm comm, MPI_Comm *newcomm);

int NQJ_Comm_dup_with_info(MPI_Comm comm, MPI_Info info, MPI_Comm *newcomm);

int NQJ_Comm_free(MPI_Comm *comm);

int NQJ_Comm_free_keyval(int *comm_keyval);

int NQJ_Comm_get_attr(MPI_Comm comm, int comm_keyval, void *attribute_val, int *flag);

int NQJ_Comm_get_errhandler(MPI_Comm comm, MPI_Errhandler *erhandler);

int NQJ_Comm_get_info(MPI_Comm comm, MPI_Info *info_used);

int NQJ_Comm_get_name(MPI_Comm comm, char *comm_name, int *resultlen);

int NQJ_Comm_get_parent(MPI_Comm *parent);

int NQJ_Comm_group(MPI_Comm comm, MPI_Group *group);

int NQJ_Comm_idup(MPI_Comm comm, MPI_Comm *newcomm, MPI_Request *request);

int NQJ_Comm_join(int fd, MPI_Comm *intercomm);

int NQJ_Comm_rank(MPI_Comm comm, int *rank);

int NQJ_Comm_remote_group(MPI_Comm comm, MPI_Group *group);

int NQJ_Comm_remote_size(MPI_Comm comm, int *size);

int NQJ_Comm_set_attr(MPI_Comm comm, int comm_keyval, void *attribute_val);

int NQJ_Comm_set_errhandler(MPI_Comm comm, MPI_Errhandler errhandler);

int NQJ_Comm_set_info(MPI_Comm comm, MPI_Info info);

int NQJ_Comm_set_name(MPI_Comm comm, const char *comm_name);

int NQJ_Comm_size(MPI_Comm comm, int *size);

int NQJ_Comm_split(MPI_Comm comm, int color, int key, MPI_Comm *newcomm);

int NQJ_Comm_split_type(MPI_Comm comm, int split_type, int key, MPI_Info info, MPI_Comm *newcomm);

int NQJ_Comm_test_inter(MPI_Comm comm, int *flag);

int NQJ_Compare_and_swap(const void *origin_addr, const void *compare_addr, void *result_addr, MPI_Datatype datatype, int target_rank, MPI_Aint target_disp, MPI_Win win);

int NQJ_Dims_create(int nnodes, int ndims, int dims[]);

int NQJ_Dist_graph_create(MPI_Comm comm_old, int n, const int nodes[], const int degrees[], const int targets[], const int weights[], MPI_Info info, int reorder, MPI_Comm *newcomm);

int NQJ_Dist_graph_create_adjacent(MPI_Comm comm_old, int indegree, const int sources[], const int sourceweights[], int outdegree, const int destinations[], const int destweights[], MPI_Info info, int reorder, MPI_Comm *comm_dist_graph);

int NQJ_Dist_graph_neighbors(MPI_Comm comm, int maxindegree, int sources[], int sourceweights[], int maxoutdegree, int destinations[], int destweights[]);

int NQJ_Dist_graph_neighbors_count(MPI_Comm comm, int *inneighbors, int *outneighbors, int *weighted);

int NQJ_Errhandler_create(MPI_Handler_function *function, MPI_Errhandler *errhandler);

int NQJ_Errhandler_free(MPI_Errhandler *errhandler);

int NQJ_Errhandler_get(MPI_Comm comm, MPI_Errhandler *errhandler);

int NQJ_Errhandler_set(MPI_Comm comm, MPI_Errhandler errhandler);

int NQJ_Error_class(int errorcode, int *errorclass);

int NQJ_Error_string(int errorcode, char *string, int *resultlen);

int NQJ_Exscan(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm);

int NQJ_Fetch_and_op(const void *origin_addr, void *result_addr, MPI_Datatype datatype, int target_rank, MPI_Aint target_disp, MPI_Op op, MPI_Win win);

int NQJ_File_call_errhandler(MPI_File fh, int errorcode);

int NQJ_File_close(MPI_File *fh);

int NQJ_File_create_errhandler(MPI_File_errhandler_function *function, MPI_Errhandler *errhandler);

int NQJ_File_delete(const char *filename, MPI_Info info);

int NQJ_File_get_amode(MPI_File fh, int *amode);

int NQJ_File_get_atomicity(MPI_File fh, int *flag);

int NQJ_File_get_byte_offset(MPI_File fh, MPI_Offset offset, MPI_Offset *disp);

int NQJ_File_get_errhandler(MPI_File file, MPI_Errhandler *errhandler);

int NQJ_File_get_group(MPI_File fh, MPI_Group *group);

int NQJ_File_get_info(MPI_File fh, MPI_Info *info_used);

int NQJ_File_get_position(MPI_File fh, MPI_Offset *offset);

int NQJ_File_get_position_shared(MPI_File fh, MPI_Offset *offset);

int NQJ_File_get_size(MPI_File fh, MPI_Offset *size);

int NQJ_File_get_type_extent(MPI_File fh, MPI_Datatype datatype, MPI_Aint *extent);

int NQJ_File_get_view(MPI_File fh, MPI_Offset *disp, MPI_Datatype *etype, MPI_Datatype *filetype, char *datarep);

int NQJ_File_iread(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Request *request);

int NQJ_File_iread_all(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Request *request);

int NQJ_File_iread_at(MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype, MPI_Request *request);

int NQJ_File_iread_at_all(MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype, MPI_Request *request);

int NQJ_File_iread_shared(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Request *request);

int NQJ_File_iwrite(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Request *request);

int NQJ_File_iwrite_all(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Request *request);

int NQJ_File_iwrite_at(MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype, MPI_Request *request);

int NQJ_File_iwrite_at_all(MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype, MPI_Request *request);

int NQJ_File_iwrite_shared(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Request *request);

int NQJ_File_open(MPI_Comm comm, const char *filename, int amode, MPI_Info info, MPI_File *fh);

int NQJ_File_preallocate(MPI_File fh, MPI_Offset size);

int NQJ_File_read(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status);

int NQJ_File_read_all(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status);

int NQJ_File_read_all_begin(MPI_File fh, void *buf, int count, MPI_Datatype datatype);

int NQJ_File_read_all_end(MPI_File fh, void *buf, MPI_Status *status);

int NQJ_File_read_at(MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype, MPI_Status *status);

int NQJ_File_read_at_all(MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype, MPI_Status *status);

int NQJ_File_read_at_all_begin(MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype);

int NQJ_File_read_at_all_end(MPI_File fh, void *buf, MPI_Status *status);

int NQJ_File_read_ordered(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status);

int NQJ_File_read_ordered_begin(MPI_File fh, void *buf, int count, MPI_Datatype datatype);

int NQJ_File_read_ordered_end(MPI_File fh, void *buf, MPI_Status *status);

int NQJ_File_read_shared(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status);

int NQJ_File_seek(MPI_File fh, MPI_Offset offset, int whence);

int NQJ_File_seek_shared(MPI_File fh, MPI_Offset offset, int whence);

int NQJ_File_set_atomicity(MPI_File fh, int flag);

int NQJ_File_set_errhandler(MPI_File file, MPI_Errhandler errhandler);

int NQJ_File_set_info(MPI_File fh, MPI_Info info);

int NQJ_File_set_size(MPI_File fh, MPI_Offset size);

int NQJ_File_set_view(MPI_File fh, MPI_Offset disp, MPI_Datatype etype, MPI_Datatype filetype, const char *datarep, MPI_Info info);

int NQJ_File_sync(MPI_File fh);

int NQJ_File_write(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status);

int NQJ_File_write_all(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status);

int NQJ_File_write_all_begin(MPI_File fh, const void *buf, int count, MPI_Datatype datatype);

int NQJ_File_write_all_end(MPI_File fh, const void *buf, MPI_Status *status);

int NQJ_File_write_at(MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status);

int NQJ_File_write_at_all(MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status);

int NQJ_File_write_at_all_begin(MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype);

int NQJ_File_write_at_all_end(MPI_File fh, const void *buf, MPI_Status *status);

int NQJ_File_write_ordered(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status);

int NQJ_File_write_ordered_begin(MPI_File fh, const void *buf, int count, MPI_Datatype datatype);

int NQJ_File_write_ordered_end(MPI_File fh, const void *buf, MPI_Status *status);

int NQJ_File_write_shared(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status);

int NQJ_Finalize();

int NQJ_Finalized(int *flag);

int NQJ_Free_mem(void *base);

int NQJ_Gather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm);

int NQJ_Gatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, int root, MPI_Comm comm);

int NQJ_Get(void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Win win);

int NQJ_Get_accumulate(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, void *result_addr, int result_count, MPI_Datatype result_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win);

int NQJ_Get_address(const void *location, MPI_Aint *address);

int NQJ_Get_count(const MPI_Status *status, MPI_Datatype datatype, int *count);

int NQJ_Get_elements(const MPI_Status *status, MPI_Datatype datatype, int *count);

int NQJ_Get_elements_x(const MPI_Status *status, MPI_Datatype datatype, MPI_Count *count);

int NQJ_Get_library_version(char *version, int *resultlen);

int NQJ_Get_processor_name(char *name, int *resultlen);

int NQJ_Get_version(int *version, int *subversion);

int NQJ_Graph_create(MPI_Comm comm_old, int nnodes, const int index[], const int edges[], int reorder, MPI_Comm *comm_graph);

int NQJ_Graph_get(MPI_Comm comm, int maxindex, int maxedges, int index[], int edges[]);

int NQJ_Graph_map(MPI_Comm comm, int nnodes, const int index[], const int edges[], int *newrank);

int NQJ_Graph_neighbors(MPI_Comm comm, int rank, int maxneighbors, int neighbors[]);

int NQJ_Graph_neighbors_count(MPI_Comm comm, int rank, int *nneighbors);

int NQJ_Graphdims_get(MPI_Comm comm, int *nnodes, int *nedges);

int NQJ_Grequest_complete(MPI_Request request);

int NQJ_Grequest_start(MPI_Grequest_query_function *query_fn, MPI_Grequest_free_function *free_fn, MPI_Grequest_cancel_function *cancel_fn, void *extra_state, MPI_Request *request);

int NQJ_Group_compare(MPI_Group group1, MPI_Group group2, int *result);

int NQJ_Group_difference(MPI_Group group1, MPI_Group group2, MPI_Group *newgroup);

int NQJ_Group_excl(MPI_Group group, int n, const int ranks[], MPI_Group *newgroup);

int NQJ_Group_free(MPI_Group *group);

int NQJ_Group_incl(MPI_Group group, int n, const int ranks[], MPI_Group *newgroup);

int NQJ_Group_intersection(MPI_Group group1, MPI_Group group2, MPI_Group *newgroup);

int NQJ_Group_range_excl(MPI_Group group, int n, int ranges[][3], MPI_Group *newgroup);

int NQJ_Group_range_incl(MPI_Group group, int n, int ranges[][3], MPI_Group *newgroup);

int NQJ_Group_rank(MPI_Group group, int *rank);

int NQJ_Group_size(MPI_Group group, int *size);

int NQJ_Group_translate_ranks(MPI_Group group1, int n, const int ranks1[], MPI_Group group2, int ranks2[]);

int NQJ_Group_union(MPI_Group group1, MPI_Group group2, MPI_Group *newgroup);

int NQJ_Iallgather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);

int NQJ_Iallgatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);

int NQJ_Iallreduce(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request);

int NQJ_Ialltoall(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);

int NQJ_Ialltoallv(const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);

int NQJ_Ialltoallw(const void *sendbuf, const int sendcounts[], const int sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const int rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm, MPI_Request *request);

int NQJ_Ibarrier(MPI_Comm comm, MPI_Request *request);

int NQJ_Ibcast(void *buffer, int count, MPI_Datatype datatype, int root, MPI_Comm comm, MPI_Request *request);

int NQJ_Ibsend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);

int NQJ_Iexscan(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request);

int NQJ_Igather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request);

int NQJ_Igatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request);

int NQJ_Improbe(int source, int tag, MPI_Comm comm, int *flag, MPI_Message *message, MPI_Status *status);

int NQJ_Imrecv(void *buf, int count, MPI_Datatype type, MPI_Message *message, MPI_Request *request);

int NQJ_Ineighbor_allgather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);

int NQJ_Ineighbor_allgatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);

int NQJ_Ineighbor_alltoall(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);

int NQJ_Ineighbor_alltoallv(const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);

int NQJ_Ineighbor_alltoallw(const void *sendbuf, const int sendcounts[], const MPI_Aint sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const MPI_Aint rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm, MPI_Request *request);

int NQJ_Info_create(MPI_Info *info);

int NQJ_Info_delete(MPI_Info info, const char *key);

int NQJ_Info_dup(MPI_Info info, MPI_Info *newinfo);

int NQJ_Info_free(MPI_Info *info);

int NQJ_Info_get(MPI_Info info, const char *key, int valuelen, char *value, int *flag);

int NQJ_Info_get_nkeys(MPI_Info info, int *nkeys);

int NQJ_Info_get_nthkey(MPI_Info info, int n, char *key);

int NQJ_Info_get_valuelen(MPI_Info info, const char *key, int *valuelen, int *flag);

int NQJ_Info_set(MPI_Info info, const char *key, const char *value);

int NQJ_Init(int *argc, char ***argv);

int NQJ_Init_thread(int *argc, char ***argv, int required, int *provided);

int NQJ_Initialized(int *flag);

int NQJ_Intercomm_create(MPI_Comm local_comm, int local_leader, MPI_Comm bridge_comm, int remote_leader, int tag, MPI_Comm *newintercomm);

int NQJ_Intercomm_merge(MPI_Comm intercomm, int high, MPI_Comm *newintercomm);

int NQJ_Iprobe(int source, int tag, MPI_Comm comm, int *flag, MPI_Status *status);

int NQJ_Irecv(void *buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Request *request);

int NQJ_Ireduce(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, int root, MPI_Comm comm, MPI_Request *request);

int NQJ_Ireduce_scatter(const void *sendbuf, void *recvbuf, const int recvcounts[], MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request);

int NQJ_Ireduce_scatter_block(const void *sendbuf, void *recvbuf, int recvcount, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request);

int NQJ_Irsend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);

int NQJ_Is_thread_main(int *flag);

int NQJ_Iscan(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request);

int NQJ_Iscatter(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request);

int NQJ_Iscatterv(const void *sendbuf, const int sendcounts[], const int displs[], MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request);

int NQJ_Isend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);

int NQJ_Issend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);

int NQJ_Keyval_create(MPI_Copy_function *copy_fn, MPI_Delete_function *delete_fn, int *keyval, void *extra_state);

int NQJ_Keyval_free(int *keyval);

int NQJ_Lookup_name(const char *service_name, MPI_Info info, char *port_name);

int NQJ_Mprobe(int source, int tag, MPI_Comm comm, MPI_Message *message, MPI_Status *status);

int NQJ_Mrecv(void *buf, int count, MPI_Datatype type, MPI_Message *message, MPI_Status *status);

int NQJ_Neighbor_allgather(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm);

int NQJ_Neighbor_allgatherv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm);

int NQJ_Neighbor_alltoall(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm);

int NQJ_Neighbor_alltoallv(const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm);

int NQJ_Neighbor_alltoallw(const void *sendbuf, const int sendcounts[], const MPI_Aint sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const MPI_Aint rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm);

int NQJ_Op_commutative(MPI_Op op, int *commute);

int NQJ_Op_create(MPI_User_function *function, int commute, MPI_Op *op);

int NQJ_Op_free(MPI_Op *op);

int NQJ_Open_port(MPI_Info info, char *port_name);

int NQJ_Pack(const void *inbuf, int incount, MPI_Datatype datatype, void *outbuf, int outsize, int *position, MPI_Comm comm);

int NQJ_Pack_external(const char datarep[], const void *inbuf, int incount, MPI_Datatype datatype, void *outbuf, MPI_Aint outsize, MPI_Aint *position);

int NQJ_Pack_external_size(const char datarep[], int incount, MPI_Datatype datatype, MPI_Aint *size);

int NQJ_Pack_size(int incount, MPI_Datatype datatype, MPI_Comm comm, int *size);

int NQJ_Probe(int source, int tag, MPI_Comm comm, MPI_Status *status);

int NQJ_Publish_name(const char *service_name, MPI_Info info, const char *port_name);

int NQJ_Put(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Win win);

int NQJ_Query_thread(int *provided);

int NQJ_Raccumulate(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win, MPI_Request *request);

int NQJ_Recv(void *buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Status *status);

int NQJ_Recv_init(void *buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Request *request);

int NQJ_Reduce(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, int root, MPI_Comm comm);

int NQJ_Reduce_local(const void *inbuf, void *inoutbuf, int count, MPI_Datatype datatype, MPI_Op op);

int NQJ_Reduce_scatter(const void *sendbuf, void *recvbuf, const int recvcounts[], MPI_Datatype datatype, MPI_Op op, MPI_Comm comm);

int NQJ_Reduce_scatter_block(const void *sendbuf, void *recvbuf, int recvcount, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm);

int NQJ_Register_datarep(const char *datarep, MPI_Datarep_conversion_function *read_conversion_fn, MPI_Datarep_conversion_function *write_conversion_fn, MPI_Datarep_extent_function *dtype_file_extent_fn, void *extra_state);

int NQJ_Request_free(MPI_Request *request);

int NQJ_Request_get_status(MPI_Request request, int *flag, MPI_Status *status);

int NQJ_Rget(void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Win win, MPI_Request *request);

int NQJ_Rget_accumulate(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, void *result_addr, int result_count, MPI_Datatype result_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win, MPI_Request *request);

int NQJ_Rput(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_cout, MPI_Datatype target_datatype, MPI_Win win, MPI_Request *request);

int NQJ_Rsend(const void *ibuf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm);

int NQJ_Rsend_init(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);

int NQJ_Scan(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm);

int NQJ_Scatter(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm);

int NQJ_Scatterv(const void *sendbuf, const int sendcounts[], const int displs[], MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm);

int NQJ_Send(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm);

int NQJ_Send_init(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);

int NQJ_Sendrecv(const void *sendbuf, int sendcount, MPI_Datatype sendtype, int dest, int sendtag, void *recvbuf, int recvcount, MPI_Datatype recvtype, int source, int recvtag, MPI_Comm comm, MPI_Status *status);

int NQJ_Sendrecv_replace(void *buf, int count, MPI_Datatype datatype, int dest, int sendtag, int source, int recvtag, MPI_Comm comm, MPI_Status *status);

int NQJ_Ssend(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm);

int NQJ_Ssend_init(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);

int NQJ_Start(MPI_Request *request);

int NQJ_Startall(int count, MPI_Request array_of_requests[]);

int NQJ_Status_set_cancelled(MPI_Status *status, int flag);

int NQJ_Status_set_elements(MPI_Status *status, MPI_Datatype datatype, int count);

int NQJ_Status_set_elements_x(MPI_Status *status, MPI_Datatype datatype, MPI_Count count);

int NQJ_Test(MPI_Request *request, int *flag, MPI_Status *status);

int NQJ_Test_cancelled(const MPI_Status *status, int *flag);

int NQJ_Testall(int count, MPI_Request array_of_requests[], int *flag, MPI_Status array_of_statuses[]);

int NQJ_Testany(int count, MPI_Request array_of_requests[], int *index, int *flag, MPI_Status *status);

int NQJ_Testsome(int incount, MPI_Request array_of_requests[], int *outcount, int array_of_indices[], MPI_Status array_of_statuses[]);

int NQJ_Topo_test(MPI_Comm comm, int *status);

int NQJ_Type_commit(MPI_Datatype *type);

int NQJ_Type_contiguous(int count, MPI_Datatype oldtype, MPI_Datatype *newtype);

int NQJ_Type_create_darray(int size, int rank, int ndims, const int gsize_array[], const int distrib_array[], const int darg_array[], const int psize_array[], int order, MPI_Datatype oldtype, MPI_Datatype *newtype);

int NQJ_Type_create_f90_complex(int p, int r, MPI_Datatype *newtype);

int NQJ_Type_create_f90_integer(int r, MPI_Datatype *newtype);

int NQJ_Type_create_f90_real(int p, int r, MPI_Datatype *newtype);

int NQJ_Type_create_hindexed(int count, const int array_of_blocklengths[], const MPI_Aint array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype);

int NQJ_Type_create_hindexed_block(int count, int blocklength, const MPI_Aint array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype);

int NQJ_Type_create_hvector(int count, int blocklength, MPI_Aint stride, MPI_Datatype oldtype, MPI_Datatype *newtype);

int NQJ_Type_create_indexed_block(int count, int blocklength, const int array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype);

int NQJ_Type_create_keyval(MPI_Type_copy_attr_function *type_copy_attr_fn, MPI_Type_delete_attr_function *type_delete_attr_fn, int *type_keyval, void *extra_state);

int NQJ_Type_create_resized(MPI_Datatype oldtype, MPI_Aint lb, MPI_Aint extent, MPI_Datatype *newtype);

int NQJ_Type_create_struct(int count, const int array_of_block_lengths[], const MPI_Aint array_of_displacements[], const MPI_Datatype array_of_types[], MPI_Datatype *newtype);

int NQJ_Type_create_subarray(int ndims, const int size_array[], const int subsize_array[], const int start_array[], int order, MPI_Datatype oldtype, MPI_Datatype *newtype);

int NQJ_Type_delete_attr(MPI_Datatype type, int type_keyval);

int NQJ_Type_dup(MPI_Datatype type, MPI_Datatype *newtype);

int NQJ_Type_extent(MPI_Datatype type, MPI_Aint *extent);

int NQJ_Type_free(MPI_Datatype *type);

int NQJ_Type_free_keyval(int *type_keyval);

int NQJ_Type_get_attr(MPI_Datatype type, int type_keyval, void *attribute_val, int *flag);

int NQJ_Type_get_contents(MPI_Datatype mtype, int max_integers, int max_addresses, int max_datatypes, int array_of_integers[], MPI_Aint array_of_addresses[], MPI_Datatype array_of_datatypes[]);

int NQJ_Type_get_envelope(MPI_Datatype type, int *num_integers, int *num_addresses, int *num_datatypes, int *combiner);

int NQJ_Type_get_extent(MPI_Datatype type, MPI_Aint *lb, MPI_Aint *extent);

int NQJ_Type_get_extent_x(MPI_Datatype type, MPI_Count *lb, MPI_Count *extent);

int NQJ_Type_get_name(MPI_Datatype type, char *type_name, int *resultlen);

int NQJ_Type_get_true_extent(MPI_Datatype datatype, MPI_Aint *true_lb, MPI_Aint *true_extent);

int NQJ_Type_get_true_extent_x(MPI_Datatype datatype, MPI_Count *true_lb, MPI_Count *true_extent);

int NQJ_Type_hindexed(int count, int array_of_blocklengths[], MPI_Aint array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype);

int NQJ_Type_hvector(int count, int blocklength, MPI_Aint stride, MPI_Datatype oldtype, MPI_Datatype *newtype);

int NQJ_Type_indexed(int count, const int array_of_blocklengths[], const int array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype);

int NQJ_Type_lb(MPI_Datatype type, MPI_Aint *lb);

int NQJ_Type_match_size(int typeclass, int size, MPI_Datatype *type);

int NQJ_Type_set_attr(MPI_Datatype type, int type_keyval, void *attr_val);

int NQJ_Type_set_name(MPI_Datatype type, const char *type_name);

int NQJ_Type_size(MPI_Datatype type, int *size);

int NQJ_Type_size_x(MPI_Datatype type, MPI_Count *size);

int NQJ_Type_struct(int count, int array_of_blocklengths[], MPI_Aint array_of_displacements[], MPI_Datatype array_of_types[], MPI_Datatype *newtype);

int NQJ_Type_ub(MPI_Datatype mtype, MPI_Aint *ub);

int NQJ_Type_vector(int count, int blocklength, int stride, MPI_Datatype oldtype, MPI_Datatype *newtype);

int NQJ_Unpack(const void *inbuf, int insize, int *position, void *outbuf, int outcount, MPI_Datatype datatype, MPI_Comm comm);

int NQJ_Unpack_external(const char datarep[], const void *inbuf, MPI_Aint insize, MPI_Aint *position, void *outbuf, int outcount, MPI_Datatype datatype);

int NQJ_Unpublish_name(const char *service_name, MPI_Info info, const char *port_name);

int NQJ_Wait(MPI_Request *request, MPI_Status *status);

int NQJ_Waitall(int count, MPI_Request array_of_requests[], MPI_Status *array_of_statuses);

int NQJ_Waitany(int count, MPI_Request array_of_requests[], int *index, MPI_Status *status);

int NQJ_Waitsome(int incount, MPI_Request array_of_requests[], int *outcount, int array_of_indices[], MPI_Status array_of_statuses[]);

int NQJ_Win_allocate(MPI_Aint size, int disp_unit, MPI_Info info, MPI_Comm comm, void *baseptr, MPI_Win *win);

int NQJ_Win_allocate_shared(MPI_Aint size, int disp_unit, MPI_Info info, MPI_Comm comm, void *baseptr, MPI_Win *win);

int NQJ_Win_attach(MPI_Win win, void *base, MPI_Aint size);

int NQJ_Win_call_errhandler(MPI_Win win, int errorcode);

int NQJ_Win_complete(MPI_Win win);

int NQJ_Win_create(void *base, MPI_Aint size, int disp_unit, MPI_Info info, MPI_Comm comm, MPI_Win *win);

int NQJ_Win_create_dynamic(MPI_Info info, MPI_Comm comm, MPI_Win *win);

int NQJ_Win_create_errhandler(MPI_Win_errhandler_function *function, MPI_Errhandler *errhandler);

int NQJ_Win_create_keyval(MPI_Win_copy_attr_function *win_copy_attr_fn, MPI_Win_delete_attr_function *win_delete_attr_fn, int *win_keyval, void *extra_state);

int NQJ_Win_delete_attr(MPI_Win win, int win_keyval);

int NQJ_Win_detach(MPI_Win win, const void *base);

int NQJ_Win_fence(int assert, MPI_Win win);

int NQJ_Win_flush(int rank, MPI_Win win);

int NQJ_Win_flush_all(MPI_Win win);

int NQJ_Win_flush_local(int rank, MPI_Win win);

int NQJ_Win_flush_local_all(MPI_Win win);

int NQJ_Win_free(MPI_Win *win);

int NQJ_Win_free_keyval(int *win_keyval);

int NQJ_Win_get_attr(MPI_Win win, int win_keyval, void *attribute_val, int *flag);

int NQJ_Win_get_errhandler(MPI_Win win, MPI_Errhandler *errhandler);

int NQJ_Win_get_group(MPI_Win win, MPI_Group *group);

int NQJ_Win_get_info(MPI_Win win, MPI_Info *info_used);

int NQJ_Win_get_name(MPI_Win win, char *win_name, int *resultlen);

int NQJ_Win_lock(int lock_type, int rank, int assert, MPI_Win win);

int NQJ_Win_lock_all(int assert, MPI_Win win);

int NQJ_Win_post(MPI_Group group, int assert, MPI_Win win);

int NQJ_Win_set_attr(MPI_Win win, int win_keyval, void *attribute_val);

int NQJ_Win_set_errhandler(MPI_Win win, MPI_Errhandler errhandler);

int NQJ_Win_set_info(MPI_Win win, MPI_Info info);

int NQJ_Win_set_name(MPI_Win win, const char *win_name);

int NQJ_Win_shared_query(MPI_Win win, int rank, MPI_Aint *size, int *disp_unit, void *baseptr);

int NQJ_Win_start(MPI_Group group, int assert, MPI_Win win);

int NQJ_Win_sync(MPI_Win win);

int NQJ_Win_test(MPI_Win win, int *flag);

int NQJ_Win_unlock(int rank, MPI_Win win);

int NQJ_Win_unlock_all(MPI_Win win);

int NQJ_Win_wait(MPI_Win win);

double NQJ_Wtick();

double NQJ_Wtime();


#include <pnmpi/xmpi.h>


#define MPI_Abort_ID 0

#define MPI_Accumulate_ID 1

#define MPI_Add_error_class_ID 2

#define MPI_Add_error_code_ID 3

#define MPI_Add_error_string_ID 4

#define MPI_Address_ID 5

#define MPI_Allgather_ID 6

#define MPI_Allgatherv_ID 7

#define MPI_Alloc_mem_ID 8

#define MPI_Allreduce_ID 9

#define MPI_Alltoall_ID 10

#define MPI_Alltoallv_ID 11

#define MPI_Alltoallw_ID 12

#define MPI_Attr_delete_ID 13

#define MPI_Attr_get_ID 14

#define MPI_Attr_put_ID 15

#define MPI_Barrier_ID 16

#define MPI_Bcast_ID 17

#define MPI_Bsend_ID 18

#define MPI_Bsend_init_ID 19

#define MPI_Buffer_attach_ID 20

#define MPI_Buffer_detach_ID 21

#define MPI_Cancel_ID 22

#define MPI_Cart_coords_ID 23

#define MPI_Cart_create_ID 24

#define MPI_Cart_get_ID 25

#define MPI_Cart_map_ID 26

#define MPI_Cart_rank_ID 27

#define MPI_Cart_shift_ID 28

#define MPI_Cart_sub_ID 29

#define MPI_Cartdim_get_ID 30

#define MPI_Close_port_ID 31

#define MPI_Comm_accept_ID 32

#define MPI_Comm_call_errhandler_ID 33

#define MPI_Comm_compare_ID 34

#define MPI_Comm_connect_ID 35

#define MPI_Comm_create_ID 36

#define MPI_Comm_create_errhandler_ID 37

#define MPI_Comm_create_group_ID 38

#define MPI_Comm_create_keyval_ID 39

#define MPI_Comm_delete_attr_ID 40

#define MPI_Comm_disconnect_ID 41

#define MPI_Comm_dup_ID 42

#define MPI_Comm_dup_with_info_ID 43

#define MPI_Comm_free_ID 44

#define MPI_Comm_free_keyval_ID 45

#define MPI_Comm_get_attr_ID 46

#define MPI_Comm_get_errhandler_ID 47

#define MPI_Comm_get_info_ID 48

#define MPI_Comm_get_name_ID 49

#define MPI_Comm_get_parent_ID 50

#define MPI_Comm_group_ID 51

#define MPI_Comm_idup_ID 52

#define MPI_Comm_join_ID 53

#define MPI_Comm_rank_ID 54

#define MPI_Comm_remote_group_ID 55

#define MPI_Comm_remote_size_ID 56

#define MPI_Comm_set_attr_ID 57

#define MPI_Comm_set_errhandler_ID 58

#define MPI_Comm_set_info_ID 59

#define MPI_Comm_set_name_ID 60

#define MPI_Comm_size_ID 61

#define MPI_Comm_split_ID 62

#define MPI_Comm_split_type_ID 63

#define MPI_Comm_test_inter_ID 64

#define MPI_Compare_and_swap_ID 65

#define MPI_Dims_create_ID 66

#define MPI_Dist_graph_create_ID 67

#define MPI_Dist_graph_create_adjacent_ID 68

#define MPI_Dist_graph_neighbors_ID 69

#define MPI_Dist_graph_neighbors_count_ID 70

#define MPI_Errhandler_create_ID 71

#define MPI_Errhandler_free_ID 72

#define MPI_Errhandler_get_ID 73

#define MPI_Errhandler_set_ID 74

#define MPI_Error_class_ID 75

#define MPI_Error_string_ID 76

#define MPI_Exscan_ID 77

#define MPI_Fetch_and_op_ID 78

#define MPI_File_call_errhandler_ID 79

#define MPI_File_close_ID 80

#define MPI_File_create_errhandler_ID 81

#define MPI_File_delete_ID 82

#define MPI_File_get_amode_ID 83

#define MPI_File_get_atomicity_ID 84

#define MPI_File_get_byte_offset_ID 85

#define MPI_File_get_errhandler_ID 86

#define MPI_File_get_group_ID 87

#define MPI_File_get_info_ID 88

#define MPI_File_get_position_ID 89

#define MPI_File_get_position_shared_ID 90

#define MPI_File_get_size_ID 91

#define MPI_File_get_type_extent_ID 92

#define MPI_File_get_view_ID 93

#define MPI_File_iread_ID 94

#define MPI_File_iread_all_ID 95

#define MPI_File_iread_at_ID 96

#define MPI_File_iread_at_all_ID 97

#define MPI_File_iread_shared_ID 98

#define MPI_File_iwrite_ID 99

#define MPI_File_iwrite_all_ID 100

#define MPI_File_iwrite_at_ID 101

#define MPI_File_iwrite_at_all_ID 102

#define MPI_File_iwrite_shared_ID 103

#define MPI_File_open_ID 104

#define MPI_File_preallocate_ID 105

#define MPI_File_read_ID 106

#define MPI_File_read_all_ID 107

#define MPI_File_read_all_begin_ID 108

#define MPI_File_read_all_end_ID 109

#define MPI_File_read_at_ID 110

#define MPI_File_read_at_all_ID 111

#define MPI_File_read_at_all_begin_ID 112

#define MPI_File_read_at_all_end_ID 113

#define MPI_File_read_ordered_ID 114

#define MPI_File_read_ordered_begin_ID 115

#define MPI_File_read_ordered_end_ID 116

#define MPI_File_read_shared_ID 117

#define MPI_File_seek_ID 118

#define MPI_File_seek_shared_ID 119

#define MPI_File_set_atomicity_ID 120

#define MPI_File_set_errhandler_ID 121

#define MPI_File_set_info_ID 122

#define MPI_File_set_size_ID 123

#define MPI_File_set_view_ID 124

#define MPI_File_sync_ID 125

#define MPI_File_write_ID 126

#define MPI_File_write_all_ID 127

#define MPI_File_write_all_begin_ID 128

#define MPI_File_write_all_end_ID 129

#define MPI_File_write_at_ID 130

#define MPI_File_write_at_all_ID 131

#define MPI_File_write_at_all_begin_ID 132

#define MPI_File_write_at_all_end_ID 133

#define MPI_File_write_ordered_ID 134

#define MPI_File_write_ordered_begin_ID 135

#define MPI_File_write_ordered_end_ID 136

#define MPI_File_write_shared_ID 137

#define MPI_Finalize_ID 138

#define MPI_Finalized_ID 139

#define MPI_Free_mem_ID 140

#define MPI_Gather_ID 141

#define MPI_Gatherv_ID 142

#define MPI_Get_ID 143

#define MPI_Get_accumulate_ID 144

#define MPI_Get_address_ID 145

#define MPI_Get_count_ID 146

#define MPI_Get_elements_ID 147

#define MPI_Get_elements_x_ID 148

#define MPI_Get_library_version_ID 149

#define MPI_Get_processor_name_ID 150

#define MPI_Get_version_ID 151

#define MPI_Graph_create_ID 152

#define MPI_Graph_get_ID 153

#define MPI_Graph_map_ID 154

#define MPI_Graph_neighbors_ID 155

#define MPI_Graph_neighbors_count_ID 156

#define MPI_Graphdims_get_ID 157

#define MPI_Grequest_complete_ID 158

#define MPI_Grequest_start_ID 159

#define MPI_Group_compare_ID 160

#define MPI_Group_difference_ID 161

#define MPI_Group_excl_ID 162

#define MPI_Group_free_ID 163

#define MPI_Group_incl_ID 164

#define MPI_Group_intersection_ID 165

#define MPI_Group_range_excl_ID 166

#define MPI_Group_range_incl_ID 167

#define MPI_Group_rank_ID 168

#define MPI_Group_size_ID 169

#define MPI_Group_translate_ranks_ID 170

#define MPI_Group_union_ID 171

#define MPI_Iallgather_ID 172

#define MPI_Iallgatherv_ID 173

#define MPI_Iallreduce_ID 174

#define MPI_Ialltoall_ID 175

#define MPI_Ialltoallv_ID 176

#define MPI_Ialltoallw_ID 177

#define MPI_Ibarrier_ID 178

#define MPI_Ibcast_ID 179

#define MPI_Ibsend_ID 180

#define MPI_Iexscan_ID 181

#define MPI_Igather_ID 182

#define MPI_Igatherv_ID 183

#define MPI_Improbe_ID 184

#define MPI_Imrecv_ID 185

#define MPI_Ineighbor_allgather_ID 186

#define MPI_Ineighbor_allgatherv_ID 187

#define MPI_Ineighbor_alltoall_ID 188

#define MPI_Ineighbor_alltoallv_ID 189

#define MPI_Ineighbor_alltoallw_ID 190

#define MPI_Info_create_ID 191

#define MPI_Info_delete_ID 192

#define MPI_Info_dup_ID 193

#define MPI_Info_free_ID 194

#define MPI_Info_get_ID 195

#define MPI_Info_get_nkeys_ID 196

#define MPI_Info_get_nthkey_ID 197

#define MPI_Info_get_valuelen_ID 198

#define MPI_Info_set_ID 199

#define MPI_Init_ID 200

#define MPI_Init_thread_ID 201

#define MPI_Initialized_ID 202

#define MPI_Intercomm_create_ID 203

#define MPI_Intercomm_merge_ID 204

#define MPI_Iprobe_ID 205

#define MPI_Irecv_ID 206

#define MPI_Ireduce_ID 207

#define MPI_Ireduce_scatter_ID 208

#define MPI_Ireduce_scatter_block_ID 209

#define MPI_Irsend_ID 210

#define MPI_Is_thread_main_ID 211

#define MPI_Iscan_ID 212

#define MPI_Iscatter_ID 213

#define MPI_Iscatterv_ID 214

#define MPI_Isend_ID 215

#define MPI_Issend_ID 216

#define MPI_Keyval_create_ID 217

#define MPI_Keyval_free_ID 218

#define MPI_Lookup_name_ID 219

#define MPI_Mprobe_ID 220

#define MPI_Mrecv_ID 221

#define MPI_Neighbor_allgather_ID 222

#define MPI_Neighbor_allgatherv_ID 223

#define MPI_Neighbor_alltoall_ID 224

#define MPI_Neighbor_alltoallv_ID 225

#define MPI_Neighbor_alltoallw_ID 226

#define MPI_Op_commutative_ID 227

#define MPI_Op_create_ID 228

#define MPI_Op_free_ID 229

#define MPI_Open_port_ID 230

#define MPI_Pack_ID 231

#define MPI_Pack_external_ID 232

#define MPI_Pack_external_size_ID 233

#define MPI_Pack_size_ID 234

#define MPI_Pcontrol_ID 235

#define MPI_Probe_ID 236

#define MPI_Publish_name_ID 237

#define MPI_Put_ID 238

#define MPI_Query_thread_ID 239

#define MPI_Raccumulate_ID 240

#define MPI_Recv_ID 241

#define MPI_Recv_init_ID 242

#define MPI_Reduce_ID 243

#define MPI_Reduce_local_ID 244

#define MPI_Reduce_scatter_ID 245

#define MPI_Reduce_scatter_block_ID 246

#define MPI_Register_datarep_ID 247

#define MPI_Request_free_ID 248

#define MPI_Request_get_status_ID 249

#define MPI_Rget_ID 250

#define MPI_Rget_accumulate_ID 251

#define MPI_Rput_ID 252

#define MPI_Rsend_ID 253

#define MPI_Rsend_init_ID 254

#define MPI_Scan_ID 255

#define MPI_Scatter_ID 256

#define MPI_Scatterv_ID 257

#define MPI_Send_ID 258

#define MPI_Send_init_ID 259

#define MPI_Sendrecv_ID 260

#define MPI_Sendrecv_replace_ID 261

#define MPI_Ssend_ID 262

#define MPI_Ssend_init_ID 263

#define MPI_Start_ID 264

#define MPI_Startall_ID 265

#define MPI_Status_set_cancelled_ID 266

#define MPI_Status_set_elements_ID 267

#define MPI_Status_set_elements_x_ID 268

#define MPI_Test_ID 269

#define MPI_Test_cancelled_ID 270

#define MPI_Testall_ID 271

#define MPI_Testany_ID 272

#define MPI_Testsome_ID 273

#define MPI_Topo_test_ID 274

#define MPI_Type_commit_ID 275

#define MPI_Type_contiguous_ID 276

#define MPI_Type_create_darray_ID 277

#define MPI_Type_create_f90_complex_ID 278

#define MPI_Type_create_f90_integer_ID 279

#define MPI_Type_create_f90_real_ID 280

#define MPI_Type_create_hindexed_ID 281

#define MPI_Type_create_hindexed_block_ID 282

#define MPI_Type_create_hvector_ID 283

#define MPI_Type_create_indexed_block_ID 284

#define MPI_Type_create_keyval_ID 285

#define MPI_Type_create_resized_ID 286

#define MPI_Type_create_struct_ID 287

#define MPI_Type_create_subarray_ID 288

#define MPI_Type_delete_attr_ID 289

#define MPI_Type_dup_ID 290

#define MPI_Type_extent_ID 291

#define MPI_Type_free_ID 292

#define MPI_Type_free_keyval_ID 293

#define MPI_Type_get_attr_ID 294

#define MPI_Type_get_contents_ID 295

#define MPI_Type_get_envelope_ID 296

#define MPI_Type_get_extent_ID 297

#define MPI_Type_get_extent_x_ID 298

#define MPI_Type_get_name_ID 299

#define MPI_Type_get_true_extent_ID 300

#define MPI_Type_get_true_extent_x_ID 301

#define MPI_Type_hindexed_ID 302

#define MPI_Type_hvector_ID 303

#define MPI_Type_indexed_ID 304

#define MPI_Type_lb_ID 305

#define MPI_Type_match_size_ID 306

#define MPI_Type_set_attr_ID 307

#define MPI_Type_set_name_ID 308

#define MPI_Type_size_ID 309

#define MPI_Type_size_x_ID 310

#define MPI_Type_struct_ID 311

#define MPI_Type_ub_ID 312

#define MPI_Type_vector_ID 313

#define MPI_Unpack_ID 314

#define MPI_Unpack_external_ID 315

#define MPI_Unpublish_name_ID 316

#define MPI_Wait_ID 317

#define MPI_Waitall_ID 318

#define MPI_Waitany_ID 319

#define MPI_Waitsome_ID 320

#define MPI_Win_allocate_ID 321

#define MPI_Win_allocate_shared_ID 322

#define MPI_Win_attach_ID 323

#define MPI_Win_call_errhandler_ID 324

#define MPI_Win_complete_ID 325

#define MPI_Win_create_ID 326

#define MPI_Win_create_dynamic_ID 327

#define MPI_Win_create_errhandler_ID 328

#define MPI_Win_create_keyval_ID 329

#define MPI_Win_delete_attr_ID 330

#define MPI_Win_detach_ID 331

#define MPI_Win_fence_ID 332

#define MPI_Win_flush_ID 333

#define MPI_Win_flush_all_ID 334

#define MPI_Win_flush_local_ID 335

#define MPI_Win_flush_local_all_ID 336

#define MPI_Win_free_ID 337

#define MPI_Win_free_keyval_ID 338

#define MPI_Win_get_attr_ID 339

#define MPI_Win_get_errhandler_ID 340

#define MPI_Win_get_group_ID 341

#define MPI_Win_get_info_ID 342

#define MPI_Win_get_name_ID 343

#define MPI_Win_lock_ID 344

#define MPI_Win_lock_all_ID 345

#define MPI_Win_post_ID 346

#define MPI_Win_set_attr_ID 347

#define MPI_Win_set_errhandler_ID 348

#define MPI_Win_set_info_ID 349

#define MPI_Win_set_name_ID 350

#define MPI_Win_shared_query_ID 351

#define MPI_Win_start_ID 352

#define MPI_Win_sync_ID 353

#define MPI_Win_test_ID 354

#define MPI_Win_unlock_ID 355

#define MPI_Win_unlock_all_ID 356

#define MPI_Win_wait_ID 357

#define MPI_Wtick_ID 358

#define MPI_Wtime_ID 359



#define Internal_XMPI_Abort NQJ_Abort

#define Internal_XMPI_Accumulate NQJ_Accumulate

#define Internal_XMPI_Add_error_class NQJ_Add_error_class

#define Internal_XMPI_Add_error_code NQJ_Add_error_code

#define Internal_XMPI_Add_error_string NQJ_Add_error_string

#define Internal_XMPI_Address NQJ_Address

#define Internal_XMPI_Allgather NQJ_Allgather

#define Internal_XMPI_Allgatherv NQJ_Allgatherv

#define Internal_XMPI_Alloc_mem NQJ_Alloc_mem

#define Internal_XMPI_Allreduce NQJ_Allreduce

#define Internal_XMPI_Alltoall NQJ_Alltoall

#define Internal_XMPI_Alltoallv NQJ_Alltoallv

#define Internal_XMPI_Alltoallw NQJ_Alltoallw

#define Internal_XMPI_Attr_delete NQJ_Attr_delete

#define Internal_XMPI_Attr_get NQJ_Attr_get

#define Internal_XMPI_Attr_put NQJ_Attr_put

#define Internal_XMPI_Barrier NQJ_Barrier

#define Internal_XMPI_Bcast NQJ_Bcast

#define Internal_XMPI_Bsend NQJ_Bsend

#define Internal_XMPI_Bsend_init NQJ_Bsend_init

#define Internal_XMPI_Buffer_attach NQJ_Buffer_attach

#define Internal_XMPI_Buffer_detach NQJ_Buffer_detach

#define Internal_XMPI_Cancel NQJ_Cancel

#define Internal_XMPI_Cart_coords NQJ_Cart_coords

#define Internal_XMPI_Cart_create NQJ_Cart_create

#define Internal_XMPI_Cart_get NQJ_Cart_get

#define Internal_XMPI_Cart_map NQJ_Cart_map

#define Internal_XMPI_Cart_rank NQJ_Cart_rank

#define Internal_XMPI_Cart_shift NQJ_Cart_shift

#define Internal_XMPI_Cart_sub NQJ_Cart_sub

#define Internal_XMPI_Cartdim_get NQJ_Cartdim_get

#define Internal_XMPI_Close_port NQJ_Close_port

#define Internal_XMPI_Comm_accept NQJ_Comm_accept

#define Internal_XMPI_Comm_call_errhandler NQJ_Comm_call_errhandler

#define Internal_XMPI_Comm_compare NQJ_Comm_compare

#define Internal_XMPI_Comm_connect NQJ_Comm_connect

#define Internal_XMPI_Comm_create NQJ_Comm_create

#define Internal_XMPI_Comm_create_errhandler NQJ_Comm_create_errhandler

#define Internal_XMPI_Comm_create_group NQJ_Comm_create_group

#define Internal_XMPI_Comm_create_keyval NQJ_Comm_create_keyval

#define Internal_XMPI_Comm_delete_attr NQJ_Comm_delete_attr

#define Internal_XMPI_Comm_disconnect NQJ_Comm_disconnect

#define Internal_XMPI_Comm_dup NQJ_Comm_dup

#define Internal_XMPI_Comm_dup_with_info NQJ_Comm_dup_with_info

#define Internal_XMPI_Comm_free NQJ_Comm_free

#define Internal_XMPI_Comm_free_keyval NQJ_Comm_free_keyval

#define Internal_XMPI_Comm_get_attr NQJ_Comm_get_attr

#define Internal_XMPI_Comm_get_errhandler NQJ_Comm_get_errhandler

#define Internal_XMPI_Comm_get_info NQJ_Comm_get_info

#define Internal_XMPI_Comm_get_name NQJ_Comm_get_name

#define Internal_XMPI_Comm_get_parent NQJ_Comm_get_parent

#define Internal_XMPI_Comm_group NQJ_Comm_group

#define Internal_XMPI_Comm_idup NQJ_Comm_idup

#define Internal_XMPI_Comm_join NQJ_Comm_join

#define Internal_XMPI_Comm_rank NQJ_Comm_rank

#define Internal_XMPI_Comm_remote_group NQJ_Comm_remote_group

#define Internal_XMPI_Comm_remote_size NQJ_Comm_remote_size

#define Internal_XMPI_Comm_set_attr NQJ_Comm_set_attr

#define Internal_XMPI_Comm_set_errhandler NQJ_Comm_set_errhandler

#define Internal_XMPI_Comm_set_info NQJ_Comm_set_info

#define Internal_XMPI_Comm_set_name NQJ_Comm_set_name

#define Internal_XMPI_Comm_size NQJ_Comm_size

#define Internal_XMPI_Comm_split NQJ_Comm_split

#define Internal_XMPI_Comm_split_type NQJ_Comm_split_type

#define Internal_XMPI_Comm_test_inter NQJ_Comm_test_inter

#define Internal_XMPI_Compare_and_swap NQJ_Compare_and_swap

#define Internal_XMPI_Dims_create NQJ_Dims_create

#define Internal_XMPI_Dist_graph_create NQJ_Dist_graph_create

#define Internal_XMPI_Dist_graph_create_adjacent NQJ_Dist_graph_create_adjacent

#define Internal_XMPI_Dist_graph_neighbors NQJ_Dist_graph_neighbors

#define Internal_XMPI_Dist_graph_neighbors_count NQJ_Dist_graph_neighbors_count

#define Internal_XMPI_Errhandler_create NQJ_Errhandler_create

#define Internal_XMPI_Errhandler_free NQJ_Errhandler_free

#define Internal_XMPI_Errhandler_get NQJ_Errhandler_get

#define Internal_XMPI_Errhandler_set NQJ_Errhandler_set

#define Internal_XMPI_Error_class NQJ_Error_class

#define Internal_XMPI_Error_string NQJ_Error_string

#define Internal_XMPI_Exscan NQJ_Exscan

#define Internal_XMPI_Fetch_and_op NQJ_Fetch_and_op

#define Internal_XMPI_File_call_errhandler NQJ_File_call_errhandler

#define Internal_XMPI_File_close NQJ_File_close

#define Internal_XMPI_File_create_errhandler NQJ_File_create_errhandler

#define Internal_XMPI_File_delete NQJ_File_delete

#define Internal_XMPI_File_get_amode NQJ_File_get_amode

#define Internal_XMPI_File_get_atomicity NQJ_File_get_atomicity

#define Internal_XMPI_File_get_byte_offset NQJ_File_get_byte_offset

#define Internal_XMPI_File_get_errhandler NQJ_File_get_errhandler

#define Internal_XMPI_File_get_group NQJ_File_get_group

#define Internal_XMPI_File_get_info NQJ_File_get_info

#define Internal_XMPI_File_get_position NQJ_File_get_position

#define Internal_XMPI_File_get_position_shared NQJ_File_get_position_shared

#define Internal_XMPI_File_get_size NQJ_File_get_size

#define Internal_XMPI_File_get_type_extent NQJ_File_get_type_extent

#define Internal_XMPI_File_get_view NQJ_File_get_view

#define Internal_XMPI_File_iread NQJ_File_iread

#define Internal_XMPI_File_iread_all NQJ_File_iread_all

#define Internal_XMPI_File_iread_at NQJ_File_iread_at

#define Internal_XMPI_File_iread_at_all NQJ_File_iread_at_all

#define Internal_XMPI_File_iread_shared NQJ_File_iread_shared

#define Internal_XMPI_File_iwrite NQJ_File_iwrite

#define Internal_XMPI_File_iwrite_all NQJ_File_iwrite_all

#define Internal_XMPI_File_iwrite_at NQJ_File_iwrite_at

#define Internal_XMPI_File_iwrite_at_all NQJ_File_iwrite_at_all

#define Internal_XMPI_File_iwrite_shared NQJ_File_iwrite_shared

#define Internal_XMPI_File_open NQJ_File_open

#define Internal_XMPI_File_preallocate NQJ_File_preallocate

#define Internal_XMPI_File_read NQJ_File_read

#define Internal_XMPI_File_read_all NQJ_File_read_all

#define Internal_XMPI_File_read_all_begin NQJ_File_read_all_begin

#define Internal_XMPI_File_read_all_end NQJ_File_read_all_end

#define Internal_XMPI_File_read_at NQJ_File_read_at

#define Internal_XMPI_File_read_at_all NQJ_File_read_at_all

#define Internal_XMPI_File_read_at_all_begin NQJ_File_read_at_all_begin

#define Internal_XMPI_File_read_at_all_end NQJ_File_read_at_all_end

#define Internal_XMPI_File_read_ordered NQJ_File_read_ordered

#define Internal_XMPI_File_read_ordered_begin NQJ_File_read_ordered_begin

#define Internal_XMPI_File_read_ordered_end NQJ_File_read_ordered_end

#define Internal_XMPI_File_read_shared NQJ_File_read_shared

#define Internal_XMPI_File_seek NQJ_File_seek

#define Internal_XMPI_File_seek_shared NQJ_File_seek_shared

#define Internal_XMPI_File_set_atomicity NQJ_File_set_atomicity

#define Internal_XMPI_File_set_errhandler NQJ_File_set_errhandler

#define Internal_XMPI_File_set_info NQJ_File_set_info

#define Internal_XMPI_File_set_size NQJ_File_set_size

#define Internal_XMPI_File_set_view NQJ_File_set_view

#define Internal_XMPI_File_sync NQJ_File_sync

#define Internal_XMPI_File_write NQJ_File_write

#define Internal_XMPI_File_write_all NQJ_File_write_all

#define Internal_XMPI_File_write_all_begin NQJ_File_write_all_begin

#define Internal_XMPI_File_write_all_end NQJ_File_write_all_end

#define Internal_XMPI_File_write_at NQJ_File_write_at

#define Internal_XMPI_File_write_at_all NQJ_File_write_at_all

#define Internal_XMPI_File_write_at_all_begin NQJ_File_write_at_all_begin

#define Internal_XMPI_File_write_at_all_end NQJ_File_write_at_all_end

#define Internal_XMPI_File_write_ordered NQJ_File_write_ordered

#define Internal_XMPI_File_write_ordered_begin NQJ_File_write_ordered_begin

#define Internal_XMPI_File_write_ordered_end NQJ_File_write_ordered_end

#define Internal_XMPI_File_write_shared NQJ_File_write_shared

#define Internal_XMPI_Finalize NQJ_Finalize

#define Internal_XMPI_Finalized NQJ_Finalized

#define Internal_XMPI_Free_mem NQJ_Free_mem

#define Internal_XMPI_Gather NQJ_Gather

#define Internal_XMPI_Gatherv NQJ_Gatherv

#define Internal_XMPI_Get NQJ_Get

#define Internal_XMPI_Get_accumulate NQJ_Get_accumulate

#define Internal_XMPI_Get_address NQJ_Get_address

#define Internal_XMPI_Get_count NQJ_Get_count

#define Internal_XMPI_Get_elements NQJ_Get_elements

#define Internal_XMPI_Get_elements_x NQJ_Get_elements_x

#define Internal_XMPI_Get_library_version NQJ_Get_library_version

#define Internal_XMPI_Get_processor_name NQJ_Get_processor_name

#define Internal_XMPI_Get_version NQJ_Get_version

#define Internal_XMPI_Graph_create NQJ_Graph_create

#define Internal_XMPI_Graph_get NQJ_Graph_get

#define Internal_XMPI_Graph_map NQJ_Graph_map

#define Internal_XMPI_Graph_neighbors NQJ_Graph_neighbors

#define Internal_XMPI_Graph_neighbors_count NQJ_Graph_neighbors_count

#define Internal_XMPI_Graphdims_get NQJ_Graphdims_get

#define Internal_XMPI_Grequest_complete NQJ_Grequest_complete

#define Internal_XMPI_Grequest_start NQJ_Grequest_start

#define Internal_XMPI_Group_compare NQJ_Group_compare

#define Internal_XMPI_Group_difference NQJ_Group_difference

#define Internal_XMPI_Group_excl NQJ_Group_excl

#define Internal_XMPI_Group_free NQJ_Group_free

#define Internal_XMPI_Group_incl NQJ_Group_incl

#define Internal_XMPI_Group_intersection NQJ_Group_intersection

#define Internal_XMPI_Group_range_excl NQJ_Group_range_excl

#define Internal_XMPI_Group_range_incl NQJ_Group_range_incl

#define Internal_XMPI_Group_rank NQJ_Group_rank

#define Internal_XMPI_Group_size NQJ_Group_size

#define Internal_XMPI_Group_translate_ranks NQJ_Group_translate_ranks

#define Internal_XMPI_Group_union NQJ_Group_union

#define Internal_XMPI_Iallgather NQJ_Iallgather

#define Internal_XMPI_Iallgatherv NQJ_Iallgatherv

#define Internal_XMPI_Iallreduce NQJ_Iallreduce

#define Internal_XMPI_Ialltoall NQJ_Ialltoall

#define Internal_XMPI_Ialltoallv NQJ_Ialltoallv

#define Internal_XMPI_Ialltoallw NQJ_Ialltoallw

#define Internal_XMPI_Ibarrier NQJ_Ibarrier

#define Internal_XMPI_Ibcast NQJ_Ibcast

#define Internal_XMPI_Ibsend NQJ_Ibsend

#define Internal_XMPI_Iexscan NQJ_Iexscan

#define Internal_XMPI_Igather NQJ_Igather

#define Internal_XMPI_Igatherv NQJ_Igatherv

#define Internal_XMPI_Improbe NQJ_Improbe

#define Internal_XMPI_Imrecv NQJ_Imrecv

#define Internal_XMPI_Ineighbor_allgather NQJ_Ineighbor_allgather

#define Internal_XMPI_Ineighbor_allgatherv NQJ_Ineighbor_allgatherv

#define Internal_XMPI_Ineighbor_alltoall NQJ_Ineighbor_alltoall

#define Internal_XMPI_Ineighbor_alltoallv NQJ_Ineighbor_alltoallv

#define Internal_XMPI_Ineighbor_alltoallw NQJ_Ineighbor_alltoallw

#define Internal_XMPI_Info_create NQJ_Info_create

#define Internal_XMPI_Info_delete NQJ_Info_delete

#define Internal_XMPI_Info_dup NQJ_Info_dup

#define Internal_XMPI_Info_free NQJ_Info_free

#define Internal_XMPI_Info_get NQJ_Info_get

#define Internal_XMPI_Info_get_nkeys NQJ_Info_get_nkeys

#define Internal_XMPI_Info_get_nthkey NQJ_Info_get_nthkey

#define Internal_XMPI_Info_get_valuelen NQJ_Info_get_valuelen

#define Internal_XMPI_Info_set NQJ_Info_set

#define Internal_XMPI_Init NQJ_Init

#define Internal_XMPI_Init_thread NQJ_Init_thread

#define Internal_XMPI_Initialized NQJ_Initialized

#define Internal_XMPI_Intercomm_create NQJ_Intercomm_create

#define Internal_XMPI_Intercomm_merge NQJ_Intercomm_merge

#define Internal_XMPI_Iprobe NQJ_Iprobe

#define Internal_XMPI_Irecv NQJ_Irecv

#define Internal_XMPI_Ireduce NQJ_Ireduce

#define Internal_XMPI_Ireduce_scatter NQJ_Ireduce_scatter

#define Internal_XMPI_Ireduce_scatter_block NQJ_Ireduce_scatter_block

#define Internal_XMPI_Irsend NQJ_Irsend

#define Internal_XMPI_Is_thread_main NQJ_Is_thread_main

#define Internal_XMPI_Iscan NQJ_Iscan

#define Internal_XMPI_Iscatter NQJ_Iscatter

#define Internal_XMPI_Iscatterv NQJ_Iscatterv

#define Internal_XMPI_Isend NQJ_Isend

#define Internal_XMPI_Issend NQJ_Issend

#define Internal_XMPI_Keyval_create NQJ_Keyval_create

#define Internal_XMPI_Keyval_free NQJ_Keyval_free

#define Internal_XMPI_Lookup_name NQJ_Lookup_name

#define Internal_XMPI_Mprobe NQJ_Mprobe

#define Internal_XMPI_Mrecv NQJ_Mrecv

#define Internal_XMPI_Neighbor_allgather NQJ_Neighbor_allgather

#define Internal_XMPI_Neighbor_allgatherv NQJ_Neighbor_allgatherv

#define Internal_XMPI_Neighbor_alltoall NQJ_Neighbor_alltoall

#define Internal_XMPI_Neighbor_alltoallv NQJ_Neighbor_alltoallv

#define Internal_XMPI_Neighbor_alltoallw NQJ_Neighbor_alltoallw

#define Internal_XMPI_Op_commutative NQJ_Op_commutative

#define Internal_XMPI_Op_create NQJ_Op_create

#define Internal_XMPI_Op_free NQJ_Op_free

#define Internal_XMPI_Open_port NQJ_Open_port

#define Internal_XMPI_Pack NQJ_Pack

#define Internal_XMPI_Pack_external NQJ_Pack_external

#define Internal_XMPI_Pack_external_size NQJ_Pack_external_size

#define Internal_XMPI_Pack_size NQJ_Pack_size

#define Internal_XMPI_Probe NQJ_Probe

#define Internal_XMPI_Publish_name NQJ_Publish_name

#define Internal_XMPI_Put NQJ_Put

#define Internal_XMPI_Query_thread NQJ_Query_thread

#define Internal_XMPI_Raccumulate NQJ_Raccumulate

#define Internal_XMPI_Recv NQJ_Recv

#define Internal_XMPI_Recv_init NQJ_Recv_init

#define Internal_XMPI_Reduce NQJ_Reduce

#define Internal_XMPI_Reduce_local NQJ_Reduce_local

#define Internal_XMPI_Reduce_scatter NQJ_Reduce_scatter

#define Internal_XMPI_Reduce_scatter_block NQJ_Reduce_scatter_block

#define Internal_XMPI_Register_datarep NQJ_Register_datarep

#define Internal_XMPI_Request_free NQJ_Request_free

#define Internal_XMPI_Request_get_status NQJ_Request_get_status

#define Internal_XMPI_Rget NQJ_Rget

#define Internal_XMPI_Rget_accumulate NQJ_Rget_accumulate

#define Internal_XMPI_Rput NQJ_Rput

#define Internal_XMPI_Rsend NQJ_Rsend

#define Internal_XMPI_Rsend_init NQJ_Rsend_init

#define Internal_XMPI_Scan NQJ_Scan

#define Internal_XMPI_Scatter NQJ_Scatter

#define Internal_XMPI_Scatterv NQJ_Scatterv

#define Internal_XMPI_Send NQJ_Send

#define Internal_XMPI_Send_init NQJ_Send_init

#define Internal_XMPI_Sendrecv NQJ_Sendrecv

#define Internal_XMPI_Sendrecv_replace NQJ_Sendrecv_replace

#define Internal_XMPI_Ssend NQJ_Ssend

#define Internal_XMPI_Ssend_init NQJ_Ssend_init

#define Internal_XMPI_Start NQJ_Start

#define Internal_XMPI_Startall NQJ_Startall

#define Internal_XMPI_Status_set_cancelled NQJ_Status_set_cancelled

#define Internal_XMPI_Status_set_elements NQJ_Status_set_elements

#define Internal_XMPI_Status_set_elements_x NQJ_Status_set_elements_x

#define Internal_XMPI_Test NQJ_Test

#define Internal_XMPI_Test_cancelled NQJ_Test_cancelled

#define Internal_XMPI_Testall NQJ_Testall

#define Internal_XMPI_Testany NQJ_Testany

#define Internal_XMPI_Testsome NQJ_Testsome

#define Internal_XMPI_Topo_test NQJ_Topo_test

#define Internal_XMPI_Type_commit NQJ_Type_commit

#define Internal_XMPI_Type_contiguous NQJ_Type_contiguous

#define Internal_XMPI_Type_create_darray NQJ_Type_create_darray

#define Internal_XMPI_Type_create_f90_complex NQJ_Type_create_f90_complex

#define Internal_XMPI_Type_create_f90_integer NQJ_Type_create_f90_integer

#define Internal_XMPI_Type_create_f90_real NQJ_Type_create_f90_real

#define Internal_XMPI_Type_create_hindexed NQJ_Type_create_hindexed

#define Internal_XMPI_Type_create_hindexed_block NQJ_Type_create_hindexed_block

#define Internal_XMPI_Type_create_hvector NQJ_Type_create_hvector

#define Internal_XMPI_Type_create_indexed_block NQJ_Type_create_indexed_block

#define Internal_XMPI_Type_create_keyval NQJ_Type_create_keyval

#define Internal_XMPI_Type_create_resized NQJ_Type_create_resized

#define Internal_XMPI_Type_create_struct NQJ_Type_create_struct

#define Internal_XMPI_Type_create_subarray NQJ_Type_create_subarray

#define Internal_XMPI_Type_delete_attr NQJ_Type_delete_attr

#define Internal_XMPI_Type_dup NQJ_Type_dup

#define Internal_XMPI_Type_extent NQJ_Type_extent

#define Internal_XMPI_Type_free NQJ_Type_free

#define Internal_XMPI_Type_free_keyval NQJ_Type_free_keyval

#define Internal_XMPI_Type_get_attr NQJ_Type_get_attr

#define Internal_XMPI_Type_get_contents NQJ_Type_get_contents

#define Internal_XMPI_Type_get_envelope NQJ_Type_get_envelope

#define Internal_XMPI_Type_get_extent NQJ_Type_get_extent

#define Internal_XMPI_Type_get_extent_x NQJ_Type_get_extent_x

#define Internal_XMPI_Type_get_name NQJ_Type_get_name

#define Internal_XMPI_Type_get_true_extent NQJ_Type_get_true_extent

#define Internal_XMPI_Type_get_true_extent_x NQJ_Type_get_true_extent_x

#define Internal_XMPI_Type_hindexed NQJ_Type_hindexed

#define Internal_XMPI_Type_hvector NQJ_Type_hvector

#define Internal_XMPI_Type_indexed NQJ_Type_indexed

#define Internal_XMPI_Type_lb NQJ_Type_lb

#define Internal_XMPI_Type_match_size NQJ_Type_match_size

#define Internal_XMPI_Type_set_attr NQJ_Type_set_attr

#define Internal_XMPI_Type_set_name NQJ_Type_set_name

#define Internal_XMPI_Type_size NQJ_Type_size

#define Internal_XMPI_Type_size_x NQJ_Type_size_x

#define Internal_XMPI_Type_struct NQJ_Type_struct

#define Internal_XMPI_Type_ub NQJ_Type_ub

#define Internal_XMPI_Type_vector NQJ_Type_vector

#define Internal_XMPI_Unpack NQJ_Unpack

#define Internal_XMPI_Unpack_external NQJ_Unpack_external

#define Internal_XMPI_Unpublish_name NQJ_Unpublish_name

#define Internal_XMPI_Wait NQJ_Wait

#define Internal_XMPI_Waitall NQJ_Waitall

#define Internal_XMPI_Waitany NQJ_Waitany

#define Internal_XMPI_Waitsome NQJ_Waitsome

#define Internal_XMPI_Win_allocate NQJ_Win_allocate

#define Internal_XMPI_Win_allocate_shared NQJ_Win_allocate_shared

#define Internal_XMPI_Win_attach NQJ_Win_attach

#define Internal_XMPI_Win_call_errhandler NQJ_Win_call_errhandler

#define Internal_XMPI_Win_complete NQJ_Win_complete

#define Internal_XMPI_Win_create NQJ_Win_create

#define Internal_XMPI_Win_create_dynamic NQJ_Win_create_dynamic

#define Internal_XMPI_Win_create_errhandler NQJ_Win_create_errhandler

#define Internal_XMPI_Win_create_keyval NQJ_Win_create_keyval

#define Internal_XMPI_Win_delete_attr NQJ_Win_delete_attr

#define Internal_XMPI_Win_detach NQJ_Win_detach

#define Internal_XMPI_Win_fence NQJ_Win_fence

#define Internal_XMPI_Win_flush NQJ_Win_flush

#define Internal_XMPI_Win_flush_all NQJ_Win_flush_all

#define Internal_XMPI_Win_flush_local NQJ_Win_flush_local

#define Internal_XMPI_Win_flush_local_all NQJ_Win_flush_local_all

#define Internal_XMPI_Win_free NQJ_Win_free

#define Internal_XMPI_Win_free_keyval NQJ_Win_free_keyval

#define Internal_XMPI_Win_get_attr NQJ_Win_get_attr

#define Internal_XMPI_Win_get_errhandler NQJ_Win_get_errhandler

#define Internal_XMPI_Win_get_group NQJ_Win_get_group

#define Internal_XMPI_Win_get_info NQJ_Win_get_info

#define Internal_XMPI_Win_get_name NQJ_Win_get_name

#define Internal_XMPI_Win_lock NQJ_Win_lock

#define Internal_XMPI_Win_lock_all NQJ_Win_lock_all

#define Internal_XMPI_Win_post NQJ_Win_post

#define Internal_XMPI_Win_set_attr NQJ_Win_set_attr

#define Internal_XMPI_Win_set_errhandler NQJ_Win_set_errhandler

#define Internal_XMPI_Win_set_info NQJ_Win_set_info

#define Internal_XMPI_Win_set_name NQJ_Win_set_name

#define Internal_XMPI_Win_shared_query NQJ_Win_shared_query

#define Internal_XMPI_Win_start NQJ_Win_start

#define Internal_XMPI_Win_sync NQJ_Win_sync

#define Internal_XMPI_Win_test NQJ_Win_test

#define Internal_XMPI_Win_unlock NQJ_Win_unlock

#define Internal_XMPI_Win_unlock_all NQJ_Win_unlock_all

#define Internal_XMPI_Win_wait NQJ_Win_wait

#define Internal_XMPI_Wtick NQJ_Wtick

#define Internal_XMPI_Wtime NQJ_Wtime



typedef int (*pnmpi_int_MPI_Abort_t)(MPI_Comm comm, int errorcode);
typedef int (*pnmpi_int_MPI_Accumulate_t)(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win);
typedef int (*pnmpi_int_MPI_Add_error_class_t)(int *errorclass);
typedef int (*pnmpi_int_MPI_Add_error_code_t)(int errorclass, int *errorcode);
typedef int (*pnmpi_int_MPI_Add_error_string_t)(int errorcode, const char *string);
typedef int (*pnmpi_int_MPI_Address_t)(void *location, MPI_Aint *address);
typedef int (*pnmpi_int_MPI_Allgather_t)(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm);
typedef int (*pnmpi_int_MPI_Allgatherv_t)(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm);
typedef int (*pnmpi_int_MPI_Alloc_mem_t)(MPI_Aint size, MPI_Info info, void *baseptr);
typedef int (*pnmpi_int_MPI_Allreduce_t)(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm);
typedef int (*pnmpi_int_MPI_Alltoall_t)(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm);
typedef int (*pnmpi_int_MPI_Alltoallv_t)(const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm);
typedef int (*pnmpi_int_MPI_Alltoallw_t)(const void *sendbuf, const int sendcounts[], const int sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const int rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm);
typedef int (*pnmpi_int_MPI_Attr_delete_t)(MPI_Comm comm, int keyval);
typedef int (*pnmpi_int_MPI_Attr_get_t)(MPI_Comm comm, int keyval, void *attribute_val, int *flag);
typedef int (*pnmpi_int_MPI_Attr_put_t)(MPI_Comm comm, int keyval, void *attribute_val);
typedef int (*pnmpi_int_MPI_Barrier_t)(MPI_Comm comm);
typedef int (*pnmpi_int_MPI_Bcast_t)(void *buffer, int count, MPI_Datatype datatype, int root, MPI_Comm comm);
typedef int (*pnmpi_int_MPI_Bsend_t)(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm);
typedef int (*pnmpi_int_MPI_Bsend_init_t)(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);
typedef int (*pnmpi_int_MPI_Buffer_attach_t)(void *buffer, int size);
typedef int (*pnmpi_int_MPI_Buffer_detach_t)(void *buffer, int *size);
typedef int (*pnmpi_int_MPI_Cancel_t)(MPI_Request *request);
typedef int (*pnmpi_int_MPI_Cart_coords_t)(MPI_Comm comm, int rank, int maxdims, int coords[]);
typedef int (*pnmpi_int_MPI_Cart_create_t)(MPI_Comm old_comm, int ndims, const int dims[], const int periods[], int reorder, MPI_Comm *comm_cart);
typedef int (*pnmpi_int_MPI_Cart_get_t)(MPI_Comm comm, int maxdims, int dims[], int periods[], int coords[]);
typedef int (*pnmpi_int_MPI_Cart_map_t)(MPI_Comm comm, int ndims, const int dims[], const int periods[], int *newrank);
typedef int (*pnmpi_int_MPI_Cart_rank_t)(MPI_Comm comm, const int coords[], int *rank);
typedef int (*pnmpi_int_MPI_Cart_shift_t)(MPI_Comm comm, int direction, int disp, int *rank_source, int *rank_dest);
typedef int (*pnmpi_int_MPI_Cart_sub_t)(MPI_Comm comm, const int remain_dims[], MPI_Comm *new_comm);
typedef int (*pnmpi_int_MPI_Cartdim_get_t)(MPI_Comm comm, int *ndims);
typedef int (*pnmpi_int_MPI_Close_port_t)(const char *port_name);
typedef int (*pnmpi_int_MPI_Comm_accept_t)(const char *port_name, MPI_Info info, int root, MPI_Comm comm, MPI_Comm *newcomm);
typedef int (*pnmpi_int_MPI_Comm_call_errhandler_t)(MPI_Comm comm, int errorcode);
typedef int (*pnmpi_int_MPI_Comm_compare_t)(MPI_Comm comm1, MPI_Comm comm2, int *result);
typedef int (*pnmpi_int_MPI_Comm_connect_t)(const char *port_name, MPI_Info info, int root, MPI_Comm comm, MPI_Comm *newcomm);
typedef int (*pnmpi_int_MPI_Comm_create_t)(MPI_Comm comm, MPI_Group group, MPI_Comm *newcomm);
typedef int (*pnmpi_int_MPI_Comm_create_errhandler_t)(MPI_Comm_errhandler_function *function, MPI_Errhandler *errhandler);
typedef int (*pnmpi_int_MPI_Comm_create_group_t)(MPI_Comm comm, MPI_Group group, int tag, MPI_Comm *newcomm);
typedef int (*pnmpi_int_MPI_Comm_create_keyval_t)(MPI_Comm_copy_attr_function *comm_copy_attr_fn, MPI_Comm_delete_attr_function *comm_delete_attr_fn, int *comm_keyval, void *extra_state);
typedef int (*pnmpi_int_MPI_Comm_delete_attr_t)(MPI_Comm comm, int comm_keyval);
typedef int (*pnmpi_int_MPI_Comm_disconnect_t)(MPI_Comm *comm);
typedef int (*pnmpi_int_MPI_Comm_dup_t)(MPI_Comm comm, MPI_Comm *newcomm);
typedef int (*pnmpi_int_MPI_Comm_dup_with_info_t)(MPI_Comm comm, MPI_Info info, MPI_Comm *newcomm);
typedef int (*pnmpi_int_MPI_Comm_free_t)(MPI_Comm *comm);
typedef int (*pnmpi_int_MPI_Comm_free_keyval_t)(int *comm_keyval);
typedef int (*pnmpi_int_MPI_Comm_get_attr_t)(MPI_Comm comm, int comm_keyval, void *attribute_val, int *flag);
typedef int (*pnmpi_int_MPI_Comm_get_errhandler_t)(MPI_Comm comm, MPI_Errhandler *erhandler);
typedef int (*pnmpi_int_MPI_Comm_get_info_t)(MPI_Comm comm, MPI_Info *info_used);
typedef int (*pnmpi_int_MPI_Comm_get_name_t)(MPI_Comm comm, char *comm_name, int *resultlen);
typedef int (*pnmpi_int_MPI_Comm_get_parent_t)(MPI_Comm *parent);
typedef int (*pnmpi_int_MPI_Comm_group_t)(MPI_Comm comm, MPI_Group *group);
typedef int (*pnmpi_int_MPI_Comm_idup_t)(MPI_Comm comm, MPI_Comm *newcomm, MPI_Request *request);
typedef int (*pnmpi_int_MPI_Comm_join_t)(int fd, MPI_Comm *intercomm);
typedef int (*pnmpi_int_MPI_Comm_rank_t)(MPI_Comm comm, int *rank);
typedef int (*pnmpi_int_MPI_Comm_remote_group_t)(MPI_Comm comm, MPI_Group *group);
typedef int (*pnmpi_int_MPI_Comm_remote_size_t)(MPI_Comm comm, int *size);
typedef int (*pnmpi_int_MPI_Comm_set_attr_t)(MPI_Comm comm, int comm_keyval, void *attribute_val);
typedef int (*pnmpi_int_MPI_Comm_set_errhandler_t)(MPI_Comm comm, MPI_Errhandler errhandler);
typedef int (*pnmpi_int_MPI_Comm_set_info_t)(MPI_Comm comm, MPI_Info info);
typedef int (*pnmpi_int_MPI_Comm_set_name_t)(MPI_Comm comm, const char *comm_name);
typedef int (*pnmpi_int_MPI_Comm_size_t)(MPI_Comm comm, int *size);
typedef int (*pnmpi_int_MPI_Comm_split_t)(MPI_Comm comm, int color, int key, MPI_Comm *newcomm);
typedef int (*pnmpi_int_MPI_Comm_split_type_t)(MPI_Comm comm, int split_type, int key, MPI_Info info, MPI_Comm *newcomm);
typedef int (*pnmpi_int_MPI_Comm_test_inter_t)(MPI_Comm comm, int *flag);
typedef int (*pnmpi_int_MPI_Compare_and_swap_t)(const void *origin_addr, const void *compare_addr, void *result_addr, MPI_Datatype datatype, int target_rank, MPI_Aint target_disp, MPI_Win win);
typedef int (*pnmpi_int_MPI_Dims_create_t)(int nnodes, int ndims, int dims[]);
typedef int (*pnmpi_int_MPI_Dist_graph_create_t)(MPI_Comm comm_old, int n, const int nodes[], const int degrees[], const int targets[], const int weights[], MPI_Info info, int reorder, MPI_Comm *newcomm);
typedef int (*pnmpi_int_MPI_Dist_graph_create_adjacent_t)(MPI_Comm comm_old, int indegree, const int sources[], const int sourceweights[], int outdegree, const int destinations[], const int destweights[], MPI_Info info, int reorder, MPI_Comm *comm_dist_graph);
typedef int (*pnmpi_int_MPI_Dist_graph_neighbors_t)(MPI_Comm comm, int maxindegree, int sources[], int sourceweights[], int maxoutdegree, int destinations[], int destweights[]);
typedef int (*pnmpi_int_MPI_Dist_graph_neighbors_count_t)(MPI_Comm comm, int *inneighbors, int *outneighbors, int *weighted);
typedef int (*pnmpi_int_MPI_Errhandler_create_t)(MPI_Handler_function *function, MPI_Errhandler *errhandler);
typedef int (*pnmpi_int_MPI_Errhandler_free_t)(MPI_Errhandler *errhandler);
typedef int (*pnmpi_int_MPI_Errhandler_get_t)(MPI_Comm comm, MPI_Errhandler *errhandler);
typedef int (*pnmpi_int_MPI_Errhandler_set_t)(MPI_Comm comm, MPI_Errhandler errhandler);
typedef int (*pnmpi_int_MPI_Error_class_t)(int errorcode, int *errorclass);
typedef int (*pnmpi_int_MPI_Error_string_t)(int errorcode, char *string, int *resultlen);
typedef int (*pnmpi_int_MPI_Exscan_t)(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm);
typedef int (*pnmpi_int_MPI_Fetch_and_op_t)(const void *origin_addr, void *result_addr, MPI_Datatype datatype, int target_rank, MPI_Aint target_disp, MPI_Op op, MPI_Win win);
typedef int (*pnmpi_int_MPI_File_call_errhandler_t)(MPI_File fh, int errorcode);
typedef int (*pnmpi_int_MPI_File_close_t)(MPI_File *fh);
typedef int (*pnmpi_int_MPI_File_create_errhandler_t)(MPI_File_errhandler_function *function, MPI_Errhandler *errhandler);
typedef int (*pnmpi_int_MPI_File_delete_t)(const char *filename, MPI_Info info);
typedef int (*pnmpi_int_MPI_File_get_amode_t)(MPI_File fh, int *amode);
typedef int (*pnmpi_int_MPI_File_get_atomicity_t)(MPI_File fh, int *flag);
typedef int (*pnmpi_int_MPI_File_get_byte_offset_t)(MPI_File fh, MPI_Offset offset, MPI_Offset *disp);
typedef int (*pnmpi_int_MPI_File_get_errhandler_t)(MPI_File file, MPI_Errhandler *errhandler);
typedef int (*pnmpi_int_MPI_File_get_group_t)(MPI_File fh, MPI_Group *group);
typedef int (*pnmpi_int_MPI_File_get_info_t)(MPI_File fh, MPI_Info *info_used);
typedef int (*pnmpi_int_MPI_File_get_position_t)(MPI_File fh, MPI_Offset *offset);
typedef int (*pnmpi_int_MPI_File_get_position_shared_t)(MPI_File fh, MPI_Offset *offset);
typedef int (*pnmpi_int_MPI_File_get_size_t)(MPI_File fh, MPI_Offset *size);
typedef int (*pnmpi_int_MPI_File_get_type_extent_t)(MPI_File fh, MPI_Datatype datatype, MPI_Aint *extent);
typedef int (*pnmpi_int_MPI_File_get_view_t)(MPI_File fh, MPI_Offset *disp, MPI_Datatype *etype, MPI_Datatype *filetype, char *datarep);
typedef int (*pnmpi_int_MPI_File_iread_t)(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Request *request);
typedef int (*pnmpi_int_MPI_File_iread_all_t)(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Request *request);
typedef int (*pnmpi_int_MPI_File_iread_at_t)(MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype, MPI_Request *request);
typedef int (*pnmpi_int_MPI_File_iread_at_all_t)(MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype, MPI_Request *request);
typedef int (*pnmpi_int_MPI_File_iread_shared_t)(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Request *request);
typedef int (*pnmpi_int_MPI_File_iwrite_t)(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Request *request);
typedef int (*pnmpi_int_MPI_File_iwrite_all_t)(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Request *request);
typedef int (*pnmpi_int_MPI_File_iwrite_at_t)(MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype, MPI_Request *request);
typedef int (*pnmpi_int_MPI_File_iwrite_at_all_t)(MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype, MPI_Request *request);
typedef int (*pnmpi_int_MPI_File_iwrite_shared_t)(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Request *request);
typedef int (*pnmpi_int_MPI_File_open_t)(MPI_Comm comm, const char *filename, int amode, MPI_Info info, MPI_File *fh);
typedef int (*pnmpi_int_MPI_File_preallocate_t)(MPI_File fh, MPI_Offset size);
typedef int (*pnmpi_int_MPI_File_read_t)(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status);
typedef int (*pnmpi_int_MPI_File_read_all_t)(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status);
typedef int (*pnmpi_int_MPI_File_read_all_begin_t)(MPI_File fh, void *buf, int count, MPI_Datatype datatype);
typedef int (*pnmpi_int_MPI_File_read_all_end_t)(MPI_File fh, void *buf, MPI_Status *status);
typedef int (*pnmpi_int_MPI_File_read_at_t)(MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype, MPI_Status *status);
typedef int (*pnmpi_int_MPI_File_read_at_all_t)(MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype, MPI_Status *status);
typedef int (*pnmpi_int_MPI_File_read_at_all_begin_t)(MPI_File fh, MPI_Offset offset, void *buf, int count, MPI_Datatype datatype);
typedef int (*pnmpi_int_MPI_File_read_at_all_end_t)(MPI_File fh, void *buf, MPI_Status *status);
typedef int (*pnmpi_int_MPI_File_read_ordered_t)(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status);
typedef int (*pnmpi_int_MPI_File_read_ordered_begin_t)(MPI_File fh, void *buf, int count, MPI_Datatype datatype);
typedef int (*pnmpi_int_MPI_File_read_ordered_end_t)(MPI_File fh, void *buf, MPI_Status *status);
typedef int (*pnmpi_int_MPI_File_read_shared_t)(MPI_File fh, void *buf, int count, MPI_Datatype datatype, MPI_Status *status);
typedef int (*pnmpi_int_MPI_File_seek_t)(MPI_File fh, MPI_Offset offset, int whence);
typedef int (*pnmpi_int_MPI_File_seek_shared_t)(MPI_File fh, MPI_Offset offset, int whence);
typedef int (*pnmpi_int_MPI_File_set_atomicity_t)(MPI_File fh, int flag);
typedef int (*pnmpi_int_MPI_File_set_errhandler_t)(MPI_File file, MPI_Errhandler errhandler);
typedef int (*pnmpi_int_MPI_File_set_info_t)(MPI_File fh, MPI_Info info);
typedef int (*pnmpi_int_MPI_File_set_size_t)(MPI_File fh, MPI_Offset size);
typedef int (*pnmpi_int_MPI_File_set_view_t)(MPI_File fh, MPI_Offset disp, MPI_Datatype etype, MPI_Datatype filetype, const char *datarep, MPI_Info info);
typedef int (*pnmpi_int_MPI_File_sync_t)(MPI_File fh);
typedef int (*pnmpi_int_MPI_File_write_t)(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status);
typedef int (*pnmpi_int_MPI_File_write_all_t)(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status);
typedef int (*pnmpi_int_MPI_File_write_all_begin_t)(MPI_File fh, const void *buf, int count, MPI_Datatype datatype);
typedef int (*pnmpi_int_MPI_File_write_all_end_t)(MPI_File fh, const void *buf, MPI_Status *status);
typedef int (*pnmpi_int_MPI_File_write_at_t)(MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status);
typedef int (*pnmpi_int_MPI_File_write_at_all_t)(MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status);
typedef int (*pnmpi_int_MPI_File_write_at_all_begin_t)(MPI_File fh, MPI_Offset offset, const void *buf, int count, MPI_Datatype datatype);
typedef int (*pnmpi_int_MPI_File_write_at_all_end_t)(MPI_File fh, const void *buf, MPI_Status *status);
typedef int (*pnmpi_int_MPI_File_write_ordered_t)(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status);
typedef int (*pnmpi_int_MPI_File_write_ordered_begin_t)(MPI_File fh, const void *buf, int count, MPI_Datatype datatype);
typedef int (*pnmpi_int_MPI_File_write_ordered_end_t)(MPI_File fh, const void *buf, MPI_Status *status);
typedef int (*pnmpi_int_MPI_File_write_shared_t)(MPI_File fh, const void *buf, int count, MPI_Datatype datatype, MPI_Status *status);
typedef int (*pnmpi_int_MPI_Finalize_t)();
typedef int (*pnmpi_int_MPI_Finalized_t)(int *flag);
typedef int (*pnmpi_int_MPI_Free_mem_t)(void *base);
typedef int (*pnmpi_int_MPI_Gather_t)(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm);
typedef int (*pnmpi_int_MPI_Gatherv_t)(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, int root, MPI_Comm comm);
typedef int (*pnmpi_int_MPI_Get_t)(void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Win win);
typedef int (*pnmpi_int_MPI_Get_accumulate_t)(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, void *result_addr, int result_count, MPI_Datatype result_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win);
typedef int (*pnmpi_int_MPI_Get_address_t)(const void *location, MPI_Aint *address);
typedef int (*pnmpi_int_MPI_Get_count_t)(const MPI_Status *status, MPI_Datatype datatype, int *count);
typedef int (*pnmpi_int_MPI_Get_elements_t)(const MPI_Status *status, MPI_Datatype datatype, int *count);
typedef int (*pnmpi_int_MPI_Get_elements_x_t)(const MPI_Status *status, MPI_Datatype datatype, MPI_Count *count);
typedef int (*pnmpi_int_MPI_Get_library_version_t)(char *version, int *resultlen);
typedef int (*pnmpi_int_MPI_Get_processor_name_t)(char *name, int *resultlen);
typedef int (*pnmpi_int_MPI_Get_version_t)(int *version, int *subversion);
typedef int (*pnmpi_int_MPI_Graph_create_t)(MPI_Comm comm_old, int nnodes, const int index[], const int edges[], int reorder, MPI_Comm *comm_graph);
typedef int (*pnmpi_int_MPI_Graph_get_t)(MPI_Comm comm, int maxindex, int maxedges, int index[], int edges[]);
typedef int (*pnmpi_int_MPI_Graph_map_t)(MPI_Comm comm, int nnodes, const int index[], const int edges[], int *newrank);
typedef int (*pnmpi_int_MPI_Graph_neighbors_t)(MPI_Comm comm, int rank, int maxneighbors, int neighbors[]);
typedef int (*pnmpi_int_MPI_Graph_neighbors_count_t)(MPI_Comm comm, int rank, int *nneighbors);
typedef int (*pnmpi_int_MPI_Graphdims_get_t)(MPI_Comm comm, int *nnodes, int *nedges);
typedef int (*pnmpi_int_MPI_Grequest_complete_t)(MPI_Request request);
typedef int (*pnmpi_int_MPI_Grequest_start_t)(MPI_Grequest_query_function *query_fn, MPI_Grequest_free_function *free_fn, MPI_Grequest_cancel_function *cancel_fn, void *extra_state, MPI_Request *request);
typedef int (*pnmpi_int_MPI_Group_compare_t)(MPI_Group group1, MPI_Group group2, int *result);
typedef int (*pnmpi_int_MPI_Group_difference_t)(MPI_Group group1, MPI_Group group2, MPI_Group *newgroup);
typedef int (*pnmpi_int_MPI_Group_excl_t)(MPI_Group group, int n, const int ranks[], MPI_Group *newgroup);
typedef int (*pnmpi_int_MPI_Group_free_t)(MPI_Group *group);
typedef int (*pnmpi_int_MPI_Group_incl_t)(MPI_Group group, int n, const int ranks[], MPI_Group *newgroup);
typedef int (*pnmpi_int_MPI_Group_intersection_t)(MPI_Group group1, MPI_Group group2, MPI_Group *newgroup);
typedef int (*pnmpi_int_MPI_Group_range_excl_t)(MPI_Group group, int n, int ranges[][3], MPI_Group *newgroup);
typedef int (*pnmpi_int_MPI_Group_range_incl_t)(MPI_Group group, int n, int ranges[][3], MPI_Group *newgroup);
typedef int (*pnmpi_int_MPI_Group_rank_t)(MPI_Group group, int *rank);
typedef int (*pnmpi_int_MPI_Group_size_t)(MPI_Group group, int *size);
typedef int (*pnmpi_int_MPI_Group_translate_ranks_t)(MPI_Group group1, int n, const int ranks1[], MPI_Group group2, int ranks2[]);
typedef int (*pnmpi_int_MPI_Group_union_t)(MPI_Group group1, MPI_Group group2, MPI_Group *newgroup);
typedef int (*pnmpi_int_MPI_Iallgather_t)(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);
typedef int (*pnmpi_int_MPI_Iallgatherv_t)(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);
typedef int (*pnmpi_int_MPI_Iallreduce_t)(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request);
typedef int (*pnmpi_int_MPI_Ialltoall_t)(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);
typedef int (*pnmpi_int_MPI_Ialltoallv_t)(const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);
typedef int (*pnmpi_int_MPI_Ialltoallw_t)(const void *sendbuf, const int sendcounts[], const int sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const int rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm, MPI_Request *request);
typedef int (*pnmpi_int_MPI_Ibarrier_t)(MPI_Comm comm, MPI_Request *request);
typedef int (*pnmpi_int_MPI_Ibcast_t)(void *buffer, int count, MPI_Datatype datatype, int root, MPI_Comm comm, MPI_Request *request);
typedef int (*pnmpi_int_MPI_Ibsend_t)(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);
typedef int (*pnmpi_int_MPI_Iexscan_t)(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request);
typedef int (*pnmpi_int_MPI_Igather_t)(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request);
typedef int (*pnmpi_int_MPI_Igatherv_t)(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request);
typedef int (*pnmpi_int_MPI_Improbe_t)(int source, int tag, MPI_Comm comm, int *flag, MPI_Message *message, MPI_Status *status);
typedef int (*pnmpi_int_MPI_Imrecv_t)(void *buf, int count, MPI_Datatype type, MPI_Message *message, MPI_Request *request);
typedef int (*pnmpi_int_MPI_Ineighbor_allgather_t)(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);
typedef int (*pnmpi_int_MPI_Ineighbor_allgatherv_t)(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);
typedef int (*pnmpi_int_MPI_Ineighbor_alltoall_t)(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);
typedef int (*pnmpi_int_MPI_Ineighbor_alltoallv_t)(const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm, MPI_Request *request);
typedef int (*pnmpi_int_MPI_Ineighbor_alltoallw_t)(const void *sendbuf, const int sendcounts[], const MPI_Aint sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const MPI_Aint rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm, MPI_Request *request);
typedef int (*pnmpi_int_MPI_Info_create_t)(MPI_Info *info);
typedef int (*pnmpi_int_MPI_Info_delete_t)(MPI_Info info, const char *key);
typedef int (*pnmpi_int_MPI_Info_dup_t)(MPI_Info info, MPI_Info *newinfo);
typedef int (*pnmpi_int_MPI_Info_free_t)(MPI_Info *info);
typedef int (*pnmpi_int_MPI_Info_get_t)(MPI_Info info, const char *key, int valuelen, char *value, int *flag);
typedef int (*pnmpi_int_MPI_Info_get_nkeys_t)(MPI_Info info, int *nkeys);
typedef int (*pnmpi_int_MPI_Info_get_nthkey_t)(MPI_Info info, int n, char *key);
typedef int (*pnmpi_int_MPI_Info_get_valuelen_t)(MPI_Info info, const char *key, int *valuelen, int *flag);
typedef int (*pnmpi_int_MPI_Info_set_t)(MPI_Info info, const char *key, const char *value);
typedef int (*pnmpi_int_MPI_Init_t)(int *argc, char ***argv);
typedef int (*pnmpi_int_MPI_Init_thread_t)(int *argc, char ***argv, int required, int *provided);
typedef int (*pnmpi_int_MPI_Initialized_t)(int *flag);
typedef int (*pnmpi_int_MPI_Intercomm_create_t)(MPI_Comm local_comm, int local_leader, MPI_Comm bridge_comm, int remote_leader, int tag, MPI_Comm *newintercomm);
typedef int (*pnmpi_int_MPI_Intercomm_merge_t)(MPI_Comm intercomm, int high, MPI_Comm *newintercomm);
typedef int (*pnmpi_int_MPI_Iprobe_t)(int source, int tag, MPI_Comm comm, int *flag, MPI_Status *status);
typedef int (*pnmpi_int_MPI_Irecv_t)(void *buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Request *request);
typedef int (*pnmpi_int_MPI_Ireduce_t)(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, int root, MPI_Comm comm, MPI_Request *request);
typedef int (*pnmpi_int_MPI_Ireduce_scatter_t)(const void *sendbuf, void *recvbuf, const int recvcounts[], MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request);
typedef int (*pnmpi_int_MPI_Ireduce_scatter_block_t)(const void *sendbuf, void *recvbuf, int recvcount, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request);
typedef int (*pnmpi_int_MPI_Irsend_t)(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);
typedef int (*pnmpi_int_MPI_Is_thread_main_t)(int *flag);
typedef int (*pnmpi_int_MPI_Iscan_t)(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm, MPI_Request *request);
typedef int (*pnmpi_int_MPI_Iscatter_t)(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request);
typedef int (*pnmpi_int_MPI_Iscatterv_t)(const void *sendbuf, const int sendcounts[], const int displs[], MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm, MPI_Request *request);
typedef int (*pnmpi_int_MPI_Isend_t)(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);
typedef int (*pnmpi_int_MPI_Issend_t)(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);
typedef int (*pnmpi_int_MPI_Keyval_create_t)(MPI_Copy_function *copy_fn, MPI_Delete_function *delete_fn, int *keyval, void *extra_state);
typedef int (*pnmpi_int_MPI_Keyval_free_t)(int *keyval);
typedef int (*pnmpi_int_MPI_Lookup_name_t)(const char *service_name, MPI_Info info, char *port_name);
typedef int (*pnmpi_int_MPI_Mprobe_t)(int source, int tag, MPI_Comm comm, MPI_Message *message, MPI_Status *status);
typedef int (*pnmpi_int_MPI_Mrecv_t)(void *buf, int count, MPI_Datatype type, MPI_Message *message, MPI_Status *status);
typedef int (*pnmpi_int_MPI_Neighbor_allgather_t)(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm);
typedef int (*pnmpi_int_MPI_Neighbor_allgatherv_t)(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int displs[], MPI_Datatype recvtype, MPI_Comm comm);
typedef int (*pnmpi_int_MPI_Neighbor_alltoall_t)(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, MPI_Comm comm);
typedef int (*pnmpi_int_MPI_Neighbor_alltoallv_t)(const void *sendbuf, const int sendcounts[], const int sdispls[], MPI_Datatype sendtype, void *recvbuf, const int recvcounts[], const int rdispls[], MPI_Datatype recvtype, MPI_Comm comm);
typedef int (*pnmpi_int_MPI_Neighbor_alltoallw_t)(const void *sendbuf, const int sendcounts[], const MPI_Aint sdispls[], const MPI_Datatype sendtypes[], void *recvbuf, const int recvcounts[], const MPI_Aint rdispls[], const MPI_Datatype recvtypes[], MPI_Comm comm);
typedef int (*pnmpi_int_MPI_Op_commutative_t)(MPI_Op op, int *commute);
typedef int (*pnmpi_int_MPI_Op_create_t)(MPI_User_function *function, int commute, MPI_Op *op);
typedef int (*pnmpi_int_MPI_Op_free_t)(MPI_Op *op);
typedef int (*pnmpi_int_MPI_Open_port_t)(MPI_Info info, char *port_name);
typedef int (*pnmpi_int_MPI_Pack_t)(const void *inbuf, int incount, MPI_Datatype datatype, void *outbuf, int outsize, int *position, MPI_Comm comm);
typedef int (*pnmpi_int_MPI_Pack_external_t)(const char datarep[], const void *inbuf, int incount, MPI_Datatype datatype, void *outbuf, MPI_Aint outsize, MPI_Aint *position);
typedef int (*pnmpi_int_MPI_Pack_external_size_t)(const char datarep[], int incount, MPI_Datatype datatype, MPI_Aint *size);
typedef int (*pnmpi_int_MPI_Pack_size_t)(int incount, MPI_Datatype datatype, MPI_Comm comm, int *size);
typedef int (*pnmpi_int_MPI_Probe_t)(int source, int tag, MPI_Comm comm, MPI_Status *status);
typedef int (*pnmpi_int_MPI_Publish_name_t)(const char *service_name, MPI_Info info, const char *port_name);
typedef int (*pnmpi_int_MPI_Put_t)(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Win win);
typedef int (*pnmpi_int_MPI_Query_thread_t)(int *provided);
typedef int (*pnmpi_int_MPI_Raccumulate_t)(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win, MPI_Request *request);
typedef int (*pnmpi_int_MPI_Recv_t)(void *buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Status *status);
typedef int (*pnmpi_int_MPI_Recv_init_t)(void *buf, int count, MPI_Datatype datatype, int source, int tag, MPI_Comm comm, MPI_Request *request);
typedef int (*pnmpi_int_MPI_Reduce_t)(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, int root, MPI_Comm comm);
typedef int (*pnmpi_int_MPI_Reduce_local_t)(const void *inbuf, void *inoutbuf, int count, MPI_Datatype datatype, MPI_Op op);
typedef int (*pnmpi_int_MPI_Reduce_scatter_t)(const void *sendbuf, void *recvbuf, const int recvcounts[], MPI_Datatype datatype, MPI_Op op, MPI_Comm comm);
typedef int (*pnmpi_int_MPI_Reduce_scatter_block_t)(const void *sendbuf, void *recvbuf, int recvcount, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm);
typedef int (*pnmpi_int_MPI_Register_datarep_t)(const char *datarep, MPI_Datarep_conversion_function *read_conversion_fn, MPI_Datarep_conversion_function *write_conversion_fn, MPI_Datarep_extent_function *dtype_file_extent_fn, void *extra_state);
typedef int (*pnmpi_int_MPI_Request_free_t)(MPI_Request *request);
typedef int (*pnmpi_int_MPI_Request_get_status_t)(MPI_Request request, int *flag, MPI_Status *status);
typedef int (*pnmpi_int_MPI_Rget_t)(void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Win win, MPI_Request *request);
typedef int (*pnmpi_int_MPI_Rget_accumulate_t)(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, void *result_addr, int result_count, MPI_Datatype result_datatype, int target_rank, MPI_Aint target_disp, int target_count, MPI_Datatype target_datatype, MPI_Op op, MPI_Win win, MPI_Request *request);
typedef int (*pnmpi_int_MPI_Rput_t)(const void *origin_addr, int origin_count, MPI_Datatype origin_datatype, int target_rank, MPI_Aint target_disp, int target_cout, MPI_Datatype target_datatype, MPI_Win win, MPI_Request *request);
typedef int (*pnmpi_int_MPI_Rsend_t)(const void *ibuf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm);
typedef int (*pnmpi_int_MPI_Rsend_init_t)(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);
typedef int (*pnmpi_int_MPI_Scan_t)(const void *sendbuf, void *recvbuf, int count, MPI_Datatype datatype, MPI_Op op, MPI_Comm comm);
typedef int (*pnmpi_int_MPI_Scatter_t)(const void *sendbuf, int sendcount, MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm);
typedef int (*pnmpi_int_MPI_Scatterv_t)(const void *sendbuf, const int sendcounts[], const int displs[], MPI_Datatype sendtype, void *recvbuf, int recvcount, MPI_Datatype recvtype, int root, MPI_Comm comm);
typedef int (*pnmpi_int_MPI_Send_t)(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm);
typedef int (*pnmpi_int_MPI_Send_init_t)(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);
typedef int (*pnmpi_int_MPI_Sendrecv_t)(const void *sendbuf, int sendcount, MPI_Datatype sendtype, int dest, int sendtag, void *recvbuf, int recvcount, MPI_Datatype recvtype, int source, int recvtag, MPI_Comm comm, MPI_Status *status);
typedef int (*pnmpi_int_MPI_Sendrecv_replace_t)(void *buf, int count, MPI_Datatype datatype, int dest, int sendtag, int source, int recvtag, MPI_Comm comm, MPI_Status *status);
typedef int (*pnmpi_int_MPI_Ssend_t)(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm);
typedef int (*pnmpi_int_MPI_Ssend_init_t)(const void *buf, int count, MPI_Datatype datatype, int dest, int tag, MPI_Comm comm, MPI_Request *request);
typedef int (*pnmpi_int_MPI_Start_t)(MPI_Request *request);
typedef int (*pnmpi_int_MPI_Startall_t)(int count, MPI_Request array_of_requests[]);
typedef int (*pnmpi_int_MPI_Status_set_cancelled_t)(MPI_Status *status, int flag);
typedef int (*pnmpi_int_MPI_Status_set_elements_t)(MPI_Status *status, MPI_Datatype datatype, int count);
typedef int (*pnmpi_int_MPI_Status_set_elements_x_t)(MPI_Status *status, MPI_Datatype datatype, MPI_Count count);
typedef int (*pnmpi_int_MPI_Test_t)(MPI_Request *request, int *flag, MPI_Status *status);
typedef int (*pnmpi_int_MPI_Test_cancelled_t)(const MPI_Status *status, int *flag);
typedef int (*pnmpi_int_MPI_Testall_t)(int count, MPI_Request array_of_requests[], int *flag, MPI_Status array_of_statuses[]);
typedef int (*pnmpi_int_MPI_Testany_t)(int count, MPI_Request array_of_requests[], int *index, int *flag, MPI_Status *status);
typedef int (*pnmpi_int_MPI_Testsome_t)(int incount, MPI_Request array_of_requests[], int *outcount, int array_of_indices[], MPI_Status array_of_statuses[]);
typedef int (*pnmpi_int_MPI_Topo_test_t)(MPI_Comm comm, int *status);
typedef int (*pnmpi_int_MPI_Type_commit_t)(MPI_Datatype *type);
typedef int (*pnmpi_int_MPI_Type_contiguous_t)(int count, MPI_Datatype oldtype, MPI_Datatype *newtype);
typedef int (*pnmpi_int_MPI_Type_create_darray_t)(int size, int rank, int ndims, const int gsize_array[], const int distrib_array[], const int darg_array[], const int psize_array[], int order, MPI_Datatype oldtype, MPI_Datatype *newtype);
typedef int (*pnmpi_int_MPI_Type_create_f90_complex_t)(int p, int r, MPI_Datatype *newtype);
typedef int (*pnmpi_int_MPI_Type_create_f90_integer_t)(int r, MPI_Datatype *newtype);
typedef int (*pnmpi_int_MPI_Type_create_f90_real_t)(int p, int r, MPI_Datatype *newtype);
typedef int (*pnmpi_int_MPI_Type_create_hindexed_t)(int count, const int array_of_blocklengths[], const MPI_Aint array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype);
typedef int (*pnmpi_int_MPI_Type_create_hindexed_block_t)(int count, int blocklength, const MPI_Aint array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype);
typedef int (*pnmpi_int_MPI_Type_create_hvector_t)(int count, int blocklength, MPI_Aint stride, MPI_Datatype oldtype, MPI_Datatype *newtype);
typedef int (*pnmpi_int_MPI_Type_create_indexed_block_t)(int count, int blocklength, const int array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype);
typedef int (*pnmpi_int_MPI_Type_create_keyval_t)(MPI_Type_copy_attr_function *type_copy_attr_fn, MPI_Type_delete_attr_function *type_delete_attr_fn, int *type_keyval, void *extra_state);
typedef int (*pnmpi_int_MPI_Type_create_resized_t)(MPI_Datatype oldtype, MPI_Aint lb, MPI_Aint extent, MPI_Datatype *newtype);
typedef int (*pnmpi_int_MPI_Type_create_struct_t)(int count, const int array_of_block_lengths[], const MPI_Aint array_of_displacements[], const MPI_Datatype array_of_types[], MPI_Datatype *newtype);
typedef int (*pnmpi_int_MPI_Type_create_subarray_t)(int ndims, const int size_array[], const int subsize_array[], const int start_array[], int order, MPI_Datatype oldtype, MPI_Datatype *newtype);
typedef int (*pnmpi_int_MPI_Type_delete_attr_t)(MPI_Datatype type, int type_keyval);
typedef int (*pnmpi_int_MPI_Type_dup_t)(MPI_Datatype type, MPI_Datatype *newtype);
typedef int (*pnmpi_int_MPI_Type_extent_t)(MPI_Datatype type, MPI_Aint *extent);
typedef int (*pnmpi_int_MPI_Type_free_t)(MPI_Datatype *type);
typedef int (*pnmpi_int_MPI_Type_free_keyval_t)(int *type_keyval);
typedef int (*pnmpi_int_MPI_Type_get_attr_t)(MPI_Datatype type, int type_keyval, void *attribute_val, int *flag);
typedef int (*pnmpi_int_MPI_Type_get_contents_t)(MPI_Datatype mtype, int max_integers, int max_addresses, int max_datatypes, int array_of_integers[], MPI_Aint array_of_addresses[], MPI_Datatype array_of_datatypes[]);
typedef int (*pnmpi_int_MPI_Type_get_envelope_t)(MPI_Datatype type, int *num_integers, int *num_addresses, int *num_datatypes, int *combiner);
typedef int (*pnmpi_int_MPI_Type_get_extent_t)(MPI_Datatype type, MPI_Aint *lb, MPI_Aint *extent);
typedef int (*pnmpi_int_MPI_Type_get_extent_x_t)(MPI_Datatype type, MPI_Count *lb, MPI_Count *extent);
typedef int (*pnmpi_int_MPI_Type_get_name_t)(MPI_Datatype type, char *type_name, int *resultlen);
typedef int (*pnmpi_int_MPI_Type_get_true_extent_t)(MPI_Datatype datatype, MPI_Aint *true_lb, MPI_Aint *true_extent);
typedef int (*pnmpi_int_MPI_Type_get_true_extent_x_t)(MPI_Datatype datatype, MPI_Count *true_lb, MPI_Count *true_extent);
typedef int (*pnmpi_int_MPI_Type_hindexed_t)(int count, int array_of_blocklengths[], MPI_Aint array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype);
typedef int (*pnmpi_int_MPI_Type_hvector_t)(int count, int blocklength, MPI_Aint stride, MPI_Datatype oldtype, MPI_Datatype *newtype);
typedef int (*pnmpi_int_MPI_Type_indexed_t)(int count, const int array_of_blocklengths[], const int array_of_displacements[], MPI_Datatype oldtype, MPI_Datatype *newtype);
typedef int (*pnmpi_int_MPI_Type_lb_t)(MPI_Datatype type, MPI_Aint *lb);
typedef int (*pnmpi_int_MPI_Type_match_size_t)(int typeclass, int size, MPI_Datatype *type);
typedef int (*pnmpi_int_MPI_Type_set_attr_t)(MPI_Datatype type, int type_keyval, void *attr_val);
typedef int (*pnmpi_int_MPI_Type_set_name_t)(MPI_Datatype type, const char *type_name);
typedef int (*pnmpi_int_MPI_Type_size_t)(MPI_Datatype type, int *size);
typedef int (*pnmpi_int_MPI_Type_size_x_t)(MPI_Datatype type, MPI_Count *size);
typedef int (*pnmpi_int_MPI_Type_struct_t)(int count, int array_of_blocklengths[], MPI_Aint array_of_displacements[], MPI_Datatype array_of_types[], MPI_Datatype *newtype);
typedef int (*pnmpi_int_MPI_Type_ub_t)(MPI_Datatype mtype, MPI_Aint *ub);
typedef int (*pnmpi_int_MPI_Type_vector_t)(int count, int blocklength, int stride, MPI_Datatype oldtype, MPI_Datatype *newtype);
typedef int (*pnmpi_int_MPI_Unpack_t)(const void *inbuf, int insize, int *position, void *outbuf, int outcount, MPI_Datatype datatype, MPI_Comm comm);
typedef int (*pnmpi_int_MPI_Unpack_external_t)(const char datarep[], const void *inbuf, MPI_Aint insize, MPI_Aint *position, void *outbuf, int outcount, MPI_Datatype datatype);
typedef int (*pnmpi_int_MPI_Unpublish_name_t)(const char *service_name, MPI_Info info, const char *port_name);
typedef int (*pnmpi_int_MPI_Wait_t)(MPI_Request *request, MPI_Status *status);
typedef int (*pnmpi_int_MPI_Waitall_t)(int count, MPI_Request array_of_requests[], MPI_Status *array_of_statuses);
typedef int (*pnmpi_int_MPI_Waitany_t)(int count, MPI_Request array_of_requests[], int *index, MPI_Status *status);
typedef int (*pnmpi_int_MPI_Waitsome_t)(int incount, MPI_Request array_of_requests[], int *outcount, int array_of_indices[], MPI_Status array_of_statuses[]);
typedef int (*pnmpi_int_MPI_Win_allocate_t)(MPI_Aint size, int disp_unit, MPI_Info info, MPI_Comm comm, void *baseptr, MPI_Win *win);
typedef int (*pnmpi_int_MPI_Win_allocate_shared_t)(MPI_Aint size, int disp_unit, MPI_Info info, MPI_Comm comm, void *baseptr, MPI_Win *win);
typedef int (*pnmpi_int_MPI_Win_attach_t)(MPI_Win win, void *base, MPI_Aint size);
typedef int (*pnmpi_int_MPI_Win_call_errhandler_t)(MPI_Win win, int errorcode);
typedef int (*pnmpi_int_MPI_Win_complete_t)(MPI_Win win);
typedef int (*pnmpi_int_MPI_Win_create_t)(void *base, MPI_Aint size, int disp_unit, MPI_Info info, MPI_Comm comm, MPI_Win *win);
typedef int (*pnmpi_int_MPI_Win_create_dynamic_t)(MPI_Info info, MPI_Comm comm, MPI_Win *win);
typedef int (*pnmpi_int_MPI_Win_create_errhandler_t)(MPI_Win_errhandler_function *function, MPI_Errhandler *errhandler);
typedef int (*pnmpi_int_MPI_Win_create_keyval_t)(MPI_Win_copy_attr_function *win_copy_attr_fn, MPI_Win_delete_attr_function *win_delete_attr_fn, int *win_keyval, void *extra_state);
typedef int (*pnmpi_int_MPI_Win_delete_attr_t)(MPI_Win win, int win_keyval);
typedef int (*pnmpi_int_MPI_Win_detach_t)(MPI_Win win, const void *base);
typedef int (*pnmpi_int_MPI_Win_fence_t)(int assert, MPI_Win win);
typedef int (*pnmpi_int_MPI_Win_flush_t)(int rank, MPI_Win win);
typedef int (*pnmpi_int_MPI_Win_flush_all_t)(MPI_Win win);
typedef int (*pnmpi_int_MPI_Win_flush_local_t)(int rank, MPI_Win win);
typedef int (*pnmpi_int_MPI_Win_flush_local_all_t)(MPI_Win win);
typedef int (*pnmpi_int_MPI_Win_free_t)(MPI_Win *win);
typedef int (*pnmpi_int_MPI_Win_free_keyval_t)(int *win_keyval);
typedef int (*pnmpi_int_MPI_Win_get_attr_t)(MPI_Win win, int win_keyval, void *attribute_val, int *flag);
typedef int (*pnmpi_int_MPI_Win_get_errhandler_t)(MPI_Win win, MPI_Errhandler *errhandler);
typedef int (*pnmpi_int_MPI_Win_get_group_t)(MPI_Win win, MPI_Group *group);
typedef int (*pnmpi_int_MPI_Win_get_info_t)(MPI_Win win, MPI_Info *info_used);
typedef int (*pnmpi_int_MPI_Win_get_name_t)(MPI_Win win, char *win_name, int *resultlen);
typedef int (*pnmpi_int_MPI_Win_lock_t)(int lock_type, int rank, int assert, MPI_Win win);
typedef int (*pnmpi_int_MPI_Win_lock_all_t)(int assert, MPI_Win win);
typedef int (*pnmpi_int_MPI_Win_post_t)(MPI_Group group, int assert, MPI_Win win);
typedef int (*pnmpi_int_MPI_Win_set_attr_t)(MPI_Win win, int win_keyval, void *attribute_val);
typedef int (*pnmpi_int_MPI_Win_set_errhandler_t)(MPI_Win win, MPI_Errhandler errhandler);
typedef int (*pnmpi_int_MPI_Win_set_info_t)(MPI_Win win, MPI_Info info);
typedef int (*pnmpi_int_MPI_Win_set_name_t)(MPI_Win win, const char *win_name);
typedef int (*pnmpi_int_MPI_Win_shared_query_t)(MPI_Win win, int rank, MPI_Aint *size, int *disp_unit, void *baseptr);
typedef int (*pnmpi_int_MPI_Win_start_t)(MPI_Group group, int assert, MPI_Win win);
typedef int (*pnmpi_int_MPI_Win_sync_t)(MPI_Win win);
typedef int (*pnmpi_int_MPI_Win_test_t)(MPI_Win win, int *flag);
typedef int (*pnmpi_int_MPI_Win_unlock_t)(int rank, MPI_Win win);
typedef int (*pnmpi_int_MPI_Win_unlock_all_t)(MPI_Win win);
typedef int (*pnmpi_int_MPI_Win_wait_t)(MPI_Win win);
typedef double (*pnmpi_int_MPI_Wtick_t)();
typedef double (*pnmpi_int_MPI_Wtime_t)();

typedef struct pnmpi_functions_d
{
  pnmpi_int_MPI_Abort_t *pnmpi_int_MPI_Abort;
  pnmpi_int_MPI_Accumulate_t *pnmpi_int_MPI_Accumulate;
  pnmpi_int_MPI_Add_error_class_t *pnmpi_int_MPI_Add_error_class;
  pnmpi_int_MPI_Add_error_code_t *pnmpi_int_MPI_Add_error_code;
  pnmpi_int_MPI_Add_error_string_t *pnmpi_int_MPI_Add_error_string;
  pnmpi_int_MPI_Address_t *pnmpi_int_MPI_Address;
  pnmpi_int_MPI_Allgather_t *pnmpi_int_MPI_Allgather;
  pnmpi_int_MPI_Allgatherv_t *pnmpi_int_MPI_Allgatherv;
  pnmpi_int_MPI_Alloc_mem_t *pnmpi_int_MPI_Alloc_mem;
  pnmpi_int_MPI_Allreduce_t *pnmpi_int_MPI_Allreduce;
  pnmpi_int_MPI_Alltoall_t *pnmpi_int_MPI_Alltoall;
  pnmpi_int_MPI_Alltoallv_t *pnmpi_int_MPI_Alltoallv;
  pnmpi_int_MPI_Alltoallw_t *pnmpi_int_MPI_Alltoallw;
  pnmpi_int_MPI_Attr_delete_t *pnmpi_int_MPI_Attr_delete;
  pnmpi_int_MPI_Attr_get_t *pnmpi_int_MPI_Attr_get;
  pnmpi_int_MPI_Attr_put_t *pnmpi_int_MPI_Attr_put;
  pnmpi_int_MPI_Barrier_t *pnmpi_int_MPI_Barrier;
  pnmpi_int_MPI_Bcast_t *pnmpi_int_MPI_Bcast;
  pnmpi_int_MPI_Bsend_t *pnmpi_int_MPI_Bsend;
  pnmpi_int_MPI_Bsend_init_t *pnmpi_int_MPI_Bsend_init;
  pnmpi_int_MPI_Buffer_attach_t *pnmpi_int_MPI_Buffer_attach;
  pnmpi_int_MPI_Buffer_detach_t *pnmpi_int_MPI_Buffer_detach;
  pnmpi_int_MPI_Cancel_t *pnmpi_int_MPI_Cancel;
  pnmpi_int_MPI_Cart_coords_t *pnmpi_int_MPI_Cart_coords;
  pnmpi_int_MPI_Cart_create_t *pnmpi_int_MPI_Cart_create;
  pnmpi_int_MPI_Cart_get_t *pnmpi_int_MPI_Cart_get;
  pnmpi_int_MPI_Cart_map_t *pnmpi_int_MPI_Cart_map;
  pnmpi_int_MPI_Cart_rank_t *pnmpi_int_MPI_Cart_rank;
  pnmpi_int_MPI_Cart_shift_t *pnmpi_int_MPI_Cart_shift;
  pnmpi_int_MPI_Cart_sub_t *pnmpi_int_MPI_Cart_sub;
  pnmpi_int_MPI_Cartdim_get_t *pnmpi_int_MPI_Cartdim_get;
  pnmpi_int_MPI_Close_port_t *pnmpi_int_MPI_Close_port;
  pnmpi_int_MPI_Comm_accept_t *pnmpi_int_MPI_Comm_accept;
  pnmpi_int_MPI_Comm_call_errhandler_t *pnmpi_int_MPI_Comm_call_errhandler;
  pnmpi_int_MPI_Comm_compare_t *pnmpi_int_MPI_Comm_compare;
  pnmpi_int_MPI_Comm_connect_t *pnmpi_int_MPI_Comm_connect;
  pnmpi_int_MPI_Comm_create_t *pnmpi_int_MPI_Comm_create;
  pnmpi_int_MPI_Comm_create_errhandler_t *pnmpi_int_MPI_Comm_create_errhandler;
  pnmpi_int_MPI_Comm_create_group_t *pnmpi_int_MPI_Comm_create_group;
  pnmpi_int_MPI_Comm_create_keyval_t *pnmpi_int_MPI_Comm_create_keyval;
  pnmpi_int_MPI_Comm_delete_attr_t *pnmpi_int_MPI_Comm_delete_attr;
  pnmpi_int_MPI_Comm_disconnect_t *pnmpi_int_MPI_Comm_disconnect;
  pnmpi_int_MPI_Comm_dup_t *pnmpi_int_MPI_Comm_dup;
  pnmpi_int_MPI_Comm_dup_with_info_t *pnmpi_int_MPI_Comm_dup_with_info;
  pnmpi_int_MPI_Comm_free_t *pnmpi_int_MPI_Comm_free;
  pnmpi_int_MPI_Comm_free_keyval_t *pnmpi_int_MPI_Comm_free_keyval;
  pnmpi_int_MPI_Comm_get_attr_t *pnmpi_int_MPI_Comm_get_attr;
  pnmpi_int_MPI_Comm_get_errhandler_t *pnmpi_int_MPI_Comm_get_errhandler;
  pnmpi_int_MPI_Comm_get_info_t *pnmpi_int_MPI_Comm_get_info;
  pnmpi_int_MPI_Comm_get_name_t *pnmpi_int_MPI_Comm_get_name;
  pnmpi_int_MPI_Comm_get_parent_t *pnmpi_int_MPI_Comm_get_parent;
  pnmpi_int_MPI_Comm_group_t *pnmpi_int_MPI_Comm_group;
  pnmpi_int_MPI_Comm_idup_t *pnmpi_int_MPI_Comm_idup;
  pnmpi_int_MPI_Comm_join_t *pnmpi_int_MPI_Comm_join;
  pnmpi_int_MPI_Comm_rank_t *pnmpi_int_MPI_Comm_rank;
  pnmpi_int_MPI_Comm_remote_group_t *pnmpi_int_MPI_Comm_remote_group;
  pnmpi_int_MPI_Comm_remote_size_t *pnmpi_int_MPI_Comm_remote_size;
  pnmpi_int_MPI_Comm_set_attr_t *pnmpi_int_MPI_Comm_set_attr;
  pnmpi_int_MPI_Comm_set_errhandler_t *pnmpi_int_MPI_Comm_set_errhandler;
  pnmpi_int_MPI_Comm_set_info_t *pnmpi_int_MPI_Comm_set_info;
  pnmpi_int_MPI_Comm_set_name_t *pnmpi_int_MPI_Comm_set_name;
  pnmpi_int_MPI_Comm_size_t *pnmpi_int_MPI_Comm_size;
  pnmpi_int_MPI_Comm_split_t *pnmpi_int_MPI_Comm_split;
  pnmpi_int_MPI_Comm_split_type_t *pnmpi_int_MPI_Comm_split_type;
  pnmpi_int_MPI_Comm_test_inter_t *pnmpi_int_MPI_Comm_test_inter;
  pnmpi_int_MPI_Compare_and_swap_t *pnmpi_int_MPI_Compare_and_swap;
  pnmpi_int_MPI_Dims_create_t *pnmpi_int_MPI_Dims_create;
  pnmpi_int_MPI_Dist_graph_create_t *pnmpi_int_MPI_Dist_graph_create;
  pnmpi_int_MPI_Dist_graph_create_adjacent_t *pnmpi_int_MPI_Dist_graph_create_adjacent;
  pnmpi_int_MPI_Dist_graph_neighbors_t *pnmpi_int_MPI_Dist_graph_neighbors;
  pnmpi_int_MPI_Dist_graph_neighbors_count_t *pnmpi_int_MPI_Dist_graph_neighbors_count;
  pnmpi_int_MPI_Errhandler_create_t *pnmpi_int_MPI_Errhandler_create;
  pnmpi_int_MPI_Errhandler_free_t *pnmpi_int_MPI_Errhandler_free;
  pnmpi_int_MPI_Errhandler_get_t *pnmpi_int_MPI_Errhandler_get;
  pnmpi_int_MPI_Errhandler_set_t *pnmpi_int_MPI_Errhandler_set;
  pnmpi_int_MPI_Error_class_t *pnmpi_int_MPI_Error_class;
  pnmpi_int_MPI_Error_string_t *pnmpi_int_MPI_Error_string;
  pnmpi_int_MPI_Exscan_t *pnmpi_int_MPI_Exscan;
  pnmpi_int_MPI_Fetch_and_op_t *pnmpi_int_MPI_Fetch_and_op;
  pnmpi_int_MPI_File_call_errhandler_t *pnmpi_int_MPI_File_call_errhandler;
  pnmpi_int_MPI_File_close_t *pnmpi_int_MPI_File_close;
  pnmpi_int_MPI_File_create_errhandler_t *pnmpi_int_MPI_File_create_errhandler;
  pnmpi_int_MPI_File_delete_t *pnmpi_int_MPI_File_delete;
  pnmpi_int_MPI_File_get_amode_t *pnmpi_int_MPI_File_get_amode;
  pnmpi_int_MPI_File_get_atomicity_t *pnmpi_int_MPI_File_get_atomicity;
  pnmpi_int_MPI_File_get_byte_offset_t *pnmpi_int_MPI_File_get_byte_offset;
  pnmpi_int_MPI_File_get_errhandler_t *pnmpi_int_MPI_File_get_errhandler;
  pnmpi_int_MPI_File_get_group_t *pnmpi_int_MPI_File_get_group;
  pnmpi_int_MPI_File_get_info_t *pnmpi_int_MPI_File_get_info;
  pnmpi_int_MPI_File_get_position_t *pnmpi_int_MPI_File_get_position;
  pnmpi_int_MPI_File_get_position_shared_t *pnmpi_int_MPI_File_get_position_shared;
  pnmpi_int_MPI_File_get_size_t *pnmpi_int_MPI_File_get_size;
  pnmpi_int_MPI_File_get_type_extent_t *pnmpi_int_MPI_File_get_type_extent;
  pnmpi_int_MPI_File_get_view_t *pnmpi_int_MPI_File_get_view;
  pnmpi_int_MPI_File_iread_t *pnmpi_int_MPI_File_iread;
  pnmpi_int_MPI_File_iread_all_t *pnmpi_int_MPI_File_iread_all;
  pnmpi_int_MPI_File_iread_at_t *pnmpi_int_MPI_File_iread_at;
  pnmpi_int_MPI_File_iread_at_all_t *pnmpi_int_MPI_File_iread_at_all;
  pnmpi_int_MPI_File_iread_shared_t *pnmpi_int_MPI_File_iread_shared;
  pnmpi_int_MPI_File_iwrite_t *pnmpi_int_MPI_File_iwrite;
  pnmpi_int_MPI_File_iwrite_all_t *pnmpi_int_MPI_File_iwrite_all;
  pnmpi_int_MPI_File_iwrite_at_t *pnmpi_int_MPI_File_iwrite_at;
  pnmpi_int_MPI_File_iwrite_at_all_t *pnmpi_int_MPI_File_iwrite_at_all;
  pnmpi_int_MPI_File_iwrite_shared_t *pnmpi_int_MPI_File_iwrite_shared;
  pnmpi_int_MPI_File_open_t *pnmpi_int_MPI_File_open;
  pnmpi_int_MPI_File_preallocate_t *pnmpi_int_MPI_File_preallocate;
  pnmpi_int_MPI_File_read_t *pnmpi_int_MPI_File_read;
  pnmpi_int_MPI_File_read_all_t *pnmpi_int_MPI_File_read_all;
  pnmpi_int_MPI_File_read_all_begin_t *pnmpi_int_MPI_File_read_all_begin;
  pnmpi_int_MPI_File_read_all_end_t *pnmpi_int_MPI_File_read_all_end;
  pnmpi_int_MPI_File_read_at_t *pnmpi_int_MPI_File_read_at;
  pnmpi_int_MPI_File_read_at_all_t *pnmpi_int_MPI_File_read_at_all;
  pnmpi_int_MPI_File_read_at_all_begin_t *pnmpi_int_MPI_File_read_at_all_begin;
  pnmpi_int_MPI_File_read_at_all_end_t *pnmpi_int_MPI_File_read_at_all_end;
  pnmpi_int_MPI_File_read_ordered_t *pnmpi_int_MPI_File_read_ordered;
  pnmpi_int_MPI_File_read_ordered_begin_t *pnmpi_int_MPI_File_read_ordered_begin;
  pnmpi_int_MPI_File_read_ordered_end_t *pnmpi_int_MPI_File_read_ordered_end;
  pnmpi_int_MPI_File_read_shared_t *pnmpi_int_MPI_File_read_shared;
  pnmpi_int_MPI_File_seek_t *pnmpi_int_MPI_File_seek;
  pnmpi_int_MPI_File_seek_shared_t *pnmpi_int_MPI_File_seek_shared;
  pnmpi_int_MPI_File_set_atomicity_t *pnmpi_int_MPI_File_set_atomicity;
  pnmpi_int_MPI_File_set_errhandler_t *pnmpi_int_MPI_File_set_errhandler;
  pnmpi_int_MPI_File_set_info_t *pnmpi_int_MPI_File_set_info;
  pnmpi_int_MPI_File_set_size_t *pnmpi_int_MPI_File_set_size;
  pnmpi_int_MPI_File_set_view_t *pnmpi_int_MPI_File_set_view;
  pnmpi_int_MPI_File_sync_t *pnmpi_int_MPI_File_sync;
  pnmpi_int_MPI_File_write_t *pnmpi_int_MPI_File_write;
  pnmpi_int_MPI_File_write_all_t *pnmpi_int_MPI_File_write_all;
  pnmpi_int_MPI_File_write_all_begin_t *pnmpi_int_MPI_File_write_all_begin;
  pnmpi_int_MPI_File_write_all_end_t *pnmpi_int_MPI_File_write_all_end;
  pnmpi_int_MPI_File_write_at_t *pnmpi_int_MPI_File_write_at;
  pnmpi_int_MPI_File_write_at_all_t *pnmpi_int_MPI_File_write_at_all;
  pnmpi_int_MPI_File_write_at_all_begin_t *pnmpi_int_MPI_File_write_at_all_begin;
  pnmpi_int_MPI_File_write_at_all_end_t *pnmpi_int_MPI_File_write_at_all_end;
  pnmpi_int_MPI_File_write_ordered_t *pnmpi_int_MPI_File_write_ordered;
  pnmpi_int_MPI_File_write_ordered_begin_t *pnmpi_int_MPI_File_write_ordered_begin;
  pnmpi_int_MPI_File_write_ordered_end_t *pnmpi_int_MPI_File_write_ordered_end;
  pnmpi_int_MPI_File_write_shared_t *pnmpi_int_MPI_File_write_shared;
  pnmpi_int_MPI_Finalize_t *pnmpi_int_MPI_Finalize;
  pnmpi_int_MPI_Finalized_t *pnmpi_int_MPI_Finalized;
  pnmpi_int_MPI_Free_mem_t *pnmpi_int_MPI_Free_mem;
  pnmpi_int_MPI_Gather_t *pnmpi_int_MPI_Gather;
  pnmpi_int_MPI_Gatherv_t *pnmpi_int_MPI_Gatherv;
  pnmpi_int_MPI_Get_t *pnmpi_int_MPI_Get;
  pnmpi_int_MPI_Get_accumulate_t *pnmpi_int_MPI_Get_accumulate;
  pnmpi_int_MPI_Get_address_t *pnmpi_int_MPI_Get_address;
  pnmpi_int_MPI_Get_count_t *pnmpi_int_MPI_Get_count;
  pnmpi_int_MPI_Get_elements_t *pnmpi_int_MPI_Get_elements;
  pnmpi_int_MPI_Get_elements_x_t *pnmpi_int_MPI_Get_elements_x;
  pnmpi_int_MPI_Get_library_version_t *pnmpi_int_MPI_Get_library_version;
  pnmpi_int_MPI_Get_processor_name_t *pnmpi_int_MPI_Get_processor_name;
  pnmpi_int_MPI_Get_version_t *pnmpi_int_MPI_Get_version;
  pnmpi_int_MPI_Graph_create_t *pnmpi_int_MPI_Graph_create;
  pnmpi_int_MPI_Graph_get_t *pnmpi_int_MPI_Graph_get;
  pnmpi_int_MPI_Graph_map_t *pnmpi_int_MPI_Graph_map;
  pnmpi_int_MPI_Graph_neighbors_t *pnmpi_int_MPI_Graph_neighbors;
  pnmpi_int_MPI_Graph_neighbors_count_t *pnmpi_int_MPI_Graph_neighbors_count;
  pnmpi_int_MPI_Graphdims_get_t *pnmpi_int_MPI_Graphdims_get;
  pnmpi_int_MPI_Grequest_complete_t *pnmpi_int_MPI_Grequest_complete;
  pnmpi_int_MPI_Grequest_start_t *pnmpi_int_MPI_Grequest_start;
  pnmpi_int_MPI_Group_compare_t *pnmpi_int_MPI_Group_compare;
  pnmpi_int_MPI_Group_difference_t *pnmpi_int_MPI_Group_difference;
  pnmpi_int_MPI_Group_excl_t *pnmpi_int_MPI_Group_excl;
  pnmpi_int_MPI_Group_free_t *pnmpi_int_MPI_Group_free;
  pnmpi_int_MPI_Group_incl_t *pnmpi_int_MPI_Group_incl;
  pnmpi_int_MPI_Group_intersection_t *pnmpi_int_MPI_Group_intersection;
  pnmpi_int_MPI_Group_range_excl_t *pnmpi_int_MPI_Group_range_excl;
  pnmpi_int_MPI_Group_range_incl_t *pnmpi_int_MPI_Group_range_incl;
  pnmpi_int_MPI_Group_rank_t *pnmpi_int_MPI_Group_rank;
  pnmpi_int_MPI_Group_size_t *pnmpi_int_MPI_Group_size;
  pnmpi_int_MPI_Group_translate_ranks_t *pnmpi_int_MPI_Group_translate_ranks;
  pnmpi_int_MPI_Group_union_t *pnmpi_int_MPI_Group_union;
  pnmpi_int_MPI_Iallgather_t *pnmpi_int_MPI_Iallgather;
  pnmpi_int_MPI_Iallgatherv_t *pnmpi_int_MPI_Iallgatherv;
  pnmpi_int_MPI_Iallreduce_t *pnmpi_int_MPI_Iallreduce;
  pnmpi_int_MPI_Ialltoall_t *pnmpi_int_MPI_Ialltoall;
  pnmpi_int_MPI_Ialltoallv_t *pnmpi_int_MPI_Ialltoallv;
  pnmpi_int_MPI_Ialltoallw_t *pnmpi_int_MPI_Ialltoallw;
  pnmpi_int_MPI_Ibarrier_t *pnmpi_int_MPI_Ibarrier;
  pnmpi_int_MPI_Ibcast_t *pnmpi_int_MPI_Ibcast;
  pnmpi_int_MPI_Ibsend_t *pnmpi_int_MPI_Ibsend;
  pnmpi_int_MPI_Iexscan_t *pnmpi_int_MPI_Iexscan;
  pnmpi_int_MPI_Igather_t *pnmpi_int_MPI_Igather;
  pnmpi_int_MPI_Igatherv_t *pnmpi_int_MPI_Igatherv;
  pnmpi_int_MPI_Improbe_t *pnmpi_int_MPI_Improbe;
  pnmpi_int_MPI_Imrecv_t *pnmpi_int_MPI_Imrecv;
  pnmpi_int_MPI_Ineighbor_allgather_t *pnmpi_int_MPI_Ineighbor_allgather;
  pnmpi_int_MPI_Ineighbor_allgatherv_t *pnmpi_int_MPI_Ineighbor_allgatherv;
  pnmpi_int_MPI_Ineighbor_alltoall_t *pnmpi_int_MPI_Ineighbor_alltoall;
  pnmpi_int_MPI_Ineighbor_alltoallv_t *pnmpi_int_MPI_Ineighbor_alltoallv;
  pnmpi_int_MPI_Ineighbor_alltoallw_t *pnmpi_int_MPI_Ineighbor_alltoallw;
  pnmpi_int_MPI_Info_create_t *pnmpi_int_MPI_Info_create;
  pnmpi_int_MPI_Info_delete_t *pnmpi_int_MPI_Info_delete;
  pnmpi_int_MPI_Info_dup_t *pnmpi_int_MPI_Info_dup;
  pnmpi_int_MPI_Info_free_t *pnmpi_int_MPI_Info_free;
  pnmpi_int_MPI_Info_get_t *pnmpi_int_MPI_Info_get;
  pnmpi_int_MPI_Info_get_nkeys_t *pnmpi_int_MPI_Info_get_nkeys;
  pnmpi_int_MPI_Info_get_nthkey_t *pnmpi_int_MPI_Info_get_nthkey;
  pnmpi_int_MPI_Info_get_valuelen_t *pnmpi_int_MPI_Info_get_valuelen;
  pnmpi_int_MPI_Info_set_t *pnmpi_int_MPI_Info_set;
  pnmpi_int_MPI_Init_t *pnmpi_int_MPI_Init;
  pnmpi_int_MPI_Init_thread_t *pnmpi_int_MPI_Init_thread;
  pnmpi_int_MPI_Initialized_t *pnmpi_int_MPI_Initialized;
  pnmpi_int_MPI_Intercomm_create_t *pnmpi_int_MPI_Intercomm_create;
  pnmpi_int_MPI_Intercomm_merge_t *pnmpi_int_MPI_Intercomm_merge;
  pnmpi_int_MPI_Iprobe_t *pnmpi_int_MPI_Iprobe;
  pnmpi_int_MPI_Irecv_t *pnmpi_int_MPI_Irecv;
  pnmpi_int_MPI_Ireduce_t *pnmpi_int_MPI_Ireduce;
  pnmpi_int_MPI_Ireduce_scatter_t *pnmpi_int_MPI_Ireduce_scatter;
  pnmpi_int_MPI_Ireduce_scatter_block_t *pnmpi_int_MPI_Ireduce_scatter_block;
  pnmpi_int_MPI_Irsend_t *pnmpi_int_MPI_Irsend;
  pnmpi_int_MPI_Is_thread_main_t *pnmpi_int_MPI_Is_thread_main;
  pnmpi_int_MPI_Iscan_t *pnmpi_int_MPI_Iscan;
  pnmpi_int_MPI_Iscatter_t *pnmpi_int_MPI_Iscatter;
  pnmpi_int_MPI_Iscatterv_t *pnmpi_int_MPI_Iscatterv;
  pnmpi_int_MPI_Isend_t *pnmpi_int_MPI_Isend;
  pnmpi_int_MPI_Issend_t *pnmpi_int_MPI_Issend;
  pnmpi_int_MPI_Keyval_create_t *pnmpi_int_MPI_Keyval_create;
  pnmpi_int_MPI_Keyval_free_t *pnmpi_int_MPI_Keyval_free;
  pnmpi_int_MPI_Lookup_name_t *pnmpi_int_MPI_Lookup_name;
  pnmpi_int_MPI_Mprobe_t *pnmpi_int_MPI_Mprobe;
  pnmpi_int_MPI_Mrecv_t *pnmpi_int_MPI_Mrecv;
  pnmpi_int_MPI_Neighbor_allgather_t *pnmpi_int_MPI_Neighbor_allgather;
  pnmpi_int_MPI_Neighbor_allgatherv_t *pnmpi_int_MPI_Neighbor_allgatherv;
  pnmpi_int_MPI_Neighbor_alltoall_t *pnmpi_int_MPI_Neighbor_alltoall;
  pnmpi_int_MPI_Neighbor_alltoallv_t *pnmpi_int_MPI_Neighbor_alltoallv;
  pnmpi_int_MPI_Neighbor_alltoallw_t *pnmpi_int_MPI_Neighbor_alltoallw;
  pnmpi_int_MPI_Op_commutative_t *pnmpi_int_MPI_Op_commutative;
  pnmpi_int_MPI_Op_create_t *pnmpi_int_MPI_Op_create;
  pnmpi_int_MPI_Op_free_t *pnmpi_int_MPI_Op_free;
  pnmpi_int_MPI_Open_port_t *pnmpi_int_MPI_Open_port;
  pnmpi_int_MPI_Pack_t *pnmpi_int_MPI_Pack;
  pnmpi_int_MPI_Pack_external_t *pnmpi_int_MPI_Pack_external;
  pnmpi_int_MPI_Pack_external_size_t *pnmpi_int_MPI_Pack_external_size;
  pnmpi_int_MPI_Pack_size_t *pnmpi_int_MPI_Pack_size;
  pnmpi_int_MPI_Pcontrol_t *pnmpi_int_MPI_Pcontrol;
  pnmpi_int_MPI_Probe_t *pnmpi_int_MPI_Probe;
  pnmpi_int_MPI_Publish_name_t *pnmpi_int_MPI_Publish_name;
  pnmpi_int_MPI_Put_t *pnmpi_int_MPI_Put;
  pnmpi_int_MPI_Query_thread_t *pnmpi_int_MPI_Query_thread;
  pnmpi_int_MPI_Raccumulate_t *pnmpi_int_MPI_Raccumulate;
  pnmpi_int_MPI_Recv_t *pnmpi_int_MPI_Recv;
  pnmpi_int_MPI_Recv_init_t *pnmpi_int_MPI_Recv_init;
  pnmpi_int_MPI_Reduce_t *pnmpi_int_MPI_Reduce;
  pnmpi_int_MPI_Reduce_local_t *pnmpi_int_MPI_Reduce_local;
  pnmpi_int_MPI_Reduce_scatter_t *pnmpi_int_MPI_Reduce_scatter;
  pnmpi_int_MPI_Reduce_scatter_block_t *pnmpi_int_MPI_Reduce_scatter_block;
  pnmpi_int_MPI_Register_datarep_t *pnmpi_int_MPI_Register_datarep;
  pnmpi_int_MPI_Request_free_t *pnmpi_int_MPI_Request_free;
  pnmpi_int_MPI_Request_get_status_t *pnmpi_int_MPI_Request_get_status;
  pnmpi_int_MPI_Rget_t *pnmpi_int_MPI_Rget;
  pnmpi_int_MPI_Rget_accumulate_t *pnmpi_int_MPI_Rget_accumulate;
  pnmpi_int_MPI_Rput_t *pnmpi_int_MPI_Rput;
  pnmpi_int_MPI_Rsend_t *pnmpi_int_MPI_Rsend;
  pnmpi_int_MPI_Rsend_init_t *pnmpi_int_MPI_Rsend_init;
  pnmpi_int_MPI_Scan_t *pnmpi_int_MPI_Scan;
  pnmpi_int_MPI_Scatter_t *pnmpi_int_MPI_Scatter;
  pnmpi_int_MPI_Scatterv_t *pnmpi_int_MPI_Scatterv;
  pnmpi_int_MPI_Send_t *pnmpi_int_MPI_Send;
  pnmpi_int_MPI_Send_init_t *pnmpi_int_MPI_Send_init;
  pnmpi_int_MPI_Sendrecv_t *pnmpi_int_MPI_Sendrecv;
  pnmpi_int_MPI_Sendrecv_replace_t *pnmpi_int_MPI_Sendrecv_replace;
  pnmpi_int_MPI_Ssend_t *pnmpi_int_MPI_Ssend;
  pnmpi_int_MPI_Ssend_init_t *pnmpi_int_MPI_Ssend_init;
  pnmpi_int_MPI_Start_t *pnmpi_int_MPI_Start;
  pnmpi_int_MPI_Startall_t *pnmpi_int_MPI_Startall;
  pnmpi_int_MPI_Status_set_cancelled_t *pnmpi_int_MPI_Status_set_cancelled;
  pnmpi_int_MPI_Status_set_elements_t *pnmpi_int_MPI_Status_set_elements;
  pnmpi_int_MPI_Status_set_elements_x_t *pnmpi_int_MPI_Status_set_elements_x;
  pnmpi_int_MPI_Test_t *pnmpi_int_MPI_Test;
  pnmpi_int_MPI_Test_cancelled_t *pnmpi_int_MPI_Test_cancelled;
  pnmpi_int_MPI_Testall_t *pnmpi_int_MPI_Testall;
  pnmpi_int_MPI_Testany_t *pnmpi_int_MPI_Testany;
  pnmpi_int_MPI_Testsome_t *pnmpi_int_MPI_Testsome;
  pnmpi_int_MPI_Topo_test_t *pnmpi_int_MPI_Topo_test;
  pnmpi_int_MPI_Type_commit_t *pnmpi_int_MPI_Type_commit;
  pnmpi_int_MPI_Type_contiguous_t *pnmpi_int_MPI_Type_contiguous;
  pnmpi_int_MPI_Type_create_darray_t *pnmpi_int_MPI_Type_create_darray;
  pnmpi_int_MPI_Type_create_f90_complex_t *pnmpi_int_MPI_Type_create_f90_complex;
  pnmpi_int_MPI_Type_create_f90_integer_t *pnmpi_int_MPI_Type_create_f90_integer;
  pnmpi_int_MPI_Type_create_f90_real_t *pnmpi_int_MPI_Type_create_f90_real;
  pnmpi_int_MPI_Type_create_hindexed_t *pnmpi_int_MPI_Type_create_hindexed;
  pnmpi_int_MPI_Type_create_hindexed_block_t *pnmpi_int_MPI_Type_create_hindexed_block;
  pnmpi_int_MPI_Type_create_hvector_t *pnmpi_int_MPI_Type_create_hvector;
  pnmpi_int_MPI_Type_create_indexed_block_t *pnmpi_int_MPI_Type_create_indexed_block;
  pnmpi_int_MPI_Type_create_keyval_t *pnmpi_int_MPI_Type_create_keyval;
  pnmpi_int_MPI_Type_create_resized_t *pnmpi_int_MPI_Type_create_resized;
  pnmpi_int_MPI_Type_create_struct_t *pnmpi_int_MPI_Type_create_struct;
  pnmpi_int_MPI_Type_create_subarray_t *pnmpi_int_MPI_Type_create_subarray;
  pnmpi_int_MPI_Type_delete_attr_t *pnmpi_int_MPI_Type_delete_attr;
  pnmpi_int_MPI_Type_dup_t *pnmpi_int_MPI_Type_dup;
  pnmpi_int_MPI_Type_extent_t *pnmpi_int_MPI_Type_extent;
  pnmpi_int_MPI_Type_free_t *pnmpi_int_MPI_Type_free;
  pnmpi_int_MPI_Type_free_keyval_t *pnmpi_int_MPI_Type_free_keyval;
  pnmpi_int_MPI_Type_get_attr_t *pnmpi_int_MPI_Type_get_attr;
  pnmpi_int_MPI_Type_get_contents_t *pnmpi_int_MPI_Type_get_contents;
  pnmpi_int_MPI_Type_get_envelope_t *pnmpi_int_MPI_Type_get_envelope;
  pnmpi_int_MPI_Type_get_extent_t *pnmpi_int_MPI_Type_get_extent;
  pnmpi_int_MPI_Type_get_extent_x_t *pnmpi_int_MPI_Type_get_extent_x;
  pnmpi_int_MPI_Type_get_name_t *pnmpi_int_MPI_Type_get_name;
  pnmpi_int_MPI_Type_get_true_extent_t *pnmpi_int_MPI_Type_get_true_extent;
  pnmpi_int_MPI_Type_get_true_extent_x_t *pnmpi_int_MPI_Type_get_true_extent_x;
  pnmpi_int_MPI_Type_hindexed_t *pnmpi_int_MPI_Type_hindexed;
  pnmpi_int_MPI_Type_hvector_t *pnmpi_int_MPI_Type_hvector;
  pnmpi_int_MPI_Type_indexed_t *pnmpi_int_MPI_Type_indexed;
  pnmpi_int_MPI_Type_lb_t *pnmpi_int_MPI_Type_lb;
  pnmpi_int_MPI_Type_match_size_t *pnmpi_int_MPI_Type_match_size;
  pnmpi_int_MPI_Type_set_attr_t *pnmpi_int_MPI_Type_set_attr;
  pnmpi_int_MPI_Type_set_name_t *pnmpi_int_MPI_Type_set_name;
  pnmpi_int_MPI_Type_size_t *pnmpi_int_MPI_Type_size;
  pnmpi_int_MPI_Type_size_x_t *pnmpi_int_MPI_Type_size_x;
  pnmpi_int_MPI_Type_struct_t *pnmpi_int_MPI_Type_struct;
  pnmpi_int_MPI_Type_ub_t *pnmpi_int_MPI_Type_ub;
  pnmpi_int_MPI_Type_vector_t *pnmpi_int_MPI_Type_vector;
  pnmpi_int_MPI_Unpack_t *pnmpi_int_MPI_Unpack;
  pnmpi_int_MPI_Unpack_external_t *pnmpi_int_MPI_Unpack_external;
  pnmpi_int_MPI_Unpublish_name_t *pnmpi_int_MPI_Unpublish_name;
  pnmpi_int_MPI_Wait_t *pnmpi_int_MPI_Wait;
  pnmpi_int_MPI_Waitall_t *pnmpi_int_MPI_Waitall;
  pnmpi_int_MPI_Waitany_t *pnmpi_int_MPI_Waitany;
  pnmpi_int_MPI_Waitsome_t *pnmpi_int_MPI_Waitsome;
  pnmpi_int_MPI_Win_allocate_t *pnmpi_int_MPI_Win_allocate;
  pnmpi_int_MPI_Win_allocate_shared_t *pnmpi_int_MPI_Win_allocate_shared;
  pnmpi_int_MPI_Win_attach_t *pnmpi_int_MPI_Win_attach;
  pnmpi_int_MPI_Win_call_errhandler_t *pnmpi_int_MPI_Win_call_errhandler;
  pnmpi_int_MPI_Win_complete_t *pnmpi_int_MPI_Win_complete;
  pnmpi_int_MPI_Win_create_t *pnmpi_int_MPI_Win_create;
  pnmpi_int_MPI_Win_create_dynamic_t *pnmpi_int_MPI_Win_create_dynamic;
  pnmpi_int_MPI_Win_create_errhandler_t *pnmpi_int_MPI_Win_create_errhandler;
  pnmpi_int_MPI_Win_create_keyval_t *pnmpi_int_MPI_Win_create_keyval;
  pnmpi_int_MPI_Win_delete_attr_t *pnmpi_int_MPI_Win_delete_attr;
  pnmpi_int_MPI_Win_detach_t *pnmpi_int_MPI_Win_detach;
  pnmpi_int_MPI_Win_fence_t *pnmpi_int_MPI_Win_fence;
  pnmpi_int_MPI_Win_flush_t *pnmpi_int_MPI_Win_flush;
  pnmpi_int_MPI_Win_flush_all_t *pnmpi_int_MPI_Win_flush_all;
  pnmpi_int_MPI_Win_flush_local_t *pnmpi_int_MPI_Win_flush_local;
  pnmpi_int_MPI_Win_flush_local_all_t *pnmpi_int_MPI_Win_flush_local_all;
  pnmpi_int_MPI_Win_free_t *pnmpi_int_MPI_Win_free;
  pnmpi_int_MPI_Win_free_keyval_t *pnmpi_int_MPI_Win_free_keyval;
  pnmpi_int_MPI_Win_get_attr_t *pnmpi_int_MPI_Win_get_attr;
  pnmpi_int_MPI_Win_get_errhandler_t *pnmpi_int_MPI_Win_get_errhandler;
  pnmpi_int_MPI_Win_get_group_t *pnmpi_int_MPI_Win_get_group;
  pnmpi_int_MPI_Win_get_info_t *pnmpi_int_MPI_Win_get_info;
  pnmpi_int_MPI_Win_get_name_t *pnmpi_int_MPI_Win_get_name;
  pnmpi_int_MPI_Win_lock_t *pnmpi_int_MPI_Win_lock;
  pnmpi_int_MPI_Win_lock_all_t *pnmpi_int_MPI_Win_lock_all;
  pnmpi_int_MPI_Win_post_t *pnmpi_int_MPI_Win_post;
  pnmpi_int_MPI_Win_set_attr_t *pnmpi_int_MPI_Win_set_attr;
  pnmpi_int_MPI_Win_set_errhandler_t *pnmpi_int_MPI_Win_set_errhandler;
  pnmpi_int_MPI_Win_set_info_t *pnmpi_int_MPI_Win_set_info;
  pnmpi_int_MPI_Win_set_name_t *pnmpi_int_MPI_Win_set_name;
  pnmpi_int_MPI_Win_shared_query_t *pnmpi_int_MPI_Win_shared_query;
  pnmpi_int_MPI_Win_start_t *pnmpi_int_MPI_Win_start;
  pnmpi_int_MPI_Win_sync_t *pnmpi_int_MPI_Win_sync;
  pnmpi_int_MPI_Win_test_t *pnmpi_int_MPI_Win_test;
  pnmpi_int_MPI_Win_unlock_t *pnmpi_int_MPI_Win_unlock;
  pnmpi_int_MPI_Win_unlock_all_t *pnmpi_int_MPI_Win_unlock_all;
  pnmpi_int_MPI_Win_wait_t *pnmpi_int_MPI_Win_wait;
  pnmpi_int_MPI_Wtick_t *pnmpi_int_MPI_Wtick;
  pnmpi_int_MPI_Wtime_t *pnmpi_int_MPI_Wtime;

} pnmpi_functions_t;

#define INITIALIZE_ALL_FUNCTION_STACKS(mods) \
INITIALIZE_FUNCTION_STACK("MPI_Abort", MPI_Abort_ID, pnmpi_int_MPI_Abort_t, pnmpi_int_MPI_Abort, mods, MPI_Abort);\
INITIALIZE_FUNCTION_STACK("MPI_Accumulate", MPI_Accumulate_ID, pnmpi_int_MPI_Accumulate_t, pnmpi_int_MPI_Accumulate, mods, MPI_Accumulate);\
INITIALIZE_FUNCTION_STACK("MPI_Add_error_class", MPI_Add_error_class_ID, pnmpi_int_MPI_Add_error_class_t, pnmpi_int_MPI_Add_error_class, mods, MPI_Add_error_class);\
INITIALIZE_FUNCTION_STACK("MPI_Add_error_code", MPI_Add_error_code_ID, pnmpi_int_MPI_Add_error_code_t, pnmpi_int_MPI_Add_error_code, mods, MPI_Add_error_code);\
INITIALIZE_FUNCTION_STACK("MPI_Add_error_string", MPI_Add_error_string_ID, pnmpi_int_MPI_Add_error_string_t, pnmpi_int_MPI_Add_error_string, mods, MPI_Add_error_string);\
INITIALIZE_FUNCTION_STACK("MPI_Address", MPI_Address_ID, pnmpi_int_MPI_Address_t, pnmpi_int_MPI_Address, mods, MPI_Address);\
INITIALIZE_FUNCTION_STACK("MPI_Allgather", MPI_Allgather_ID, pnmpi_int_MPI_Allgather_t, pnmpi_int_MPI_Allgather, mods, MPI_Allgather);\
INITIALIZE_FUNCTION_STACK("MPI_Allgatherv", MPI_Allgatherv_ID, pnmpi_int_MPI_Allgatherv_t, pnmpi_int_MPI_Allgatherv, mods, MPI_Allgatherv);\
INITIALIZE_FUNCTION_STACK("MPI_Alloc_mem", MPI_Alloc_mem_ID, pnmpi_int_MPI_Alloc_mem_t, pnmpi_int_MPI_Alloc_mem, mods, MPI_Alloc_mem);\
INITIALIZE_FUNCTION_STACK("MPI_Allreduce", MPI_Allreduce_ID, pnmpi_int_MPI_Allreduce_t, pnmpi_int_MPI_Allreduce, mods, MPI_Allreduce);\
INITIALIZE_FUNCTION_STACK("MPI_Alltoall", MPI_Alltoall_ID, pnmpi_int_MPI_Alltoall_t, pnmpi_int_MPI_Alltoall, mods, MPI_Alltoall);\
INITIALIZE_FUNCTION_STACK("MPI_Alltoallv", MPI_Alltoallv_ID, pnmpi_int_MPI_Alltoallv_t, pnmpi_int_MPI_Alltoallv, mods, MPI_Alltoallv);\
INITIALIZE_FUNCTION_STACK("MPI_Alltoallw", MPI_Alltoallw_ID, pnmpi_int_MPI_Alltoallw_t, pnmpi_int_MPI_Alltoallw, mods, MPI_Alltoallw);\
INITIALIZE_FUNCTION_STACK("MPI_Attr_delete", MPI_Attr_delete_ID, pnmpi_int_MPI_Attr_delete_t, pnmpi_int_MPI_Attr_delete, mods, MPI_Attr_delete);\
INITIALIZE_FUNCTION_STACK("MPI_Attr_get", MPI_Attr_get_ID, pnmpi_int_MPI_Attr_get_t, pnmpi_int_MPI_Attr_get, mods, MPI_Attr_get);\
INITIALIZE_FUNCTION_STACK("MPI_Attr_put", MPI_Attr_put_ID, pnmpi_int_MPI_Attr_put_t, pnmpi_int_MPI_Attr_put, mods, MPI_Attr_put);\
INITIALIZE_FUNCTION_STACK("MPI_Barrier", MPI_Barrier_ID, pnmpi_int_MPI_Barrier_t, pnmpi_int_MPI_Barrier, mods, MPI_Barrier);\
INITIALIZE_FUNCTION_STACK("MPI_Bcast", MPI_Bcast_ID, pnmpi_int_MPI_Bcast_t, pnmpi_int_MPI_Bcast, mods, MPI_Bcast);\
INITIALIZE_FUNCTION_STACK("MPI_Bsend", MPI_Bsend_ID, pnmpi_int_MPI_Bsend_t, pnmpi_int_MPI_Bsend, mods, MPI_Bsend);\
INITIALIZE_FUNCTION_STACK("MPI_Bsend_init", MPI_Bsend_init_ID, pnmpi_int_MPI_Bsend_init_t, pnmpi_int_MPI_Bsend_init, mods, MPI_Bsend_init);\
INITIALIZE_FUNCTION_STACK("MPI_Buffer_attach", MPI_Buffer_attach_ID, pnmpi_int_MPI_Buffer_attach_t, pnmpi_int_MPI_Buffer_attach, mods, MPI_Buffer_attach);\
INITIALIZE_FUNCTION_STACK("MPI_Buffer_detach", MPI_Buffer_detach_ID, pnmpi_int_MPI_Buffer_detach_t, pnmpi_int_MPI_Buffer_detach, mods, MPI_Buffer_detach);\
INITIALIZE_FUNCTION_STACK("MPI_Cancel", MPI_Cancel_ID, pnmpi_int_MPI_Cancel_t, pnmpi_int_MPI_Cancel, mods, MPI_Cancel);\
INITIALIZE_FUNCTION_STACK("MPI_Cart_coords", MPI_Cart_coords_ID, pnmpi_int_MPI_Cart_coords_t, pnmpi_int_MPI_Cart_coords, mods, MPI_Cart_coords);\
INITIALIZE_FUNCTION_STACK("MPI_Cart_create", MPI_Cart_create_ID, pnmpi_int_MPI_Cart_create_t, pnmpi_int_MPI_Cart_create, mods, MPI_Cart_create);\
INITIALIZE_FUNCTION_STACK("MPI_Cart_get", MPI_Cart_get_ID, pnmpi_int_MPI_Cart_get_t, pnmpi_int_MPI_Cart_get, mods, MPI_Cart_get);\
INITIALIZE_FUNCTION_STACK("MPI_Cart_map", MPI_Cart_map_ID, pnmpi_int_MPI_Cart_map_t, pnmpi_int_MPI_Cart_map, mods, MPI_Cart_map);\
INITIALIZE_FUNCTION_STACK("MPI_Cart_rank", MPI_Cart_rank_ID, pnmpi_int_MPI_Cart_rank_t, pnmpi_int_MPI_Cart_rank, mods, MPI_Cart_rank);\
INITIALIZE_FUNCTION_STACK("MPI_Cart_shift", MPI_Cart_shift_ID, pnmpi_int_MPI_Cart_shift_t, pnmpi_int_MPI_Cart_shift, mods, MPI_Cart_shift);\
INITIALIZE_FUNCTION_STACK("MPI_Cart_sub", MPI_Cart_sub_ID, pnmpi_int_MPI_Cart_sub_t, pnmpi_int_MPI_Cart_sub, mods, MPI_Cart_sub);\
INITIALIZE_FUNCTION_STACK("MPI_Cartdim_get", MPI_Cartdim_get_ID, pnmpi_int_MPI_Cartdim_get_t, pnmpi_int_MPI_Cartdim_get, mods, MPI_Cartdim_get);\
INITIALIZE_FUNCTION_STACK("MPI_Close_port", MPI_Close_port_ID, pnmpi_int_MPI_Close_port_t, pnmpi_int_MPI_Close_port, mods, MPI_Close_port);\
INITIALIZE_FUNCTION_STACK("MPI_Comm_accept", MPI_Comm_accept_ID, pnmpi_int_MPI_Comm_accept_t, pnmpi_int_MPI_Comm_accept, mods, MPI_Comm_accept);\
INITIALIZE_FUNCTION_STACK("MPI_Comm_call_errhandler", MPI_Comm_call_errhandler_ID, pnmpi_int_MPI_Comm_call_errhandler_t, pnmpi_int_MPI_Comm_call_errhandler, mods, MPI_Comm_call_errhandler);\
INITIALIZE_FUNCTION_STACK("MPI_Comm_compare", MPI_Comm_compare_ID, pnmpi_int_MPI_Comm_compare_t, pnmpi_int_MPI_Comm_compare, mods, MPI_Comm_compare);\
INITIALIZE_FUNCTION_STACK("MPI_Comm_connect", MPI_Comm_connect_ID, pnmpi_int_MPI_Comm_connect_t, pnmpi_int_MPI_Comm_connect, mods, MPI_Comm_connect);\
INITIALIZE_FUNCTION_STACK("MPI_Comm_create", MPI_Comm_create_ID, pnmpi_int_MPI_Comm_create_t, pnmpi_int_MPI_Comm_create, mods, MPI_Comm_create);\
INITIALIZE_FUNCTION_STACK("MPI_Comm_create_errhandler", MPI_Comm_create_errhandler_ID, pnmpi_int_MPI_Comm_create_errhandler_t, pnmpi_int_MPI_Comm_create_errhandler, mods, MPI_Comm_create_errhandler);\
INITIALIZE_FUNCTION_STACK("MPI_Comm_create_group", MPI_Comm_create_group_ID, pnmpi_int_MPI_Comm_create_group_t, pnmpi_int_MPI_Comm_create_group, mods, MPI_Comm_create_group);\
INITIALIZE_FUNCTION_STACK("MPI_Comm_create_keyval", MPI_Comm_create_keyval_ID, pnmpi_int_MPI_Comm_create_keyval_t, pnmpi_int_MPI_Comm_create_keyval, mods, MPI_Comm_create_keyval);\
INITIALIZE_FUNCTION_STACK("MPI_Comm_delete_attr", MPI_Comm_delete_attr_ID, pnmpi_int_MPI_Comm_delete_attr_t, pnmpi_int_MPI_Comm_delete_attr, mods, MPI_Comm_delete_attr);\
INITIALIZE_FUNCTION_STACK("MPI_Comm_disconnect", MPI_Comm_disconnect_ID, pnmpi_int_MPI_Comm_disconnect_t, pnmpi_int_MPI_Comm_disconnect, mods, MPI_Comm_disconnect);\
INITIALIZE_FUNCTION_STACK("MPI_Comm_dup", MPI_Comm_dup_ID, pnmpi_int_MPI_Comm_dup_t, pnmpi_int_MPI_Comm_dup, mods, MPI_Comm_dup);\
INITIALIZE_FUNCTION_STACK("MPI_Comm_dup_with_info", MPI_Comm_dup_with_info_ID, pnmpi_int_MPI_Comm_dup_with_info_t, pnmpi_int_MPI_Comm_dup_with_info, mods, MPI_Comm_dup_with_info);\
INITIALIZE_FUNCTION_STACK("MPI_Comm_free", MPI_Comm_free_ID, pnmpi_int_MPI_Comm_free_t, pnmpi_int_MPI_Comm_free, mods, MPI_Comm_free);\
INITIALIZE_FUNCTION_STACK("MPI_Comm_free_keyval", MPI_Comm_free_keyval_ID, pnmpi_int_MPI_Comm_free_keyval_t, pnmpi_int_MPI_Comm_free_keyval, mods, MPI_Comm_free_keyval);\
INITIALIZE_FUNCTION_STACK("MPI_Comm_get_attr", MPI_Comm_get_attr_ID, pnmpi_int_MPI_Comm_get_attr_t, pnmpi_int_MPI_Comm_get_attr, mods, MPI_Comm_get_attr);\
INITIALIZE_FUNCTION_STACK("MPI_Comm_get_errhandler", MPI_Comm_get_errhandler_ID, pnmpi_int_MPI_Comm_get_errhandler_t, pnmpi_int_MPI_Comm_get_errhandler, mods, MPI_Comm_get_errhandler);\
INITIALIZE_FUNCTION_STACK("MPI_Comm_get_info", MPI_Comm_get_info_ID, pnmpi_int_MPI_Comm_get_info_t, pnmpi_int_MPI_Comm_get_info, mods, MPI_Comm_get_info);\
INITIALIZE_FUNCTION_STACK("MPI_Comm_get_name", MPI_Comm_get_name_ID, pnmpi_int_MPI_Comm_get_name_t, pnmpi_int_MPI_Comm_get_name, mods, MPI_Comm_get_name);\
INITIALIZE_FUNCTION_STACK("MPI_Comm_get_parent", MPI_Comm_get_parent_ID, pnmpi_int_MPI_Comm_get_parent_t, pnmpi_int_MPI_Comm_get_parent, mods, MPI_Comm_get_parent);\
INITIALIZE_FUNCTION_STACK("MPI_Comm_group", MPI_Comm_group_ID, pnmpi_int_MPI_Comm_group_t, pnmpi_int_MPI_Comm_group, mods, MPI_Comm_group);\
INITIALIZE_FUNCTION_STACK("MPI_Comm_idup", MPI_Comm_idup_ID, pnmpi_int_MPI_Comm_idup_t, pnmpi_int_MPI_Comm_idup, mods, MPI_Comm_idup);\
INITIALIZE_FUNCTION_STACK("MPI_Comm_join", MPI_Comm_join_ID, pnmpi_int_MPI_Comm_join_t, pnmpi_int_MPI_Comm_join, mods, MPI_Comm_join);\
INITIALIZE_FUNCTION_STACK("MPI_Comm_rank", MPI_Comm_rank_ID, pnmpi_int_MPI_Comm_rank_t, pnmpi_int_MPI_Comm_rank, mods, MPI_Comm_rank);\
INITIALIZE_FUNCTION_STACK("MPI_Comm_remote_group", MPI_Comm_remote_group_ID, pnmpi_int_MPI_Comm_remote_group_t, pnmpi_int_MPI_Comm_remote_group, mods, MPI_Comm_remote_group);\
INITIALIZE_FUNCTION_STACK("MPI_Comm_remote_size", MPI_Comm_remote_size_ID, pnmpi_int_MPI_Comm_remote_size_t, pnmpi_int_MPI_Comm_remote_size, mods, MPI_Comm_remote_size);\
INITIALIZE_FUNCTION_STACK("MPI_Comm_set_attr", MPI_Comm_set_attr_ID, pnmpi_int_MPI_Comm_set_attr_t, pnmpi_int_MPI_Comm_set_attr, mods, MPI_Comm_set_attr);\
INITIALIZE_FUNCTION_STACK("MPI_Comm_set_errhandler", MPI_Comm_set_errhandler_ID, pnmpi_int_MPI_Comm_set_errhandler_t, pnmpi_int_MPI_Comm_set_errhandler, mods, MPI_Comm_set_errhandler);\
INITIALIZE_FUNCTION_STACK("MPI_Comm_set_info", MPI_Comm_set_info_ID, pnmpi_int_MPI_Comm_set_info_t, pnmpi_int_MPI_Comm_set_info, mods, MPI_Comm_set_info);\
INITIALIZE_FUNCTION_STACK("MPI_Comm_set_name", MPI_Comm_set_name_ID, pnmpi_int_MPI_Comm_set_name_t, pnmpi_int_MPI_Comm_set_name, mods, MPI_Comm_set_name);\
INITIALIZE_FUNCTION_STACK("MPI_Comm_size", MPI_Comm_size_ID, pnmpi_int_MPI_Comm_size_t, pnmpi_int_MPI_Comm_size, mods, MPI_Comm_size);\
INITIALIZE_FUNCTION_STACK("MPI_Comm_split", MPI_Comm_split_ID, pnmpi_int_MPI_Comm_split_t, pnmpi_int_MPI_Comm_split, mods, MPI_Comm_split);\
INITIALIZE_FUNCTION_STACK("MPI_Comm_split_type", MPI_Comm_split_type_ID, pnmpi_int_MPI_Comm_split_type_t, pnmpi_int_MPI_Comm_split_type, mods, MPI_Comm_split_type);\
INITIALIZE_FUNCTION_STACK("MPI_Comm_test_inter", MPI_Comm_test_inter_ID, pnmpi_int_MPI_Comm_test_inter_t, pnmpi_int_MPI_Comm_test_inter, mods, MPI_Comm_test_inter);\
INITIALIZE_FUNCTION_STACK("MPI_Compare_and_swap", MPI_Compare_and_swap_ID, pnmpi_int_MPI_Compare_and_swap_t, pnmpi_int_MPI_Compare_and_swap, mods, MPI_Compare_and_swap);\
INITIALIZE_FUNCTION_STACK("MPI_Dims_create", MPI_Dims_create_ID, pnmpi_int_MPI_Dims_create_t, pnmpi_int_MPI_Dims_create, mods, MPI_Dims_create);\
INITIALIZE_FUNCTION_STACK("MPI_Dist_graph_create", MPI_Dist_graph_create_ID, pnmpi_int_MPI_Dist_graph_create_t, pnmpi_int_MPI_Dist_graph_create, mods, MPI_Dist_graph_create);\
INITIALIZE_FUNCTION_STACK("MPI_Dist_graph_create_adjacent", MPI_Dist_graph_create_adjacent_ID, pnmpi_int_MPI_Dist_graph_create_adjacent_t, pnmpi_int_MPI_Dist_graph_create_adjacent, mods, MPI_Dist_graph_create_adjacent);\
INITIALIZE_FUNCTION_STACK("MPI_Dist_graph_neighbors", MPI_Dist_graph_neighbors_ID, pnmpi_int_MPI_Dist_graph_neighbors_t, pnmpi_int_MPI_Dist_graph_neighbors, mods, MPI_Dist_graph_neighbors);\
INITIALIZE_FUNCTION_STACK("MPI_Dist_graph_neighbors_count", MPI_Dist_graph_neighbors_count_ID, pnmpi_int_MPI_Dist_graph_neighbors_count_t, pnmpi_int_MPI_Dist_graph_neighbors_count, mods, MPI_Dist_graph_neighbors_count);\
INITIALIZE_FUNCTION_STACK("MPI_Errhandler_create", MPI_Errhandler_create_ID, pnmpi_int_MPI_Errhandler_create_t, pnmpi_int_MPI_Errhandler_create, mods, MPI_Errhandler_create);\
INITIALIZE_FUNCTION_STACK("MPI_Errhandler_free", MPI_Errhandler_free_ID, pnmpi_int_MPI_Errhandler_free_t, pnmpi_int_MPI_Errhandler_free, mods, MPI_Errhandler_free);\
INITIALIZE_FUNCTION_STACK("MPI_Errhandler_get", MPI_Errhandler_get_ID, pnmpi_int_MPI_Errhandler_get_t, pnmpi_int_MPI_Errhandler_get, mods, MPI_Errhandler_get);\
INITIALIZE_FUNCTION_STACK("MPI_Errhandler_set", MPI_Errhandler_set_ID, pnmpi_int_MPI_Errhandler_set_t, pnmpi_int_MPI_Errhandler_set, mods, MPI_Errhandler_set);\
INITIALIZE_FUNCTION_STACK("MPI_Error_class", MPI_Error_class_ID, pnmpi_int_MPI_Error_class_t, pnmpi_int_MPI_Error_class, mods, MPI_Error_class);\
INITIALIZE_FUNCTION_STACK("MPI_Error_string", MPI_Error_string_ID, pnmpi_int_MPI_Error_string_t, pnmpi_int_MPI_Error_string, mods, MPI_Error_string);\
INITIALIZE_FUNCTION_STACK("MPI_Exscan", MPI_Exscan_ID, pnmpi_int_MPI_Exscan_t, pnmpi_int_MPI_Exscan, mods, MPI_Exscan);\
INITIALIZE_FUNCTION_STACK("MPI_Fetch_and_op", MPI_Fetch_and_op_ID, pnmpi_int_MPI_Fetch_and_op_t, pnmpi_int_MPI_Fetch_and_op, mods, MPI_Fetch_and_op);\
INITIALIZE_FUNCTION_STACK("MPI_File_call_errhandler", MPI_File_call_errhandler_ID, pnmpi_int_MPI_File_call_errhandler_t, pnmpi_int_MPI_File_call_errhandler, mods, MPI_File_call_errhandler);\
INITIALIZE_FUNCTION_STACK("MPI_File_close", MPI_File_close_ID, pnmpi_int_MPI_File_close_t, pnmpi_int_MPI_File_close, mods, MPI_File_close);\
INITIALIZE_FUNCTION_STACK("MPI_File_create_errhandler", MPI_File_create_errhandler_ID, pnmpi_int_MPI_File_create_errhandler_t, pnmpi_int_MPI_File_create_errhandler, mods, MPI_File_create_errhandler);\
INITIALIZE_FUNCTION_STACK("MPI_File_delete", MPI_File_delete_ID, pnmpi_int_MPI_File_delete_t, pnmpi_int_MPI_File_delete, mods, MPI_File_delete);\
INITIALIZE_FUNCTION_STACK("MPI_File_get_amode", MPI_File_get_amode_ID, pnmpi_int_MPI_File_get_amode_t, pnmpi_int_MPI_File_get_amode, mods, MPI_File_get_amode);\
INITIALIZE_FUNCTION_STACK("MPI_File_get_atomicity", MPI_File_get_atomicity_ID, pnmpi_int_MPI_File_get_atomicity_t, pnmpi_int_MPI_File_get_atomicity, mods, MPI_File_get_atomicity);\
INITIALIZE_FUNCTION_STACK("MPI_File_get_byte_offset", MPI_File_get_byte_offset_ID, pnmpi_int_MPI_File_get_byte_offset_t, pnmpi_int_MPI_File_get_byte_offset, mods, MPI_File_get_byte_offset);\
INITIALIZE_FUNCTION_STACK("MPI_File_get_errhandler", MPI_File_get_errhandler_ID, pnmpi_int_MPI_File_get_errhandler_t, pnmpi_int_MPI_File_get_errhandler, mods, MPI_File_get_errhandler);\
INITIALIZE_FUNCTION_STACK("MPI_File_get_group", MPI_File_get_group_ID, pnmpi_int_MPI_File_get_group_t, pnmpi_int_MPI_File_get_group, mods, MPI_File_get_group);\
INITIALIZE_FUNCTION_STACK("MPI_File_get_info", MPI_File_get_info_ID, pnmpi_int_MPI_File_get_info_t, pnmpi_int_MPI_File_get_info, mods, MPI_File_get_info);\
INITIALIZE_FUNCTION_STACK("MPI_File_get_position", MPI_File_get_position_ID, pnmpi_int_MPI_File_get_position_t, pnmpi_int_MPI_File_get_position, mods, MPI_File_get_position);\
INITIALIZE_FUNCTION_STACK("MPI_File_get_position_shared", MPI_File_get_position_shared_ID, pnmpi_int_MPI_File_get_position_shared_t, pnmpi_int_MPI_File_get_position_shared, mods, MPI_File_get_position_shared);\
INITIALIZE_FUNCTION_STACK("MPI_File_get_size", MPI_File_get_size_ID, pnmpi_int_MPI_File_get_size_t, pnmpi_int_MPI_File_get_size, mods, MPI_File_get_size);\
INITIALIZE_FUNCTION_STACK("MPI_File_get_type_extent", MPI_File_get_type_extent_ID, pnmpi_int_MPI_File_get_type_extent_t, pnmpi_int_MPI_File_get_type_extent, mods, MPI_File_get_type_extent);\
INITIALIZE_FUNCTION_STACK("MPI_File_get_view", MPI_File_get_view_ID, pnmpi_int_MPI_File_get_view_t, pnmpi_int_MPI_File_get_view, mods, MPI_File_get_view);\
INITIALIZE_FUNCTION_STACK("MPI_File_iread", MPI_File_iread_ID, pnmpi_int_MPI_File_iread_t, pnmpi_int_MPI_File_iread, mods, MPI_File_iread);\
INITIALIZE_FUNCTION_STACK("MPI_File_iread_all", MPI_File_iread_all_ID, pnmpi_int_MPI_File_iread_all_t, pnmpi_int_MPI_File_iread_all, mods, MPI_File_iread_all);\
INITIALIZE_FUNCTION_STACK("MPI_File_iread_at", MPI_File_iread_at_ID, pnmpi_int_MPI_File_iread_at_t, pnmpi_int_MPI_File_iread_at, mods, MPI_File_iread_at);\
INITIALIZE_FUNCTION_STACK("MPI_File_iread_at_all", MPI_File_iread_at_all_ID, pnmpi_int_MPI_File_iread_at_all_t, pnmpi_int_MPI_File_iread_at_all, mods, MPI_File_iread_at_all);\
INITIALIZE_FUNCTION_STACK("MPI_File_iread_shared", MPI_File_iread_shared_ID, pnmpi_int_MPI_File_iread_shared_t, pnmpi_int_MPI_File_iread_shared, mods, MPI_File_iread_shared);\
INITIALIZE_FUNCTION_STACK("MPI_File_iwrite", MPI_File_iwrite_ID, pnmpi_int_MPI_File_iwrite_t, pnmpi_int_MPI_File_iwrite, mods, MPI_File_iwrite);\
INITIALIZE_FUNCTION_STACK("MPI_File_iwrite_all", MPI_File_iwrite_all_ID, pnmpi_int_MPI_File_iwrite_all_t, pnmpi_int_MPI_File_iwrite_all, mods, MPI_File_iwrite_all);\
INITIALIZE_FUNCTION_STACK("MPI_File_iwrite_at", MPI_File_iwrite_at_ID, pnmpi_int_MPI_File_iwrite_at_t, pnmpi_int_MPI_File_iwrite_at, mods, MPI_File_iwrite_at);\
INITIALIZE_FUNCTION_STACK("MPI_File_iwrite_at_all", MPI_File_iwrite_at_all_ID, pnmpi_int_MPI_File_iwrite_at_all_t, pnmpi_int_MPI_File_iwrite_at_all, mods, MPI_File_iwrite_at_all);\
INITIALIZE_FUNCTION_STACK("MPI_File_iwrite_shared", MPI_File_iwrite_shared_ID, pnmpi_int_MPI_File_iwrite_shared_t, pnmpi_int_MPI_File_iwrite_shared, mods, MPI_File_iwrite_shared);\
INITIALIZE_FUNCTION_STACK("MPI_File_open", MPI_File_open_ID, pnmpi_int_MPI_File_open_t, pnmpi_int_MPI_File_open, mods, MPI_File_open);\
INITIALIZE_FUNCTION_STACK("MPI_File_preallocate", MPI_File_preallocate_ID, pnmpi_int_MPI_File_preallocate_t, pnmpi_int_MPI_File_preallocate, mods, MPI_File_preallocate);\
INITIALIZE_FUNCTION_STACK("MPI_File_read", MPI_File_read_ID, pnmpi_int_MPI_File_read_t, pnmpi_int_MPI_File_read, mods, MPI_File_read);\
INITIALIZE_FUNCTION_STACK("MPI_File_read_all", MPI_File_read_all_ID, pnmpi_int_MPI_File_read_all_t, pnmpi_int_MPI_File_read_all, mods, MPI_File_read_all);\
INITIALIZE_FUNCTION_STACK("MPI_File_read_all_begin", MPI_File_read_all_begin_ID, pnmpi_int_MPI_File_read_all_begin_t, pnmpi_int_MPI_File_read_all_begin, mods, MPI_File_read_all_begin);\
INITIALIZE_FUNCTION_STACK("MPI_File_read_all_end", MPI_File_read_all_end_ID, pnmpi_int_MPI_File_read_all_end_t, pnmpi_int_MPI_File_read_all_end, mods, MPI_File_read_all_end);\
INITIALIZE_FUNCTION_STACK("MPI_File_read_at", MPI_File_read_at_ID, pnmpi_int_MPI_File_read_at_t, pnmpi_int_MPI_File_read_at, mods, MPI_File_read_at);\
INITIALIZE_FUNCTION_STACK("MPI_File_read_at_all", MPI_File_read_at_all_ID, pnmpi_int_MPI_File_read_at_all_t, pnmpi_int_MPI_File_read_at_all, mods, MPI_File_read_at_all);\
INITIALIZE_FUNCTION_STACK("MPI_File_read_at_all_begin", MPI_File_read_at_all_begin_ID, pnmpi_int_MPI_File_read_at_all_begin_t, pnmpi_int_MPI_File_read_at_all_begin, mods, MPI_File_read_at_all_begin);\
INITIALIZE_FUNCTION_STACK("MPI_File_read_at_all_end", MPI_File_read_at_all_end_ID, pnmpi_int_MPI_File_read_at_all_end_t, pnmpi_int_MPI_File_read_at_all_end, mods, MPI_File_read_at_all_end);\
INITIALIZE_FUNCTION_STACK("MPI_File_read_ordered", MPI_File_read_ordered_ID, pnmpi_int_MPI_File_read_ordered_t, pnmpi_int_MPI_File_read_ordered, mods, MPI_File_read_ordered);\
INITIALIZE_FUNCTION_STACK("MPI_File_read_ordered_begin", MPI_File_read_ordered_begin_ID, pnmpi_int_MPI_File_read_ordered_begin_t, pnmpi_int_MPI_File_read_ordered_begin, mods, MPI_File_read_ordered_begin);\
INITIALIZE_FUNCTION_STACK("MPI_File_read_ordered_end", MPI_File_read_ordered_end_ID, pnmpi_int_MPI_File_read_ordered_end_t, pnmpi_int_MPI_File_read_ordered_end, mods, MPI_File_read_ordered_end);\
INITIALIZE_FUNCTION_STACK("MPI_File_read_shared", MPI_File_read_shared_ID, pnmpi_int_MPI_File_read_shared_t, pnmpi_int_MPI_File_read_shared, mods, MPI_File_read_shared);\
INITIALIZE_FUNCTION_STACK("MPI_File_seek", MPI_File_seek_ID, pnmpi_int_MPI_File_seek_t, pnmpi_int_MPI_File_seek, mods, MPI_File_seek);\
INITIALIZE_FUNCTION_STACK("MPI_File_seek_shared", MPI_File_seek_shared_ID, pnmpi_int_MPI_File_seek_shared_t, pnmpi_int_MPI_File_seek_shared, mods, MPI_File_seek_shared);\
INITIALIZE_FUNCTION_STACK("MPI_File_set_atomicity", MPI_File_set_atomicity_ID, pnmpi_int_MPI_File_set_atomicity_t, pnmpi_int_MPI_File_set_atomicity, mods, MPI_File_set_atomicity);\
INITIALIZE_FUNCTION_STACK("MPI_File_set_errhandler", MPI_File_set_errhandler_ID, pnmpi_int_MPI_File_set_errhandler_t, pnmpi_int_MPI_File_set_errhandler, mods, MPI_File_set_errhandler);\
INITIALIZE_FUNCTION_STACK("MPI_File_set_info", MPI_File_set_info_ID, pnmpi_int_MPI_File_set_info_t, pnmpi_int_MPI_File_set_info, mods, MPI_File_set_info);\
INITIALIZE_FUNCTION_STACK("MPI_File_set_size", MPI_File_set_size_ID, pnmpi_int_MPI_File_set_size_t, pnmpi_int_MPI_File_set_size, mods, MPI_File_set_size);\
INITIALIZE_FUNCTION_STACK("MPI_File_set_view", MPI_File_set_view_ID, pnmpi_int_MPI_File_set_view_t, pnmpi_int_MPI_File_set_view, mods, MPI_File_set_view);\
INITIALIZE_FUNCTION_STACK("MPI_File_sync", MPI_File_sync_ID, pnmpi_int_MPI_File_sync_t, pnmpi_int_MPI_File_sync, mods, MPI_File_sync);\
INITIALIZE_FUNCTION_STACK("MPI_File_write", MPI_File_write_ID, pnmpi_int_MPI_File_write_t, pnmpi_int_MPI_File_write, mods, MPI_File_write);\
INITIALIZE_FUNCTION_STACK("MPI_File_write_all", MPI_File_write_all_ID, pnmpi_int_MPI_File_write_all_t, pnmpi_int_MPI_File_write_all, mods, MPI_File_write_all);\
INITIALIZE_FUNCTION_STACK("MPI_File_write_all_begin", MPI_File_write_all_begin_ID, pnmpi_int_MPI_File_write_all_begin_t, pnmpi_int_MPI_File_write_all_begin, mods, MPI_File_write_all_begin);\
INITIALIZE_FUNCTION_STACK("MPI_File_write_all_end", MPI_File_write_all_end_ID, pnmpi_int_MPI_File_write_all_end_t, pnmpi_int_MPI_File_write_all_end, mods, MPI_File_write_all_end);\
INITIALIZE_FUNCTION_STACK("MPI_File_write_at", MPI_File_write_at_ID, pnmpi_int_MPI_File_write_at_t, pnmpi_int_MPI_File_write_at, mods, MPI_File_write_at);\
INITIALIZE_FUNCTION_STACK("MPI_File_write_at_all", MPI_File_write_at_all_ID, pnmpi_int_MPI_File_write_at_all_t, pnmpi_int_MPI_File_write_at_all, mods, MPI_File_write_at_all);\
INITIALIZE_FUNCTION_STACK("MPI_File_write_at_all_begin", MPI_File_write_at_all_begin_ID, pnmpi_int_MPI_File_write_at_all_begin_t, pnmpi_int_MPI_File_write_at_all_begin, mods, MPI_File_write_at_all_begin);\
INITIALIZE_FUNCTION_STACK("MPI_File_write_at_all_end", MPI_File_write_at_all_end_ID, pnmpi_int_MPI_File_write_at_all_end_t, pnmpi_int_MPI_File_write_at_all_end, mods, MPI_File_write_at_all_end);\
INITIALIZE_FUNCTION_STACK("MPI_File_write_ordered", MPI_File_write_ordered_ID, pnmpi_int_MPI_File_write_ordered_t, pnmpi_int_MPI_File_write_ordered, mods, MPI_File_write_ordered);\
INITIALIZE_FUNCTION_STACK("MPI_File_write_ordered_begin", MPI_File_write_ordered_begin_ID, pnmpi_int_MPI_File_write_ordered_begin_t, pnmpi_int_MPI_File_write_ordered_begin, mods, MPI_File_write_ordered_begin);\
INITIALIZE_FUNCTION_STACK("MPI_File_write_ordered_end", MPI_File_write_ordered_end_ID, pnmpi_int_MPI_File_write_ordered_end_t, pnmpi_int_MPI_File_write_ordered_end, mods, MPI_File_write_ordered_end);\
INITIALIZE_FUNCTION_STACK("MPI_File_write_shared", MPI_File_write_shared_ID, pnmpi_int_MPI_File_write_shared_t, pnmpi_int_MPI_File_write_shared, mods, MPI_File_write_shared);\
INITIALIZE_FUNCTION_STACK("MPI_Finalize", MPI_Finalize_ID, pnmpi_int_MPI_Finalize_t, pnmpi_int_MPI_Finalize, mods, MPI_Finalize);\
INITIALIZE_FUNCTION_STACK("MPI_Finalized", MPI_Finalized_ID, pnmpi_int_MPI_Finalized_t, pnmpi_int_MPI_Finalized, mods, MPI_Finalized);\
INITIALIZE_FUNCTION_STACK("MPI_Free_mem", MPI_Free_mem_ID, pnmpi_int_MPI_Free_mem_t, pnmpi_int_MPI_Free_mem, mods, MPI_Free_mem);\
INITIALIZE_FUNCTION_STACK("MPI_Gather", MPI_Gather_ID, pnmpi_int_MPI_Gather_t, pnmpi_int_MPI_Gather, mods, MPI_Gather);\
INITIALIZE_FUNCTION_STACK("MPI_Gatherv", MPI_Gatherv_ID, pnmpi_int_MPI_Gatherv_t, pnmpi_int_MPI_Gatherv, mods, MPI_Gatherv);\
INITIALIZE_FUNCTION_STACK("MPI_Get", MPI_Get_ID, pnmpi_int_MPI_Get_t, pnmpi_int_MPI_Get, mods, MPI_Get);\
INITIALIZE_FUNCTION_STACK("MPI_Get_accumulate", MPI_Get_accumulate_ID, pnmpi_int_MPI_Get_accumulate_t, pnmpi_int_MPI_Get_accumulate, mods, MPI_Get_accumulate);\
INITIALIZE_FUNCTION_STACK("MPI_Get_address", MPI_Get_address_ID, pnmpi_int_MPI_Get_address_t, pnmpi_int_MPI_Get_address, mods, MPI_Get_address);\
INITIALIZE_FUNCTION_STACK("MPI_Get_count", MPI_Get_count_ID, pnmpi_int_MPI_Get_count_t, pnmpi_int_MPI_Get_count, mods, MPI_Get_count);\
INITIALIZE_FUNCTION_STACK("MPI_Get_elements", MPI_Get_elements_ID, pnmpi_int_MPI_Get_elements_t, pnmpi_int_MPI_Get_elements, mods, MPI_Get_elements);\
INITIALIZE_FUNCTION_STACK("MPI_Get_elements_x", MPI_Get_elements_x_ID, pnmpi_int_MPI_Get_elements_x_t, pnmpi_int_MPI_Get_elements_x, mods, MPI_Get_elements_x);\
INITIALIZE_FUNCTION_STACK("MPI_Get_library_version", MPI_Get_library_version_ID, pnmpi_int_MPI_Get_library_version_t, pnmpi_int_MPI_Get_library_version, mods, MPI_Get_library_version);\
INITIALIZE_FUNCTION_STACK("MPI_Get_processor_name", MPI_Get_processor_name_ID, pnmpi_int_MPI_Get_processor_name_t, pnmpi_int_MPI_Get_processor_name, mods, MPI_Get_processor_name);\
INITIALIZE_FUNCTION_STACK("MPI_Get_version", MPI_Get_version_ID, pnmpi_int_MPI_Get_version_t, pnmpi_int_MPI_Get_version, mods, MPI_Get_version);\
INITIALIZE_FUNCTION_STACK("MPI_Graph_create", MPI_Graph_create_ID, pnmpi_int_MPI_Graph_create_t, pnmpi_int_MPI_Graph_create, mods, MPI_Graph_create);\
INITIALIZE_FUNCTION_STACK("MPI_Graph_get", MPI_Graph_get_ID, pnmpi_int_MPI_Graph_get_t, pnmpi_int_MPI_Graph_get, mods, MPI_Graph_get);\
INITIALIZE_FUNCTION_STACK("MPI_Graph_map", MPI_Graph_map_ID, pnmpi_int_MPI_Graph_map_t, pnmpi_int_MPI_Graph_map, mods, MPI_Graph_map);\
INITIALIZE_FUNCTION_STACK("MPI_Graph_neighbors", MPI_Graph_neighbors_ID, pnmpi_int_MPI_Graph_neighbors_t, pnmpi_int_MPI_Graph_neighbors, mods, MPI_Graph_neighbors);\
INITIALIZE_FUNCTION_STACK("MPI_Graph_neighbors_count", MPI_Graph_neighbors_count_ID, pnmpi_int_MPI_Graph_neighbors_count_t, pnmpi_int_MPI_Graph_neighbors_count, mods, MPI_Graph_neighbors_count);\
INITIALIZE_FUNCTION_STACK("MPI_Graphdims_get", MPI_Graphdims_get_ID, pnmpi_int_MPI_Graphdims_get_t, pnmpi_int_MPI_Graphdims_get, mods, MPI_Graphdims_get);\
INITIALIZE_FUNCTION_STACK("MPI_Grequest_complete", MPI_Grequest_complete_ID, pnmpi_int_MPI_Grequest_complete_t, pnmpi_int_MPI_Grequest_complete, mods, MPI_Grequest_complete);\
INITIALIZE_FUNCTION_STACK("MPI_Grequest_start", MPI_Grequest_start_ID, pnmpi_int_MPI_Grequest_start_t, pnmpi_int_MPI_Grequest_start, mods, MPI_Grequest_start);\
INITIALIZE_FUNCTION_STACK("MPI_Group_compare", MPI_Group_compare_ID, pnmpi_int_MPI_Group_compare_t, pnmpi_int_MPI_Group_compare, mods, MPI_Group_compare);\
INITIALIZE_FUNCTION_STACK("MPI_Group_difference", MPI_Group_difference_ID, pnmpi_int_MPI_Group_difference_t, pnmpi_int_MPI_Group_difference, mods, MPI_Group_difference);\
INITIALIZE_FUNCTION_STACK("MPI_Group_excl", MPI_Group_excl_ID, pnmpi_int_MPI_Group_excl_t, pnmpi_int_MPI_Group_excl, mods, MPI_Group_excl);\
INITIALIZE_FUNCTION_STACK("MPI_Group_free", MPI_Group_free_ID, pnmpi_int_MPI_Group_free_t, pnmpi_int_MPI_Group_free, mods, MPI_Group_free);\
INITIALIZE_FUNCTION_STACK("MPI_Group_incl", MPI_Group_incl_ID, pnmpi_int_MPI_Group_incl_t, pnmpi_int_MPI_Group_incl, mods, MPI_Group_incl);\
INITIALIZE_FUNCTION_STACK("MPI_Group_intersection", MPI_Group_intersection_ID, pnmpi_int_MPI_Group_intersection_t, pnmpi_int_MPI_Group_intersection, mods, MPI_Group_intersection);\
INITIALIZE_FUNCTION_STACK("MPI_Group_range_excl", MPI_Group_range_excl_ID, pnmpi_int_MPI_Group_range_excl_t, pnmpi_int_MPI_Group_range_excl, mods, MPI_Group_range_excl);\
INITIALIZE_FUNCTION_STACK("MPI_Group_range_incl", MPI_Group_range_incl_ID, pnmpi_int_MPI_Group_range_incl_t, pnmpi_int_MPI_Group_range_incl, mods, MPI_Group_range_incl);\
INITIALIZE_FUNCTION_STACK("MPI_Group_rank", MPI_Group_rank_ID, pnmpi_int_MPI_Group_rank_t, pnmpi_int_MPI_Group_rank, mods, MPI_Group_rank);\
INITIALIZE_FUNCTION_STACK("MPI_Group_size", MPI_Group_size_ID, pnmpi_int_MPI_Group_size_t, pnmpi_int_MPI_Group_size, mods, MPI_Group_size);\
INITIALIZE_FUNCTION_STACK("MPI_Group_translate_ranks", MPI_Group_translate_ranks_ID, pnmpi_int_MPI_Group_translate_ranks_t, pnmpi_int_MPI_Group_translate_ranks, mods, MPI_Group_translate_ranks);\
INITIALIZE_FUNCTION_STACK("MPI_Group_union", MPI_Group_union_ID, pnmpi_int_MPI_Group_union_t, pnmpi_int_MPI_Group_union, mods, MPI_Group_union);\
INITIALIZE_FUNCTION_STACK("MPI_Iallgather", MPI_Iallgather_ID, pnmpi_int_MPI_Iallgather_t, pnmpi_int_MPI_Iallgather, mods, MPI_Iallgather);\
INITIALIZE_FUNCTION_STACK("MPI_Iallgatherv", MPI_Iallgatherv_ID, pnmpi_int_MPI_Iallgatherv_t, pnmpi_int_MPI_Iallgatherv, mods, MPI_Iallgatherv);\
INITIALIZE_FUNCTION_STACK("MPI_Iallreduce", MPI_Iallreduce_ID, pnmpi_int_MPI_Iallreduce_t, pnmpi_int_MPI_Iallreduce, mods, MPI_Iallreduce);\
INITIALIZE_FUNCTION_STACK("MPI_Ialltoall", MPI_Ialltoall_ID, pnmpi_int_MPI_Ialltoall_t, pnmpi_int_MPI_Ialltoall, mods, MPI_Ialltoall);\
INITIALIZE_FUNCTION_STACK("MPI_Ialltoallv", MPI_Ialltoallv_ID, pnmpi_int_MPI_Ialltoallv_t, pnmpi_int_MPI_Ialltoallv, mods, MPI_Ialltoallv);\
INITIALIZE_FUNCTION_STACK("MPI_Ialltoallw", MPI_Ialltoallw_ID, pnmpi_int_MPI_Ialltoallw_t, pnmpi_int_MPI_Ialltoallw, mods, MPI_Ialltoallw);\
INITIALIZE_FUNCTION_STACK("MPI_Ibarrier", MPI_Ibarrier_ID, pnmpi_int_MPI_Ibarrier_t, pnmpi_int_MPI_Ibarrier, mods, MPI_Ibarrier);\
INITIALIZE_FUNCTION_STACK("MPI_Ibcast", MPI_Ibcast_ID, pnmpi_int_MPI_Ibcast_t, pnmpi_int_MPI_Ibcast, mods, MPI_Ibcast);\
INITIALIZE_FUNCTION_STACK("MPI_Ibsend", MPI_Ibsend_ID, pnmpi_int_MPI_Ibsend_t, pnmpi_int_MPI_Ibsend, mods, MPI_Ibsend);\
INITIALIZE_FUNCTION_STACK("MPI_Iexscan", MPI_Iexscan_ID, pnmpi_int_MPI_Iexscan_t, pnmpi_int_MPI_Iexscan, mods, MPI_Iexscan);\
INITIALIZE_FUNCTION_STACK("MPI_Igather", MPI_Igather_ID, pnmpi_int_MPI_Igather_t, pnmpi_int_MPI_Igather, mods, MPI_Igather);\
INITIALIZE_FUNCTION_STACK("MPI_Igatherv", MPI_Igatherv_ID, pnmpi_int_MPI_Igatherv_t, pnmpi_int_MPI_Igatherv, mods, MPI_Igatherv);\
INITIALIZE_FUNCTION_STACK("MPI_Improbe", MPI_Improbe_ID, pnmpi_int_MPI_Improbe_t, pnmpi_int_MPI_Improbe, mods, MPI_Improbe);\
INITIALIZE_FUNCTION_STACK("MPI_Imrecv", MPI_Imrecv_ID, pnmpi_int_MPI_Imrecv_t, pnmpi_int_MPI_Imrecv, mods, MPI_Imrecv);\
INITIALIZE_FUNCTION_STACK("MPI_Ineighbor_allgather", MPI_Ineighbor_allgather_ID, pnmpi_int_MPI_Ineighbor_allgather_t, pnmpi_int_MPI_Ineighbor_allgather, mods, MPI_Ineighbor_allgather);\
INITIALIZE_FUNCTION_STACK("MPI_Ineighbor_allgatherv", MPI_Ineighbor_allgatherv_ID, pnmpi_int_MPI_Ineighbor_allgatherv_t, pnmpi_int_MPI_Ineighbor_allgatherv, mods, MPI_Ineighbor_allgatherv);\
INITIALIZE_FUNCTION_STACK("MPI_Ineighbor_alltoall", MPI_Ineighbor_alltoall_ID, pnmpi_int_MPI_Ineighbor_alltoall_t, pnmpi_int_MPI_Ineighbor_alltoall, mods, MPI_Ineighbor_alltoall);\
INITIALIZE_FUNCTION_STACK("MPI_Ineighbor_alltoallv", MPI_Ineighbor_alltoallv_ID, pnmpi_int_MPI_Ineighbor_alltoallv_t, pnmpi_int_MPI_Ineighbor_alltoallv, mods, MPI_Ineighbor_alltoallv);\
INITIALIZE_FUNCTION_STACK("MPI_Ineighbor_alltoallw", MPI_Ineighbor_alltoallw_ID, pnmpi_int_MPI_Ineighbor_alltoallw_t, pnmpi_int_MPI_Ineighbor_alltoallw, mods, MPI_Ineighbor_alltoallw);\
INITIALIZE_FUNCTION_STACK("MPI_Info_create", MPI_Info_create_ID, pnmpi_int_MPI_Info_create_t, pnmpi_int_MPI_Info_create, mods, MPI_Info_create);\
INITIALIZE_FUNCTION_STACK("MPI_Info_delete", MPI_Info_delete_ID, pnmpi_int_MPI_Info_delete_t, pnmpi_int_MPI_Info_delete, mods, MPI_Info_delete);\
INITIALIZE_FUNCTION_STACK("MPI_Info_dup", MPI_Info_dup_ID, pnmpi_int_MPI_Info_dup_t, pnmpi_int_MPI_Info_dup, mods, MPI_Info_dup);\
INITIALIZE_FUNCTION_STACK("MPI_Info_free", MPI_Info_free_ID, pnmpi_int_MPI_Info_free_t, pnmpi_int_MPI_Info_free, mods, MPI_Info_free);\
INITIALIZE_FUNCTION_STACK("MPI_Info_get", MPI_Info_get_ID, pnmpi_int_MPI_Info_get_t, pnmpi_int_MPI_Info_get, mods, MPI_Info_get);\
INITIALIZE_FUNCTION_STACK("MPI_Info_get_nkeys", MPI_Info_get_nkeys_ID, pnmpi_int_MPI_Info_get_nkeys_t, pnmpi_int_MPI_Info_get_nkeys, mods, MPI_Info_get_nkeys);\
INITIALIZE_FUNCTION_STACK("MPI_Info_get_nthkey", MPI_Info_get_nthkey_ID, pnmpi_int_MPI_Info_get_nthkey_t, pnmpi_int_MPI_Info_get_nthkey, mods, MPI_Info_get_nthkey);\
INITIALIZE_FUNCTION_STACK("MPI_Info_get_valuelen", MPI_Info_get_valuelen_ID, pnmpi_int_MPI_Info_get_valuelen_t, pnmpi_int_MPI_Info_get_valuelen, mods, MPI_Info_get_valuelen);\
INITIALIZE_FUNCTION_STACK("MPI_Info_set", MPI_Info_set_ID, pnmpi_int_MPI_Info_set_t, pnmpi_int_MPI_Info_set, mods, MPI_Info_set);\
INITIALIZE_FUNCTION_STACK("MPI_Init", MPI_Init_ID, pnmpi_int_MPI_Init_t, pnmpi_int_MPI_Init, mods, MPI_Init);\
INITIALIZE_FUNCTION_STACK("MPI_Init_thread", MPI_Init_thread_ID, pnmpi_int_MPI_Init_thread_t, pnmpi_int_MPI_Init_thread, mods, MPI_Init_thread);\
INITIALIZE_FUNCTION_STACK("MPI_Initialized", MPI_Initialized_ID, pnmpi_int_MPI_Initialized_t, pnmpi_int_MPI_Initialized, mods, MPI_Initialized);\
INITIALIZE_FUNCTION_STACK("MPI_Intercomm_create", MPI_Intercomm_create_ID, pnmpi_int_MPI_Intercomm_create_t, pnmpi_int_MPI_Intercomm_create, mods, MPI_Intercomm_create);\
INITIALIZE_FUNCTION_STACK("MPI_Intercomm_merge", MPI_Intercomm_merge_ID, pnmpi_int_MPI_Intercomm_merge_t, pnmpi_int_MPI_Intercomm_merge, mods, MPI_Intercomm_merge);\
INITIALIZE_FUNCTION_STACK("MPI_Iprobe", MPI_Iprobe_ID, pnmpi_int_MPI_Iprobe_t, pnmpi_int_MPI_Iprobe, mods, MPI_Iprobe);\
INITIALIZE_FUNCTION_STACK("MPI_Irecv", MPI_Irecv_ID, pnmpi_int_MPI_Irecv_t, pnmpi_int_MPI_Irecv, mods, MPI_Irecv);\
INITIALIZE_FUNCTION_STACK("MPI_Ireduce", MPI_Ireduce_ID, pnmpi_int_MPI_Ireduce_t, pnmpi_int_MPI_Ireduce, mods, MPI_Ireduce);\
INITIALIZE_FUNCTION_STACK("MPI_Ireduce_scatter", MPI_Ireduce_scatter_ID, pnmpi_int_MPI_Ireduce_scatter_t, pnmpi_int_MPI_Ireduce_scatter, mods, MPI_Ireduce_scatter);\
INITIALIZE_FUNCTION_STACK("MPI_Ireduce_scatter_block", MPI_Ireduce_scatter_block_ID, pnmpi_int_MPI_Ireduce_scatter_block_t, pnmpi_int_MPI_Ireduce_scatter_block, mods, MPI_Ireduce_scatter_block);\
INITIALIZE_FUNCTION_STACK("MPI_Irsend", MPI_Irsend_ID, pnmpi_int_MPI_Irsend_t, pnmpi_int_MPI_Irsend, mods, MPI_Irsend);\
INITIALIZE_FUNCTION_STACK("MPI_Is_thread_main", MPI_Is_thread_main_ID, pnmpi_int_MPI_Is_thread_main_t, pnmpi_int_MPI_Is_thread_main, mods, MPI_Is_thread_main);\
INITIALIZE_FUNCTION_STACK("MPI_Iscan", MPI_Iscan_ID, pnmpi_int_MPI_Iscan_t, pnmpi_int_MPI_Iscan, mods, MPI_Iscan);\
INITIALIZE_FUNCTION_STACK("MPI_Iscatter", MPI_Iscatter_ID, pnmpi_int_MPI_Iscatter_t, pnmpi_int_MPI_Iscatter, mods, MPI_Iscatter);\
INITIALIZE_FUNCTION_STACK("MPI_Iscatterv", MPI_Iscatterv_ID, pnmpi_int_MPI_Iscatterv_t, pnmpi_int_MPI_Iscatterv, mods, MPI_Iscatterv);\
INITIALIZE_FUNCTION_STACK("MPI_Isend", MPI_Isend_ID, pnmpi_int_MPI_Isend_t, pnmpi_int_MPI_Isend, mods, MPI_Isend);\
INITIALIZE_FUNCTION_STACK("MPI_Issend", MPI_Issend_ID, pnmpi_int_MPI_Issend_t, pnmpi_int_MPI_Issend, mods, MPI_Issend);\
INITIALIZE_FUNCTION_STACK("MPI_Keyval_create", MPI_Keyval_create_ID, pnmpi_int_MPI_Keyval_create_t, pnmpi_int_MPI_Keyval_create, mods, MPI_Keyval_create);\
INITIALIZE_FUNCTION_STACK("MPI_Keyval_free", MPI_Keyval_free_ID, pnmpi_int_MPI_Keyval_free_t, pnmpi_int_MPI_Keyval_free, mods, MPI_Keyval_free);\
INITIALIZE_FUNCTION_STACK("MPI_Lookup_name", MPI_Lookup_name_ID, pnmpi_int_MPI_Lookup_name_t, pnmpi_int_MPI_Lookup_name, mods, MPI_Lookup_name);\
INITIALIZE_FUNCTION_STACK("MPI_Mprobe", MPI_Mprobe_ID, pnmpi_int_MPI_Mprobe_t, pnmpi_int_MPI_Mprobe, mods, MPI_Mprobe);\
INITIALIZE_FUNCTION_STACK("MPI_Mrecv", MPI_Mrecv_ID, pnmpi_int_MPI_Mrecv_t, pnmpi_int_MPI_Mrecv, mods, MPI_Mrecv);\
INITIALIZE_FUNCTION_STACK("MPI_Neighbor_allgather", MPI_Neighbor_allgather_ID, pnmpi_int_MPI_Neighbor_allgather_t, pnmpi_int_MPI_Neighbor_allgather, mods, MPI_Neighbor_allgather);\
INITIALIZE_FUNCTION_STACK("MPI_Neighbor_allgatherv", MPI_Neighbor_allgatherv_ID, pnmpi_int_MPI_Neighbor_allgatherv_t, pnmpi_int_MPI_Neighbor_allgatherv, mods, MPI_Neighbor_allgatherv);\
INITIALIZE_FUNCTION_STACK("MPI_Neighbor_alltoall", MPI_Neighbor_alltoall_ID, pnmpi_int_MPI_Neighbor_alltoall_t, pnmpi_int_MPI_Neighbor_alltoall, mods, MPI_Neighbor_alltoall);\
INITIALIZE_FUNCTION_STACK("MPI_Neighbor_alltoallv", MPI_Neighbor_alltoallv_ID, pnmpi_int_MPI_Neighbor_alltoallv_t, pnmpi_int_MPI_Neighbor_alltoallv, mods, MPI_Neighbor_alltoallv);\
INITIALIZE_FUNCTION_STACK("MPI_Neighbor_alltoallw", MPI_Neighbor_alltoallw_ID, pnmpi_int_MPI_Neighbor_alltoallw_t, pnmpi_int_MPI_Neighbor_alltoallw, mods, MPI_Neighbor_alltoallw);\
INITIALIZE_FUNCTION_STACK("MPI_Op_commutative", MPI_Op_commutative_ID, pnmpi_int_MPI_Op_commutative_t, pnmpi_int_MPI_Op_commutative, mods, MPI_Op_commutative);\
INITIALIZE_FUNCTION_STACK("MPI_Op_create", MPI_Op_create_ID, pnmpi_int_MPI_Op_create_t, pnmpi_int_MPI_Op_create, mods, MPI_Op_create);\
INITIALIZE_FUNCTION_STACK("MPI_Op_free", MPI_Op_free_ID, pnmpi_int_MPI_Op_free_t, pnmpi_int_MPI_Op_free, mods, MPI_Op_free);\
INITIALIZE_FUNCTION_STACK("MPI_Open_port", MPI_Open_port_ID, pnmpi_int_MPI_Open_port_t, pnmpi_int_MPI_Open_port, mods, MPI_Open_port);\
INITIALIZE_FUNCTION_STACK("MPI_Pack", MPI_Pack_ID, pnmpi_int_MPI_Pack_t, pnmpi_int_MPI_Pack, mods, MPI_Pack);\
INITIALIZE_FUNCTION_STACK("MPI_Pack_external", MPI_Pack_external_ID, pnmpi_int_MPI_Pack_external_t, pnmpi_int_MPI_Pack_external, mods, MPI_Pack_external);\
INITIALIZE_FUNCTION_STACK("MPI_Pack_external_size", MPI_Pack_external_size_ID, pnmpi_int_MPI_Pack_external_size_t, pnmpi_int_MPI_Pack_external_size, mods, MPI_Pack_external_size);\
INITIALIZE_FUNCTION_STACK("MPI_Pack_size", MPI_Pack_size_ID, pnmpi_int_MPI_Pack_size_t, pnmpi_int_MPI_Pack_size, mods, MPI_Pack_size);\
INITIALIZE_FUNCTION_STACK("MPI_Pcontrol", MPI_Pcontrol_ID, pnmpi_int_MPI_Pcontrol_t, pnmpi_int_MPI_Pcontrol, mods, MPI_Pcontrol);\
INITIALIZE_FUNCTION_STACK("MPI_Probe", MPI_Probe_ID, pnmpi_int_MPI_Probe_t, pnmpi_int_MPI_Probe, mods, MPI_Probe);\
INITIALIZE_FUNCTION_STACK("MPI_Publish_name", MPI_Publish_name_ID, pnmpi_int_MPI_Publish_name_t, pnmpi_int_MPI_Publish_name, mods, MPI_Publish_name);\
INITIALIZE_FUNCTION_STACK("MPI_Put", MPI_Put_ID, pnmpi_int_MPI_Put_t, pnmpi_int_MPI_Put, mods, MPI_Put);\
INITIALIZE_FUNCTION_STACK("MPI_Query_thread", MPI_Query_thread_ID, pnmpi_int_MPI_Query_thread_t, pnmpi_int_MPI_Query_thread, mods, MPI_Query_thread);\
INITIALIZE_FUNCTION_STACK("MPI_Raccumulate", MPI_Raccumulate_ID, pnmpi_int_MPI_Raccumulate_t, pnmpi_int_MPI_Raccumulate, mods, MPI_Raccumulate);\
INITIALIZE_FUNCTION_STACK("MPI_Recv", MPI_Recv_ID, pnmpi_int_MPI_Recv_t, pnmpi_int_MPI_Recv, mods, MPI_Recv);\
INITIALIZE_FUNCTION_STACK("MPI_Recv_init", MPI_Recv_init_ID, pnmpi_int_MPI_Recv_init_t, pnmpi_int_MPI_Recv_init, mods, MPI_Recv_init);\
INITIALIZE_FUNCTION_STACK("MPI_Reduce", MPI_Reduce_ID, pnmpi_int_MPI_Reduce_t, pnmpi_int_MPI_Reduce, mods, MPI_Reduce);\
INITIALIZE_FUNCTION_STACK("MPI_Reduce_local", MPI_Reduce_local_ID, pnmpi_int_MPI_Reduce_local_t, pnmpi_int_MPI_Reduce_local, mods, MPI_Reduce_local);\
INITIALIZE_FUNCTION_STACK("MPI_Reduce_scatter", MPI_Reduce_scatter_ID, pnmpi_int_MPI_Reduce_scatter_t, pnmpi_int_MPI_Reduce_scatter, mods, MPI_Reduce_scatter);\
INITIALIZE_FUNCTION_STACK("MPI_Reduce_scatter_block", MPI_Reduce_scatter_block_ID, pnmpi_int_MPI_Reduce_scatter_block_t, pnmpi_int_MPI_Reduce_scatter_block, mods, MPI_Reduce_scatter_block);\
INITIALIZE_FUNCTION_STACK("MPI_Register_datarep", MPI_Register_datarep_ID, pnmpi_int_MPI_Register_datarep_t, pnmpi_int_MPI_Register_datarep, mods, MPI_Register_datarep);\
INITIALIZE_FUNCTION_STACK("MPI_Request_free", MPI_Request_free_ID, pnmpi_int_MPI_Request_free_t, pnmpi_int_MPI_Request_free, mods, MPI_Request_free);\
INITIALIZE_FUNCTION_STACK("MPI_Request_get_status", MPI_Request_get_status_ID, pnmpi_int_MPI_Request_get_status_t, pnmpi_int_MPI_Request_get_status, mods, MPI_Request_get_status);\
INITIALIZE_FUNCTION_STACK("MPI_Rget", MPI_Rget_ID, pnmpi_int_MPI_Rget_t, pnmpi_int_MPI_Rget, mods, MPI_Rget);\
INITIALIZE_FUNCTION_STACK("MPI_Rget_accumulate", MPI_Rget_accumulate_ID, pnmpi_int_MPI_Rget_accumulate_t, pnmpi_int_MPI_Rget_accumulate, mods, MPI_Rget_accumulate);\
INITIALIZE_FUNCTION_STACK("MPI_Rput", MPI_Rput_ID, pnmpi_int_MPI_Rput_t, pnmpi_int_MPI_Rput, mods, MPI_Rput);\
INITIALIZE_FUNCTION_STACK("MPI_Rsend", MPI_Rsend_ID, pnmpi_int_MPI_Rsend_t, pnmpi_int_MPI_Rsend, mods, MPI_Rsend);\
INITIALIZE_FUNCTION_STACK("MPI_Rsend_init", MPI_Rsend_init_ID, pnmpi_int_MPI_Rsend_init_t, pnmpi_int_MPI_Rsend_init, mods, MPI_Rsend_init);\
INITIALIZE_FUNCTION_STACK("MPI_Scan", MPI_Scan_ID, pnmpi_int_MPI_Scan_t, pnmpi_int_MPI_Scan, mods, MPI_Scan);\
INITIALIZE_FUNCTION_STACK("MPI_Scatter", MPI_Scatter_ID, pnmpi_int_MPI_Scatter_t, pnmpi_int_MPI_Scatter, mods, MPI_Scatter);\
INITIALIZE_FUNCTION_STACK("MPI_Scatterv", MPI_Scatterv_ID, pnmpi_int_MPI_Scatterv_t, pnmpi_int_MPI_Scatterv, mods, MPI_Scatterv);\
INITIALIZE_FUNCTION_STACK("MPI_Send", MPI_Send_ID, pnmpi_int_MPI_Send_t, pnmpi_int_MPI_Send, mods, MPI_Send);\
INITIALIZE_FUNCTION_STACK("MPI_Send_init", MPI_Send_init_ID, pnmpi_int_MPI_Send_init_t, pnmpi_int_MPI_Send_init, mods, MPI_Send_init);\
INITIALIZE_FUNCTION_STACK("MPI_Sendrecv", MPI_Sendrecv_ID, pnmpi_int_MPI_Sendrecv_t, pnmpi_int_MPI_Sendrecv, mods, MPI_Sendrecv);\
INITIALIZE_FUNCTION_STACK("MPI_Sendrecv_replace", MPI_Sendrecv_replace_ID, pnmpi_int_MPI_Sendrecv_replace_t, pnmpi_int_MPI_Sendrecv_replace, mods, MPI_Sendrecv_replace);\
INITIALIZE_FUNCTION_STACK("MPI_Ssend", MPI_Ssend_ID, pnmpi_int_MPI_Ssend_t, pnmpi_int_MPI_Ssend, mods, MPI_Ssend);\
INITIALIZE_FUNCTION_STACK("MPI_Ssend_init", MPI_Ssend_init_ID, pnmpi_int_MPI_Ssend_init_t, pnmpi_int_MPI_Ssend_init, mods, MPI_Ssend_init);\
INITIALIZE_FUNCTION_STACK("MPI_Start", MPI_Start_ID, pnmpi_int_MPI_Start_t, pnmpi_int_MPI_Start, mods, MPI_Start);\
INITIALIZE_FUNCTION_STACK("MPI_Startall", MPI_Startall_ID, pnmpi_int_MPI_Startall_t, pnmpi_int_MPI_Startall, mods, MPI_Startall);\
INITIALIZE_FUNCTION_STACK("MPI_Status_set_cancelled", MPI_Status_set_cancelled_ID, pnmpi_int_MPI_Status_set_cancelled_t, pnmpi_int_MPI_Status_set_cancelled, mods, MPI_Status_set_cancelled);\
INITIALIZE_FUNCTION_STACK("MPI_Status_set_elements", MPI_Status_set_elements_ID, pnmpi_int_MPI_Status_set_elements_t, pnmpi_int_MPI_Status_set_elements, mods, MPI_Status_set_elements);\
INITIALIZE_FUNCTION_STACK("MPI_Status_set_elements_x", MPI_Status_set_elements_x_ID, pnmpi_int_MPI_Status_set_elements_x_t, pnmpi_int_MPI_Status_set_elements_x, mods, MPI_Status_set_elements_x);\
INITIALIZE_FUNCTION_STACK("MPI_Test", MPI_Test_ID, pnmpi_int_MPI_Test_t, pnmpi_int_MPI_Test, mods, MPI_Test);\
INITIALIZE_FUNCTION_STACK("MPI_Test_cancelled", MPI_Test_cancelled_ID, pnmpi_int_MPI_Test_cancelled_t, pnmpi_int_MPI_Test_cancelled, mods, MPI_Test_cancelled);\
INITIALIZE_FUNCTION_STACK("MPI_Testall", MPI_Testall_ID, pnmpi_int_MPI_Testall_t, pnmpi_int_MPI_Testall, mods, MPI_Testall);\
INITIALIZE_FUNCTION_STACK("MPI_Testany", MPI_Testany_ID, pnmpi_int_MPI_Testany_t, pnmpi_int_MPI_Testany, mods, MPI_Testany);\
INITIALIZE_FUNCTION_STACK("MPI_Testsome", MPI_Testsome_ID, pnmpi_int_MPI_Testsome_t, pnmpi_int_MPI_Testsome, mods, MPI_Testsome);\
INITIALIZE_FUNCTION_STACK("MPI_Topo_test", MPI_Topo_test_ID, pnmpi_int_MPI_Topo_test_t, pnmpi_int_MPI_Topo_test, mods, MPI_Topo_test);\
INITIALIZE_FUNCTION_STACK("MPI_Type_commit", MPI_Type_commit_ID, pnmpi_int_MPI_Type_commit_t, pnmpi_int_MPI_Type_commit, mods, MPI_Type_commit);\
INITIALIZE_FUNCTION_STACK("MPI_Type_contiguous", MPI_Type_contiguous_ID, pnmpi_int_MPI_Type_contiguous_t, pnmpi_int_MPI_Type_contiguous, mods, MPI_Type_contiguous);\
INITIALIZE_FUNCTION_STACK("MPI_Type_create_darray", MPI_Type_create_darray_ID, pnmpi_int_MPI_Type_create_darray_t, pnmpi_int_MPI_Type_create_darray, mods, MPI_Type_create_darray);\
INITIALIZE_FUNCTION_STACK("MPI_Type_create_f90_complex", MPI_Type_create_f90_complex_ID, pnmpi_int_MPI_Type_create_f90_complex_t, pnmpi_int_MPI_Type_create_f90_complex, mods, MPI_Type_create_f90_complex);\
INITIALIZE_FUNCTION_STACK("MPI_Type_create_f90_integer", MPI_Type_create_f90_integer_ID, pnmpi_int_MPI_Type_create_f90_integer_t, pnmpi_int_MPI_Type_create_f90_integer, mods, MPI_Type_create_f90_integer);\
INITIALIZE_FUNCTION_STACK("MPI_Type_create_f90_real", MPI_Type_create_f90_real_ID, pnmpi_int_MPI_Type_create_f90_real_t, pnmpi_int_MPI_Type_create_f90_real, mods, MPI_Type_create_f90_real);\
INITIALIZE_FUNCTION_STACK("MPI_Type_create_hindexed", MPI_Type_create_hindexed_ID, pnmpi_int_MPI_Type_create_hindexed_t, pnmpi_int_MPI_Type_create_hindexed, mods, MPI_Type_create_hindexed);\
INITIALIZE_FUNCTION_STACK("MPI_Type_create_hindexed_block", MPI_Type_create_hindexed_block_ID, pnmpi_int_MPI_Type_create_hindexed_block_t, pnmpi_int_MPI_Type_create_hindexed_block, mods, MPI_Type_create_hindexed_block);\
INITIALIZE_FUNCTION_STACK("MPI_Type_create_hvector", MPI_Type_create_hvector_ID, pnmpi_int_MPI_Type_create_hvector_t, pnmpi_int_MPI_Type_create_hvector, mods, MPI_Type_create_hvector);\
INITIALIZE_FUNCTION_STACK("MPI_Type_create_indexed_block", MPI_Type_create_indexed_block_ID, pnmpi_int_MPI_Type_create_indexed_block_t, pnmpi_int_MPI_Type_create_indexed_block, mods, MPI_Type_create_indexed_block);\
INITIALIZE_FUNCTION_STACK("MPI_Type_create_keyval", MPI_Type_create_keyval_ID, pnmpi_int_MPI_Type_create_keyval_t, pnmpi_int_MPI_Type_create_keyval, mods, MPI_Type_create_keyval);\
INITIALIZE_FUNCTION_STACK("MPI_Type_create_resized", MPI_Type_create_resized_ID, pnmpi_int_MPI_Type_create_resized_t, pnmpi_int_MPI_Type_create_resized, mods, MPI_Type_create_resized);\
INITIALIZE_FUNCTION_STACK("MPI_Type_create_struct", MPI_Type_create_struct_ID, pnmpi_int_MPI_Type_create_struct_t, pnmpi_int_MPI_Type_create_struct, mods, MPI_Type_create_struct);\
INITIALIZE_FUNCTION_STACK("MPI_Type_create_subarray", MPI_Type_create_subarray_ID, pnmpi_int_MPI_Type_create_subarray_t, pnmpi_int_MPI_Type_create_subarray, mods, MPI_Type_create_subarray);\
INITIALIZE_FUNCTION_STACK("MPI_Type_delete_attr", MPI_Type_delete_attr_ID, pnmpi_int_MPI_Type_delete_attr_t, pnmpi_int_MPI_Type_delete_attr, mods, MPI_Type_delete_attr);\
INITIALIZE_FUNCTION_STACK("MPI_Type_dup", MPI_Type_dup_ID, pnmpi_int_MPI_Type_dup_t, pnmpi_int_MPI_Type_dup, mods, MPI_Type_dup);\
INITIALIZE_FUNCTION_STACK("MPI_Type_extent", MPI_Type_extent_ID, pnmpi_int_MPI_Type_extent_t, pnmpi_int_MPI_Type_extent, mods, MPI_Type_extent);\
INITIALIZE_FUNCTION_STACK("MPI_Type_free", MPI_Type_free_ID, pnmpi_int_MPI_Type_free_t, pnmpi_int_MPI_Type_free, mods, MPI_Type_free);\
INITIALIZE_FUNCTION_STACK("MPI_Type_free_keyval", MPI_Type_free_keyval_ID, pnmpi_int_MPI_Type_free_keyval_t, pnmpi_int_MPI_Type_free_keyval, mods, MPI_Type_free_keyval);\
INITIALIZE_FUNCTION_STACK("MPI_Type_get_attr", MPI_Type_get_attr_ID, pnmpi_int_MPI_Type_get_attr_t, pnmpi_int_MPI_Type_get_attr, mods, MPI_Type_get_attr);\
INITIALIZE_FUNCTION_STACK("MPI_Type_get_contents", MPI_Type_get_contents_ID, pnmpi_int_MPI_Type_get_contents_t, pnmpi_int_MPI_Type_get_contents, mods, MPI_Type_get_contents);\
INITIALIZE_FUNCTION_STACK("MPI_Type_get_envelope", MPI_Type_get_envelope_ID, pnmpi_int_MPI_Type_get_envelope_t, pnmpi_int_MPI_Type_get_envelope, mods, MPI_Type_get_envelope);\
INITIALIZE_FUNCTION_STACK("MPI_Type_get_extent", MPI_Type_get_extent_ID, pnmpi_int_MPI_Type_get_extent_t, pnmpi_int_MPI_Type_get_extent, mods, MPI_Type_get_extent);\
INITIALIZE_FUNCTION_STACK("MPI_Type_get_extent_x", MPI_Type_get_extent_x_ID, pnmpi_int_MPI_Type_get_extent_x_t, pnmpi_int_MPI_Type_get_extent_x, mods, MPI_Type_get_extent_x);\
INITIALIZE_FUNCTION_STACK("MPI_Type_get_name", MPI_Type_get_name_ID, pnmpi_int_MPI_Type_get_name_t, pnmpi_int_MPI_Type_get_name, mods, MPI_Type_get_name);\
INITIALIZE_FUNCTION_STACK("MPI_Type_get_true_extent", MPI_Type_get_true_extent_ID, pnmpi_int_MPI_Type_get_true_extent_t, pnmpi_int_MPI_Type_get_true_extent, mods, MPI_Type_get_true_extent);\
INITIALIZE_FUNCTION_STACK("MPI_Type_get_true_extent_x", MPI_Type_get_true_extent_x_ID, pnmpi_int_MPI_Type_get_true_extent_x_t, pnmpi_int_MPI_Type_get_true_extent_x, mods, MPI_Type_get_true_extent_x);\
INITIALIZE_FUNCTION_STACK("MPI_Type_hindexed", MPI_Type_hindexed_ID, pnmpi_int_MPI_Type_hindexed_t, pnmpi_int_MPI_Type_hindexed, mods, MPI_Type_hindexed);\
INITIALIZE_FUNCTION_STACK("MPI_Type_hvector", MPI_Type_hvector_ID, pnmpi_int_MPI_Type_hvector_t, pnmpi_int_MPI_Type_hvector, mods, MPI_Type_hvector);\
INITIALIZE_FUNCTION_STACK("MPI_Type_indexed", MPI_Type_indexed_ID, pnmpi_int_MPI_Type_indexed_t, pnmpi_int_MPI_Type_indexed, mods, MPI_Type_indexed);\
INITIALIZE_FUNCTION_STACK("MPI_Type_lb", MPI_Type_lb_ID, pnmpi_int_MPI_Type_lb_t, pnmpi_int_MPI_Type_lb, mods, MPI_Type_lb);\
INITIALIZE_FUNCTION_STACK("MPI_Type_match_size", MPI_Type_match_size_ID, pnmpi_int_MPI_Type_match_size_t, pnmpi_int_MPI_Type_match_size, mods, MPI_Type_match_size);\
INITIALIZE_FUNCTION_STACK("MPI_Type_set_attr", MPI_Type_set_attr_ID, pnmpi_int_MPI_Type_set_attr_t, pnmpi_int_MPI_Type_set_attr, mods, MPI_Type_set_attr);\
INITIALIZE_FUNCTION_STACK("MPI_Type_set_name", MPI_Type_set_name_ID, pnmpi_int_MPI_Type_set_name_t, pnmpi_int_MPI_Type_set_name, mods, MPI_Type_set_name);\
INITIALIZE_FUNCTION_STACK("MPI_Type_size", MPI_Type_size_ID, pnmpi_int_MPI_Type_size_t, pnmpi_int_MPI_Type_size, mods, MPI_Type_size);\
INITIALIZE_FUNCTION_STACK("MPI_Type_size_x", MPI_Type_size_x_ID, pnmpi_int_MPI_Type_size_x_t, pnmpi_int_MPI_Type_size_x, mods, MPI_Type_size_x);\
INITIALIZE_FUNCTION_STACK("MPI_Type_struct", MPI_Type_struct_ID, pnmpi_int_MPI_Type_struct_t, pnmpi_int_MPI_Type_struct, mods, MPI_Type_struct);\
INITIALIZE_FUNCTION_STACK("MPI_Type_ub", MPI_Type_ub_ID, pnmpi_int_MPI_Type_ub_t, pnmpi_int_MPI_Type_ub, mods, MPI_Type_ub);\
INITIALIZE_FUNCTION_STACK("MPI_Type_vector", MPI_Type_vector_ID, pnmpi_int_MPI_Type_vector_t, pnmpi_int_MPI_Type_vector, mods, MPI_Type_vector);\
INITIALIZE_FUNCTION_STACK("MPI_Unpack", MPI_Unpack_ID, pnmpi_int_MPI_Unpack_t, pnmpi_int_MPI_Unpack, mods, MPI_Unpack);\
INITIALIZE_FUNCTION_STACK("MPI_Unpack_external", MPI_Unpack_external_ID, pnmpi_int_MPI_Unpack_external_t, pnmpi_int_MPI_Unpack_external, mods, MPI_Unpack_external);\
INITIALIZE_FUNCTION_STACK("MPI_Unpublish_name", MPI_Unpublish_name_ID, pnmpi_int_MPI_Unpublish_name_t, pnmpi_int_MPI_Unpublish_name, mods, MPI_Unpublish_name);\
INITIALIZE_FUNCTION_STACK("MPI_Wait", MPI_Wait_ID, pnmpi_int_MPI_Wait_t, pnmpi_int_MPI_Wait, mods, MPI_Wait);\
INITIALIZE_FUNCTION_STACK("MPI_Waitall", MPI_Waitall_ID, pnmpi_int_MPI_Waitall_t, pnmpi_int_MPI_Waitall, mods, MPI_Waitall);\
INITIALIZE_FUNCTION_STACK("MPI_Waitany", MPI_Waitany_ID, pnmpi_int_MPI_Waitany_t, pnmpi_int_MPI_Waitany, mods, MPI_Waitany);\
INITIALIZE_FUNCTION_STACK("MPI_Waitsome", MPI_Waitsome_ID, pnmpi_int_MPI_Waitsome_t, pnmpi_int_MPI_Waitsome, mods, MPI_Waitsome);\
INITIALIZE_FUNCTION_STACK("MPI_Win_allocate", MPI_Win_allocate_ID, pnmpi_int_MPI_Win_allocate_t, pnmpi_int_MPI_Win_allocate, mods, MPI_Win_allocate);\
INITIALIZE_FUNCTION_STACK("MPI_Win_allocate_shared", MPI_Win_allocate_shared_ID, pnmpi_int_MPI_Win_allocate_shared_t, pnmpi_int_MPI_Win_allocate_shared, mods, MPI_Win_allocate_shared);\
INITIALIZE_FUNCTION_STACK("MPI_Win_attach", MPI_Win_attach_ID, pnmpi_int_MPI_Win_attach_t, pnmpi_int_MPI_Win_attach, mods, MPI_Win_attach);\
INITIALIZE_FUNCTION_STACK("MPI_Win_call_errhandler", MPI_Win_call_errhandler_ID, pnmpi_int_MPI_Win_call_errhandler_t, pnmpi_int_MPI_Win_call_errhandler, mods, MPI_Win_call_errhandler);\
INITIALIZE_FUNCTION_STACK("MPI_Win_complete", MPI_Win_complete_ID, pnmpi_int_MPI_Win_complete_t, pnmpi_int_MPI_Win_complete, mods, MPI_Win_complete);\
INITIALIZE_FUNCTION_STACK("MPI_Win_create", MPI_Win_create_ID, pnmpi_int_MPI_Win_create_t, pnmpi_int_MPI_Win_create, mods, MPI_Win_create);\
INITIALIZE_FUNCTION_STACK("MPI_Win_create_dynamic", MPI_Win_create_dynamic_ID, pnmpi_int_MPI_Win_create_dynamic_t, pnmpi_int_MPI_Win_create_dynamic, mods, MPI_Win_create_dynamic);\
INITIALIZE_FUNCTION_STACK("MPI_Win_create_errhandler", MPI_Win_create_errhandler_ID, pnmpi_int_MPI_Win_create_errhandler_t, pnmpi_int_MPI_Win_create_errhandler, mods, MPI_Win_create_errhandler);\
INITIALIZE_FUNCTION_STACK("MPI_Win_create_keyval", MPI_Win_create_keyval_ID, pnmpi_int_MPI_Win_create_keyval_t, pnmpi_int_MPI_Win_create_keyval, mods, MPI_Win_create_keyval);\
INITIALIZE_FUNCTION_STACK("MPI_Win_delete_attr", MPI_Win_delete_attr_ID, pnmpi_int_MPI_Win_delete_attr_t, pnmpi_int_MPI_Win_delete_attr, mods, MPI_Win_delete_attr);\
INITIALIZE_FUNCTION_STACK("MPI_Win_detach", MPI_Win_detach_ID, pnmpi_int_MPI_Win_detach_t, pnmpi_int_MPI_Win_detach, mods, MPI_Win_detach);\
INITIALIZE_FUNCTION_STACK("MPI_Win_fence", MPI_Win_fence_ID, pnmpi_int_MPI_Win_fence_t, pnmpi_int_MPI_Win_fence, mods, MPI_Win_fence);\
INITIALIZE_FUNCTION_STACK("MPI_Win_flush", MPI_Win_flush_ID, pnmpi_int_MPI_Win_flush_t, pnmpi_int_MPI_Win_flush, mods, MPI_Win_flush);\
INITIALIZE_FUNCTION_STACK("MPI_Win_flush_all", MPI_Win_flush_all_ID, pnmpi_int_MPI_Win_flush_all_t, pnmpi_int_MPI_Win_flush_all, mods, MPI_Win_flush_all);\
INITIALIZE_FUNCTION_STACK("MPI_Win_flush_local", MPI_Win_flush_local_ID, pnmpi_int_MPI_Win_flush_local_t, pnmpi_int_MPI_Win_flush_local, mods, MPI_Win_flush_local);\
INITIALIZE_FUNCTION_STACK("MPI_Win_flush_local_all", MPI_Win_flush_local_all_ID, pnmpi_int_MPI_Win_flush_local_all_t, pnmpi_int_MPI_Win_flush_local_all, mods, MPI_Win_flush_local_all);\
INITIALIZE_FUNCTION_STACK("MPI_Win_free", MPI_Win_free_ID, pnmpi_int_MPI_Win_free_t, pnmpi_int_MPI_Win_free, mods, MPI_Win_free);\
INITIALIZE_FUNCTION_STACK("MPI_Win_free_keyval", MPI_Win_free_keyval_ID, pnmpi_int_MPI_Win_free_keyval_t, pnmpi_int_MPI_Win_free_keyval, mods, MPI_Win_free_keyval);\
INITIALIZE_FUNCTION_STACK("MPI_Win_get_attr", MPI_Win_get_attr_ID, pnmpi_int_MPI_Win_get_attr_t, pnmpi_int_MPI_Win_get_attr, mods, MPI_Win_get_attr);\
INITIALIZE_FUNCTION_STACK("MPI_Win_get_errhandler", MPI_Win_get_errhandler_ID, pnmpi_int_MPI_Win_get_errhandler_t, pnmpi_int_MPI_Win_get_errhandler, mods, MPI_Win_get_errhandler);\
INITIALIZE_FUNCTION_STACK("MPI_Win_get_group", MPI_Win_get_group_ID, pnmpi_int_MPI_Win_get_group_t, pnmpi_int_MPI_Win_get_group, mods, MPI_Win_get_group);\
INITIALIZE_FUNCTION_STACK("MPI_Win_get_info", MPI_Win_get_info_ID, pnmpi_int_MPI_Win_get_info_t, pnmpi_int_MPI_Win_get_info, mods, MPI_Win_get_info);\
INITIALIZE_FUNCTION_STACK("MPI_Win_get_name", MPI_Win_get_name_ID, pnmpi_int_MPI_Win_get_name_t, pnmpi_int_MPI_Win_get_name, mods, MPI_Win_get_name);\
INITIALIZE_FUNCTION_STACK("MPI_Win_lock", MPI_Win_lock_ID, pnmpi_int_MPI_Win_lock_t, pnmpi_int_MPI_Win_lock, mods, MPI_Win_lock);\
INITIALIZE_FUNCTION_STACK("MPI_Win_lock_all", MPI_Win_lock_all_ID, pnmpi_int_MPI_Win_lock_all_t, pnmpi_int_MPI_Win_lock_all, mods, MPI_Win_lock_all);\
INITIALIZE_FUNCTION_STACK("MPI_Win_post", MPI_Win_post_ID, pnmpi_int_MPI_Win_post_t, pnmpi_int_MPI_Win_post, mods, MPI_Win_post);\
INITIALIZE_FUNCTION_STACK("MPI_Win_set_attr", MPI_Win_set_attr_ID, pnmpi_int_MPI_Win_set_attr_t, pnmpi_int_MPI_Win_set_attr, mods, MPI_Win_set_attr);\
INITIALIZE_FUNCTION_STACK("MPI_Win_set_errhandler", MPI_Win_set_errhandler_ID, pnmpi_int_MPI_Win_set_errhandler_t, pnmpi_int_MPI_Win_set_errhandler, mods, MPI_Win_set_errhandler);\
INITIALIZE_FUNCTION_STACK("MPI_Win_set_info", MPI_Win_set_info_ID, pnmpi_int_MPI_Win_set_info_t, pnmpi_int_MPI_Win_set_info, mods, MPI_Win_set_info);\
INITIALIZE_FUNCTION_STACK("MPI_Win_set_name", MPI_Win_set_name_ID, pnmpi_int_MPI_Win_set_name_t, pnmpi_int_MPI_Win_set_name, mods, MPI_Win_set_name);\
INITIALIZE_FUNCTION_STACK("MPI_Win_shared_query", MPI_Win_shared_query_ID, pnmpi_int_MPI_Win_shared_query_t, pnmpi_int_MPI_Win_shared_query, mods, MPI_Win_shared_query);\
INITIALIZE_FUNCTION_STACK("MPI_Win_start", MPI_Win_start_ID, pnmpi_int_MPI_Win_start_t, pnmpi_int_MPI_Win_start, mods, MPI_Win_start);\
INITIALIZE_FUNCTION_STACK("MPI_Win_sync", MPI_Win_sync_ID, pnmpi_int_MPI_Win_sync_t, pnmpi_int_MPI_Win_sync, mods, MPI_Win_sync);\
INITIALIZE_FUNCTION_STACK("MPI_Win_test", MPI_Win_test_ID, pnmpi_int_MPI_Win_test_t, pnmpi_int_MPI_Win_test, mods, MPI_Win_test);\
INITIALIZE_FUNCTION_STACK("MPI_Win_unlock", MPI_Win_unlock_ID, pnmpi_int_MPI_Win_unlock_t, pnmpi_int_MPI_Win_unlock, mods, MPI_Win_unlock);\
INITIALIZE_FUNCTION_STACK("MPI_Win_unlock_all", MPI_Win_unlock_all_ID, pnmpi_int_MPI_Win_unlock_all_t, pnmpi_int_MPI_Win_unlock_all, mods, MPI_Win_unlock_all);\
INITIALIZE_FUNCTION_STACK("MPI_Win_wait", MPI_Win_wait_ID, pnmpi_int_MPI_Win_wait_t, pnmpi_int_MPI_Win_wait, mods, MPI_Win_wait);\
INITIALIZE_FUNCTION_STACK("MPI_Wtick", MPI_Wtick_ID, pnmpi_int_MPI_Wtick_t, pnmpi_int_MPI_Wtick, mods, MPI_Wtick);\
INITIALIZE_FUNCTION_STACK("MPI_Wtime", MPI_Wtime_ID, pnmpi_int_MPI_Wtime_t, pnmpi_int_MPI_Wtime, mods, MPI_Wtime);\

